package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.service.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 租户Controller
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
@RestController
@RequestMapping("/system/tenant")
public class SysTenantController extends BaseController
{
    @Autowired
    private ISysTenantService sysTenantService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysPostService postService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ISysMenuService menuService;



    /**
     * 查询租户列表
     */
    @PreAuthorize("@ss.hasPermi('system:tenant:list')")
    @GetMapping("/list")
    public TableDataInfo list(SysTenant sysTenant)
    {
        startPage();
        List<SysTenant> list = sysTenantService.selectSysTenantList(sysTenant);
        return getDataTable(list);
    }

    /**
     * 查询租户列表（不分页）
     */
    // @PreAuthorize("@ss.hasPermi('system:tenant:listNoPage')")
    @GetMapping("/listNoPage")
    public TableDataInfo listNoPage(SysTenant sysTenant)
    {
        List<SysTenant> list = sysTenantService.selectSysTenantNoPageList(sysTenant);
        return getDataTable(list);
    }

    /**
     * 导出租户列表
     */
    @PreAuthorize("@ss.hasPermi('system:tenant:export')")
    @Log(title = "租户", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysTenant sysTenant)
    {
        List<SysTenant> list = sysTenantService.selectSysTenantList(sysTenant);
        ExcelUtil<SysTenant> util = new ExcelUtil<SysTenant>(SysTenant.class);
        util.exportExcel(response, list, "租户数据");
    }

    /**
     * 获取租户详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:tenant:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(sysTenantService.selectSysTenantById(id));
    }

    /**
     * 新增租户
     */
    @PreAuthorize("@ss.hasPermi('system:tenant:add')")
    @Log(title = "租户", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SysTenant sysTenant) {
        return toAjax(sysTenantService.insertSysTenant(sysTenant));
    }

    /**
     * 新增租户
     */
//    @PreAuthorize("@ss.hasPermi('system:tenant:add')")
//    @Log(title = "租户", businessType = BusinessType.INSERT)
//    @PostMapping
//    public AjaxResult addBak(@RequestBody SysTenant sysTenant) {
//        if (sysTenantService.insertSysTenant(sysTenant) > 0) {
//            SysDept dept = new SysDept();
//            dept.setDeptName(sysTenant.getTenantName());
//            dept.setOrderNum(1);
//            dept.setLeader("1");
//            dept.setParentId(100L);
//            dept.setPhone(sysTenant.getContactMobile());
//            dept.setStatus("0");
//            dept.setRemark("租户创建自动生成部门");
//            dept.setTenantId(sysTenant.getId());
//            if (!deptService.checkDeptNameUnique(dept)) {
//                SysDept deptQuery = new SysDept();
//                deptQuery.setDeptName(sysTenant.getTenantName());
//                List<SysDept> deptList = deptService.selectDeptList(deptQuery);
//                dept = deptList.get(0);
//            } else {
//                dept.setCreateBy(getUsername());
//                deptService.insertDept(dept);
//            }
//            System.err.println("deptId" + dept.getDeptId());
//            if (dept != null && dept.getDeptId() != null) {
//                //生成职位 默认给当前用户
//                SysPost post = new SysPost();
//                post.setPostName(sysTenant.getTenantName() + "管理员岗位");
//                post.setPostCode(sysTenant.getTenantAccount());
//                post.setStatus("0");
//                post.setRemark("租户创建自动生成岗位");
//                post.setTenantId(sysTenant.getId());
//                if (!postService.checkPostNameUnique(post)) {
//                    SysPost querySysPost = new SysPost();
//                    querySysPost.setPostName(sysTenant.getTenantName() + "管理员岗位");
//                    post = postService.selectPostByName(querySysPost);
//                } else if (!postService.checkPostCodeUnique(post)) {
//                    SysPost querySysPost = new SysPost();
//                    post.setPostCode(sysTenant.getTenantAccount());
//                    post = postService.selectPostByCode(querySysPost);
//                } else {
//                    post.setCreateBy(getUsername());
//                    postService.insertPost(post);
//                }
//                //批量新增菜单表
//                SysMenu menu = new SysMenu();
//
//                //menuService.batchInsertMenu(menu);
//
//                //生成角色 默认给当前用户
//                SysRole role = new SysRole();
//                role.setRoleName(sysTenant.getTenantName() + "管理员角色");
//                role.setRoleKey(sysTenant.getTenantAccount());
//                role.setDataScope("0");
//                role.setStatus("0");
//                role.setRemark("租户创建自动生成角色");
//                role.setTenantId(sysTenant.getId());
//
//                //role.setMenuIds(); //新增角色和菜单关系
//                if (!roleService.checkRoleNameUnique(role)) {
//                    SysRole roleQuery = new SysRole();
//                    roleQuery.setRoleName(sysTenant.getTenantName() + "管理员角色");
//                    role = roleService.selectRoleByRoleName(roleQuery);
//                } else if (!roleService.checkRoleKeyUnique(role)) {
//                    SysRole roleQuery = new SysRole();
//                    roleQuery.setRoleKey(sysTenant.getTenantAccount());
//                    role = roleService.selectRoleByRoleKey(roleQuery);
//                } else {
//                    role.setCreateBy(getUsername());
//                    roleService.insertRole(role);
//                }
//
//                SysUser user = new SysUser();
//                user.setDeptId(dept.getDeptId());
//                user.setUserName(sysTenant.getTenantAccount());
//                user.setNickName(sysTenant.getTenantAccount());
//                user.setEmail("1@qq.com");
//                user.setPhonenumber(sysTenant.getContactMobile());
//                user.setPassword("123456");
//                user.setStatus("0");
//                user.setPostIds(new Long[]{post.getPostId()});
//                user.setRoleIds(new Long[]{role.getRoleId()});
//                user.setRemark("租户新建自动创建的用户");
//                user.setIsAdmin("0");//租户创建用户默认为超级管理员
//                if (!userService.checkUserNameUnique(user)) {
//                    return error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
//                }
//                user.setTenantId(sysTenant.getId());
//                user.setCreateBy(getUsername());
//                user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
//                return toAjax(userService.insertUser(user));
//            } else {
//                return error("新增部门'" + dept.getDeptName() + "'失败");
//            }
//        } else {
//            return error("新增租户'" + sysTenant.getTenantName() + "'失败");
//        }
//    }

    /**
     * 修改租户
     */
    @PreAuthorize("@ss.hasPermi('system:tenant:edit')")
    @Log(title = "租户", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SysTenant sysTenant)
    {
        return toAjax(sysTenantService.updateSysTenant(sysTenant));
    }

    /**
     * 删除租户
     */
    @PreAuthorize("@ss.hasPermi('system:tenant:remove')")
    @Log(title = "租户", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(sysTenantService.deleteSysTenantByIds(ids));
    }
}
