package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.psi.domain.ApprovalRecording;
import com.ruoyi.psi.domain.vo.ApprovalRecordingVO;
import com.ruoyi.psi.service.IApprovalRecordingService;
import com.ruoyi.psi.service.IPsiSupplierService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * 审批记录Controller
 * 
 * @author ruoyi
 * @date 2024-01-19
 */
@RestController
@RequestMapping("/psi/recording")
public class ApprovalRecordingController extends BaseController
{
    @Autowired
    private IApprovalRecordingService approvalRecordingService;

    @Autowired
    private IPsiSupplierService supplierService;

    /**
     * 查询代办列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:recording:list')")
    @GetMapping("/list")
    public TableDataInfo list(ApprovalRecording approvalRecording)
    {
        SysUser user = this.getLoginUser().getUser();
        approvalRecording.setApprovalUser(user.getUserId());
        approvalRecording.setApprovarType(0);
        startPage();
        List<ApprovalRecordingVO> list = approvalRecordingService.selectApprovalRecordingList(approvalRecording);
        return getDataTable(list);
    }

    /**
     * 查询已经办理列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:recording:list')")
    @GetMapping("/doneList")
    public TableDataInfo doneList(ApprovalRecording approvalRecording)
    {
        SysUser user = this.getLoginUser().getUser();
        approvalRecording.setApprovalUser(user.getUserId());
        approvalRecording.setApprovarType(1);
        startPage();
        List<ApprovalRecordingVO> list = approvalRecordingService.selectApprovalDoneRecordingList(approvalRecording);
        return getDataTable(list);

    }


    /**
     * 导出审批记录列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:recording:export')")
    // @Log(title = "审批记录", businessType = BusinessType.EXPORT)
    // @PostMapping("/export")
    // public void export(HttpServletResponse response, ApprovalRecording approvalRecording)
    // {
    //     List<ApprovalRecording> list = approvalRecordingService.selectApprovalRecordingList(approvalRecording);
    //     ExcelUtil<ApprovalRecording> util = new ExcelUtil<ApprovalRecording>(ApprovalRecording.class);
    //     util.exportExcel(response, list, "审批记录数据");
    // }

    /**
     * 获取审批记录详细信息 bak
     */
    // @PreAuthorize("@ss.hasPermi('psi:recording:query')")
    // @GetMapping(value = "/getRecordingInfo/{documentId}/{documentType}")
    // public AjaxResult getInfo(@PathVariable("documentId") Long documentId, @PathVariable("documentType") Integer documentType)
    // {
    //     SysUser user = this.getLoginUser().getUser();
    //     String userType = user.getUserType();
    //     ApprovalRecording approvalRecording = new ApprovalRecording();
    //     // approvalRecording.setTenantId(tenantId);
    //     approvalRecording.setDocumentId(documentId);
    //     approvalRecording.setDocumentType(documentType);
    //     Map<String, Object> params = approvalRecording.getParams();
    //     if("02".equals(userType)){
    //         List<SupplierTenantVO> tenantByPhone = supplierService.getTenantByPhone(user.getPhonenumber());
    //         params.put("ids",tenantByPhone.stream().map(SupplierTenantVO::getTenantId).collect(Collectors.toList()));
    //     }else if("01".equals(userType)){
    //         // approvalRecording.setTenantId(user.getTenantId());
    //         approvalRecording.setzTenantId(user.getTenantId());
    //     }
    //     AjaxResult ajaxResult = AjaxResult.success(approvalRecordingService.selectApprovalRecordingById(approvalRecording));
    //     ApprovalRecording approvalRecordingQuery = approvalRecordingService.selectApprovalRecordingNoApproval(approvalRecording);
    //     if(Objects.isNull(approvalRecordingQuery)){
    //         ajaxResult.put("currentNode","over");
    //     }else {
    //         ajaxResult.put("currentNode",approvalRecordingQuery);
    //     }
    //     return ajaxResult;
    // }

    /**
     * 获取审批记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('psi:recording:query')")
    @GetMapping(value = "/getRecordingInfo/{documentId}/{documentType}")
    public AjaxResult getInfo(@PathVariable("documentId") Long documentId, @PathVariable("documentType") Integer documentType)
    {
        ApprovalRecording approvalRecording = new ApprovalRecording();
        approvalRecording.setDocumentId(documentId);
        approvalRecording.setDocumentType(documentType);
        AjaxResult ajaxResult = AjaxResult.success(approvalRecordingService.selectApprovalRecordingById(approvalRecording));
        ApprovalRecording approvalRecordingQuery = approvalRecordingService.selectApprovalRecordingNoApproval(approvalRecording);
        if(Objects.isNull(approvalRecordingQuery)){
            ajaxResult.put("currentNode","over");
        }else {
            ajaxResult.put("currentNode",approvalRecordingQuery);
        }
        return ajaxResult;
    }

    /**
     * 新增审批记录
     */
    @PreAuthorize("@ss.hasPermi('psi:recording:add')")
    @Log(title = "审批记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody List<ApprovalRecording> approvalRecordings)
    {
        return toAjax(approvalRecordingService.insertApprovalRecording(approvalRecordings));
    }

    /**
     * 修改审批记录
     */
    // @PreAuthorize("@ss.hasPermi('psi:recording:edit')")
    @Log(title = "审批记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody List<ApprovalRecording> approvalRecordings)
    {
        Long tenantId = this.getTenantId();
        // approvalRecording.setTenantId(tenantId);
        return toAjax(approvalRecordingService.updateApprovalRecording(approvalRecordings,tenantId));
    }

    /**
     * 删除审批记录
     */
    @PreAuthorize("@ss.hasPermi('psi:recording:remove')")
    @Log(title = "审批记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(approvalRecordingService.deleteApprovalRecordingByIds(ids));
    }

    /**
     * 查看单据详情(租户)
     * @return
     */
    @PreAuthorize("@ss.hasPermi('psi:pRecording:query')")
    @GetMapping(value = "/psiGetRecordingInfo/{documentId}/{documentType}")
    public AjaxResult psiGetRecordingInfo(@PathVariable("documentId") Long documentId,
                              @PathVariable("documentType") Integer documentType){
        return approvalRecordingService.psiGetRecordingInfo(documentId,documentType);
    }
}
