package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.psi.domain.Bidding;
import com.ruoyi.psi.domain.vo.AddBiddingVo;
import com.ruoyi.psi.service.IBiddingService;
import com.ruoyi.sup.service.IPsiBiddingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 竞价单Controller
 * 
 * @author ruoyi
 * @date 2023-12-17
 */
@RestController
@RequestMapping("/psi/bidding")
public class BiddingController extends BaseController
{
    @Autowired
    private IBiddingService BiddingService;

    /**
     *  根据采购单获取可以报价的供应商列表
     * @param purchaseId
     * @return
     */
    // @Log(title = "竞价单", businessType = BusinessType.OTHER)
    // @PreAuthorize("@ss.hasPermi('psi:bidding:getBiddingSupplier')")
    @GetMapping("/getBiddingSupplier/{purchaseId}")
    public AjaxResult getBiddingSupplier(@PathVariable Long purchaseId)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(BiddingService.getBiddingSupplier(purchaseId,tenantId));
    }

    /**
     *  传入选择的供应商，生产报价单
     * @param vo
     * @return
     */
    // @Log(title = "竞价单", businessType = BusinessType.INSERT)
    // @PreAuthorize("@ss.hasPermi('psi:bidding:getBiddingSupplier')")
    @PostMapping("/addBidding")
    public AjaxResult addBidding(@RequestBody AddBiddingVo vo)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        vo.setTenantId(tenantId);
        return success(BiddingService.addBidding(vo));
    }

}
