package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.psi.domain.PsiGoods;
import com.ruoyi.psi.domain.dto.ImportRequestDTO;
import com.ruoyi.psi.domain.dto.PsiGoodsDTO;
import com.ruoyi.psi.service.IPsiGoodsService;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 商品Controller
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
@RestController
@RequestMapping("/psi/goods")
public class PsiGoodsController extends BaseController
{
    @Autowired
    private IPsiGoodsService psiGoodsService;

    /**
     * 查询商品列表（采购单、手动录入收货单调用）
     */
    @PreAuthorize("@ss.hasPermi('psi:goods:list')")
    @GetMapping("/list")
    public TableDataInfo list(PsiGoods psiGoods)
    {
        // startPage();
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        List<PsiGoods> list = psiGoodsService.selectPsiGoodsList(psiGoods);
        return getDataTable(list);
    }

    /**
     * 导出商品列表
     */
    @PreAuthorize("@ss.hasPermi('psi:goods:export')")
    @Log(title = "商品", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiGoods psiGoods)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        List<PsiGoods> list = psiGoodsService.selectExportPsiGoodsList(psiGoods);
        ExcelUtil<PsiGoods> util = new ExcelUtil<PsiGoods>(PsiGoods.class);
        util.exportExcel(response, list, "商品数据");
    }

    /**
     * 获取商品详细信息
     */
    @PreAuthorize("@ss.hasPermi('psi:goods:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiGoodsService.selectPsiGoodsById(id,tenantId));
    }

    /**
     * 新增商品
     */
    @PreAuthorize("@ss.hasPermi('psi:goods:add')")
    @Log(title = "商品", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PsiGoodsDTO psiGoods)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        return toAjax(psiGoodsService.insertPsiGoods(psiGoods));
    }

    /**
     * 修改商品
     */
    @PreAuthorize("@ss.hasPermi('psi:goods:edit')")
    @Log(title = "商品", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PsiGoodsDTO psiGoods)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        return toAjax(psiGoodsService.updatePsiGoods(psiGoods));
    }

    /**
     * 删除商品
     */
    @PreAuthorize("@ss.hasPermi('psi:goods:remove')")
    @Log(title = "商品", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return toAjax(psiGoodsService.deletePsiGoodsByIds(ids,tenantId));
    }


    /**
     * 商品导入模板
     * @param response
     */
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<PsiGoods> util = new ExcelUtil<PsiGoods>(PsiGoods.class);
        util.importTemplateExcel(response, "商品数据");
    }


    /**
     * 商品导入
     * *
     * @param requestDTO
     * @return
     * @throws Exception
     */
    @Log(title = "商品管理", businessType = BusinessType.IMPORT)
    @PreAuthorize("@ss.hasPermi('system:user:import')")
    @PostMapping("/importData/{supplierId}")
    public AjaxResult importData(ImportRequestDTO requestDTO) throws Exception
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        MultipartFile file = requestDTO.getFile();

        ExcelUtil<PsiGoods> util = new ExcelUtil<PsiGoods>(PsiGoods.class);
        List<PsiGoods> psiGoodsList = util.importExcel(file.getInputStream());
        String message = psiGoodsService.importPsiGoods(psiGoodsList,tenantId,requestDTO);
        return success(message);
    }

    /**
     * 查询商品列表（不分页）
     */
//    @PreAuthorize("@ss.hasPermi('psi:goods:list')")
    @GetMapping("/listAll")
    public AjaxResult listAll(PsiGoods psiGoods)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        return success(psiGoodsService.listAll(psiGoods));
    }


    /**
     * 查询商品列表 商品首页调用
     */
//    @PreAuthorize("@ss.hasPermi('psi:goods:goodsList')")
    @GetMapping("/goodsList")
    public TableDataInfo goodsList(PsiGoods psiGoods)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        List<PsiGoods> goods = psiGoodsService.goodsList(psiGoods);
        return getDataTable(goods);
    }

    /**
     * 撤回商品
     * TODO 看是否要撤回的时候删除依据文件以及依据备注
     */
    // @PreAuthorize("@ss.hasPermi('psi:goods:edit')")
    @Log(title = "商品", businessType = BusinessType.UPDATE)
    @PutMapping("/revoke")
    public AjaxResult revoke(@RequestBody PsiGoods psiGoods)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiGoods.setTenantId(tenantId);
        return toAjax(psiGoodsService.revokePsiGoods(psiGoods));
    }

}
