package com.ruoyi.psi.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.psi.domain.PsiStorage;
import com.ruoyi.psi.domain.vo.PsiSrorageItemVO;
import com.ruoyi.psi.domain.vo.StorageItemExportVO;
import com.ruoyi.psi.domain.vo.StorageVO;
import com.ruoyi.psi.domain.vo.TableDataInfoVO;
import com.ruoyi.psi.service.PsiOutboundService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName PsiOutboundController
 * @Description
 * @Author fm
 * @Date 2023/12/29 15:30
 * @Version V1.0
 */
@RestController
@RequestMapping("/psi/outbound")
public class PsiOutboundController extends BaseController {

    @Autowired
    private PsiOutboundService outboundService;

    /**
     *
     * 查询出库单列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:outbound:list')")
    @GetMapping("/list")
    public TableDataInfoVO list(StorageVO psiStorage)
    {
        Long tenantId = this.getTenantId();
        psiStorage.setTenantId(tenantId);
        Map<String, Object> params = psiStorage.getParams();
        String supplierIds = params.get("supplierIds") == null ? "" : params.get("supplierIds").toString();
        if(!StringUtils.isBlank(supplierIds)){
            List<Long> collect = Arrays.stream(supplierIds.split(",")).map(Long::parseLong).collect(Collectors.toList());
            params.put("supplierIdsList",collect);
        }
        startPage();
        List<StorageVO> list = outboundService.selectPsiOutboundList(psiStorage);
        // 获得入库总金额
        Map<String,Object> totalAmountMap = outboundService.getTotalAmount(psiStorage);
        TableDataInfo dataTable = getDataTable(list);
        TableDataInfoVO vo = new TableDataInfoVO();
        BeanUtils.copyProperties(dataTable,vo);
        vo.setExc(totalAmountMap);
        return vo;
    }

    /**
     * 获得出库单详细
     */
    // @PreAuthorize("@ss.hasPermi('psi:outbound:query')")
    @GetMapping(value = "/getInfoById/{id}")
    public AjaxResult getInfoByIds(@PathVariable("id") Long id)
    {
        return success(outboundService.getOutboundInfoById(id));
    }

    /**
     * 导出出库单详情
     */
    // @PreAuthorize("@ss.hasPermi('psi:outbound:export')")
    @PostMapping("/exportItem")
    public void exportItem(HttpServletResponse response, PsiStorage psiStorage)
    {
        // 获取当前登录用户所属租户ID
        // Long tenantId = this.getTenantId();
        // psiStorage.setTenantId(tenantId);
        List<StorageItemExportVO> list = outboundService.exportItem(psiStorage);
        ExcelUtil<StorageItemExportVO> util = new ExcelUtil<StorageItemExportVO>(StorageItemExportVO.class);
        util.exportExcel(response, list, "出库单","出库单");
    }

    /**
     * 撤回 (将已出库的订单改为未出库)
     */
    @PreAuthorize("@ss.hasPermi('psi:outbound:revoke')")
    @GetMapping(value = "/updateStatus/{id}")
    public AjaxResult updateStatus(@PathVariable("id") Long id)
    {
        return toAjax(outboundService.updateStatus(id));
    }


}