package com.ruoyi.psi.controller;

import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.psi.domain.PsiPurchaseItem;
import com.ruoyi.psi.domain.vo.PsiPurchaseVo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.psi.domain.PsiPurchase;
import com.ruoyi.psi.service.IPsiPurchaseService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 采购订单Controller
 * 
 * @author ruoyi
 * @date 2023-12-13
 */
@RestController
@RequestMapping("/psi/purchase")
public class PsiPurchaseController extends BaseController
{
    @Autowired
    private IPsiPurchaseService psiPurchaseService;

    /**
     * 查询采购订单列表
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:list')")
    @GetMapping("/list")
    public TableDataInfo list(PsiPurchase psiPurchase)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiPurchase.setTenantId(tenantId);
        startPage();
        List<Map> list = psiPurchaseService.selectMapResultList(psiPurchase);
        return getDataTable(list);
    }

    /**
     * 导出采购订单列表
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:export')")
    @Log(title = "采购订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiPurchase psiPurchase)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiPurchase.setTenantId(tenantId);
        List<PsiPurchase> list = psiPurchaseService.selectPsiPurchaseList(psiPurchase);
        ExcelUtil<PsiPurchase> util = new ExcelUtil<PsiPurchase>(PsiPurchase.class);
        util.exportExcel(response, list, "采购订单数据");
    }

    /**
     * 获取采购订单详细信息
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiPurchaseService.selectPsiPurchaseById(id, tenantId));
    }

    /**
     * 新增采购订单
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:add')")
    @Log(title = "采购订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PsiPurchaseVo psiPurchaseVo)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiPurchaseVo.setTenantId(tenantId);
        // 设置创建人
        psiPurchaseVo.setCreateUser(this.getUserId());
        return toAjax(psiPurchaseService.insertPsiPurchase(psiPurchaseVo));
    }

    /**
     * 修改采购订单
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:edit')")
    @Log(title = "采购订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PsiPurchaseVo psiPurchaseVo)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiPurchaseVo.setTenantId(tenantId);
        return toAjax(psiPurchaseService.updatePsiPurchase(psiPurchaseVo));
    }

    /**
     * 修改采购订单状态
     * @param id
     * @param action 动作 0：撤回 1：提交
     * @return
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:updStatus')")
    @Log(title = "修改采购订单状态", businessType = BusinessType.UPDATE)
    @GetMapping("/updStatus/{id}/{action}")
    public AjaxResult updPurchaseStatus(@PathVariable("id") Long id, @PathVariable("action") Integer action)
    {
        if(action != 0 && action != 1){
            return error();
        }
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        PsiPurchase psiPurchase = new PsiPurchase();
        psiPurchase.setTenantId(tenantId);
        psiPurchase.setId(id);
        psiPurchase.setStatus(action);
        psiPurchase.setUpdateTime(DateUtils.getNowDate());
        return toAjax(psiPurchaseService.updatePsiPurchaseStatus(psiPurchase));
    }

    /**
     * 审核采购订单
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:examine')")
    @Log(title = "审核采购订单", businessType = BusinessType.UPDATE)
    @GetMapping("/examine/{id}")
    public AjaxResult examinePurchaseOrder(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        PsiPurchase psiPurchase = new PsiPurchase();
        psiPurchase.setTenantId(tenantId);
        psiPurchase.setId(id);
        psiPurchase.setStatus(2);
        psiPurchase.setExamineUser(this.getUserId());
        psiPurchase.setExamineTime(DateUtils.getNowDate());
        return toAjax(psiPurchaseService.updatePsiPurchaseStatus(psiPurchase));
    }

    /**
     * 撤回 (将订货单撤回为采购单)
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:revoke')")
    @GetMapping(value = "/revokePurchase/{id}")
    public AjaxResult revokePurchase(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        PsiPurchase psiPurchase = new PsiPurchase();
        psiPurchase.setTenantId(tenantId);
        psiPurchase.setId(id);
        psiPurchase.setStatus(0);
        psiPurchase.setExamineUser(this.getUserId());
        psiPurchase.setExamineTime(DateUtils.getNowDate());
        return toAjax(psiPurchaseService.updatePsiPurchaseStatus(psiPurchase));
    }

    /**
     * 删除采购订单
     */
    @PreAuthorize("@ss.hasPermi('psi:purchase:remove')")
    @Log(title = "采购订单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return toAjax(psiPurchaseService.deletePsiPurchaseByIds(ids, tenantId));
    }


    /**
     *
     * 汇总报表目录下采购明细
     */
    // @PreAuthorize("@ss.hasPermi('psi:purchase:list')")
    @GetMapping("/listAll")
    public TableDataInfo listAll(PsiPurchase psiPurchase)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiPurchase.setTenantId(tenantId);
        List<PsiPurchaseItem> list = psiPurchaseService.listAll(psiPurchase);
        return getDataTable(list);
    }

    /**
     * 汇总报表，目录下采购查询 导出
     */
    // @PreAuthorize("@ss.hasPermi('psi:purchase:list')")
    @PostMapping("/exportQueryListAll")
    public void exportQueryListAll(HttpServletResponse response,PsiPurchase psiPurchase)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiPurchase.setTenantId(tenantId);
        List<PsiPurchaseItem> list = psiPurchaseService.exportListAll(psiPurchase);
        ExcelUtil<PsiPurchaseItem> util = new ExcelUtil<PsiPurchaseItem>(PsiPurchaseItem.class);
        util.exportExcel(response, list, "采购订单数据","采购明细");
    }

}
