package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.psi.domain.PsiPurchase;
import com.ruoyi.psi.domain.PsiSupply;
import com.ruoyi.psi.domain.vo.ReceiptExportVO;
import com.ruoyi.psi.domain.vo.SupplyVO;
import com.ruoyi.psi.domain.vo.TableDataInfoVO;
import com.ruoyi.psi.service.PsiReceiptService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName PsiReceiptController
 * @Description
 * @Author fm
 * @Date 2023/12/19 21:36
 * @Version V1.0
 */
@RestController
@RequestMapping("/psi/receipt")
public class PsiReceiptController extends BaseController {

    @Autowired
    private PsiReceiptService receiptService;

    // 收货
    @PreAuthorize("@ss.hasPermi('psi:receipt:add')")
    @Log(title = "收货", businessType = BusinessType.INSERT)
    @PostMapping("/addReceipt")
    public AjaxResult addReceipt(@RequestBody SupplyVO supplyVO)
    {
        // 获取当前登录用户所属租户ID
        // Long tenantId = this.getTenantId();
        // supplyVO.setTenantId(tenantId);
        return toAjax(receiptService.addReceipt(supplyVO));
    }

    // 已收货订单列表
    @PreAuthorize("@ss.hasPermi('psi:receipt:list')")
    @GetMapping("/list")
    public TableDataInfoVO list(PsiSupply psiSupply)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiSupply.setTenantId(tenantId);
        Map<String, Object> params = psiSupply.getParams();
        String supplierIds = params.get("supplierIds") == null ? "" : params.get("supplierIds").toString();
        if(!StringUtils.isBlank(supplierIds)){
            List<Long> collect = Arrays.stream(supplierIds.split(",")).map(Long::parseLong).collect(Collectors.toList());
            params.put("supplierIdsList",collect);
        }
        startPage();
        List<PsiSupply> list = receiptService.selectList(psiSupply);
        // 获得收货总金额
        Map<String,Object> totalAmountMap = receiptService.getTotalAmount(psiSupply);
        TableDataInfo dataTable = getDataTable(list);
        TableDataInfoVO vo = new TableDataInfoVO();
        BeanUtils.copyProperties(dataTable,vo);
        vo.setExc(totalAmountMap);
        return vo;
    }

    // 查看已收货详细
    @PreAuthorize("@ss.hasPermi('psi:receipt:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(receiptService.selectPsiSupplyById(id,tenantId));
    }

    /**
     * 导出收货单
     */
    @PreAuthorize("@ss.hasPermi('psi:receipt:export')")
    @Log(title = "收货单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiSupply psiSupply)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiSupply.setTenantId(tenantId);
        List<ReceiptExportVO> result = receiptService.exportItem(psiSupply);
        ExcelUtil<ReceiptExportVO> util = new ExcelUtil<ReceiptExportVO>(ReceiptExportVO.class);
        util.exportExcel(response, result, "收货单详情数据");
    }


    // 手动收货
    @PreAuthorize("@ss.hasPermi('psi:receipt:manually')")
    @Log(title = "收货", businessType = BusinessType.INSERT)
    @PostMapping("/manuallyReceipt")
    public AjaxResult manuallyReceipt(@RequestBody SupplyVO supplyVO)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        supplyVO.setTenantId(tenantId);
        return toAjax(receiptService.manuallyReceipt(supplyVO));
    }

    /**
     *
     * * 已收货的订单(不分页)
     * @param psiSupply
     * @return
     */
    // @PreAuthorize("@ss.hasPermi('psi:receipt:list')")
    @GetMapping("/listAll")
    public AjaxResult listAll(PsiSupply psiSupply)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiSupply.setTenantId(tenantId);
        List<PsiSupply> list = receiptService.listAll(psiSupply);
        return success(list);
    }

    /**
     * @param id
     * @param status 2:暂存 3:已确认
     * @return
     */
    // @PreAuthorize("@ss.hasPermi('psi:receipt:updStatus')")
    @Log(title = "修改收货单状态", businessType = BusinessType.UPDATE)
    @PostMapping("/updStatus/{id}/{status}")
    public AjaxResult updateReceiptStatus(@PathVariable("id") Long id, @PathVariable("status") Integer status)
    {
        if(status != 2 && status != 1){
            return error();
        }
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        PsiSupply supply = new PsiSupply();
        supply.setId(id);
        supply.setStatus(status);
        supply.setTenantId(tenantId);
        return toAjax(receiptService.updateReceiptStatus(supply));
    }


    // 更改收货订单的信息
    // @PreAuthorize("@ss.hasPermi('psi:receipt:update')")
    @Log(title = "收货", businessType = BusinessType.UPDATE)
    @PostMapping("/updateReceipt")
    public AjaxResult updateReceipt(@RequestBody SupplyVO supplyVO)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        supplyVO.setTenantId(tenantId);
        return toAjax(receiptService.updateReceipt(supplyVO));
    }

    /**
     * 撤回 (将已出库的订单改为未出库)
     */
    @PreAuthorize("@ss.hasPermi('psi:receipt:revoke')")
    @GetMapping(value = "/revokeReceipt/{id}")
    public AjaxResult revokeReceipt(@PathVariable("id") Long id)
    {
        return toAjax(receiptService.revokeReceipt(id));
    }
}