package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.psi.domain.PsiPurchase;
import com.ruoyi.psi.domain.PsiPurchaseItem;
import com.ruoyi.psi.domain.PsiSrorageItem;
import com.ruoyi.psi.domain.PsiStorage;
import com.ruoyi.psi.domain.vo.*;
import com.ruoyi.psi.service.IPsiStorageService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 入库单Controller
 * 
 * @author ruoyi
 * @date 2023-12-22
 */
@RestController
@RequestMapping("/psi/storage")
public class PsiStorageController extends BaseController
{
    @Autowired
    private IPsiStorageService psiStorageService;

    /**
     *
     * 查询入库单列表
     */
    @PreAuthorize("@ss.hasPermi('psi:storage:list')")
    @GetMapping("/list")
    public TableDataInfoVO list(StorageVO psiStorage)
    {
        Long tenantId = this.getTenantId();
        psiStorage.setTenantId(tenantId);
        Map<String, Object> params = psiStorage.getParams();
        String supplierIds = params.get("supplierIds") == null ? "" : params.get("supplierIds").toString();
        if(!StringUtils.isBlank(supplierIds)){
            List<Long> collect = Arrays.stream(supplierIds.split(",")).map(Long::parseLong).collect(Collectors.toList());
            params.put("supplierIdsList",collect);
        }
        startPage();
        List<StorageVO> list = psiStorageService.selectPsiStorageVoList(psiStorage);
        // 获得入库总金额
        Map<String,Object> totalAmountMap = psiStorageService.getTotalAmount(psiStorage);
        TableDataInfo dataTable = getDataTable(list);
        TableDataInfoVO vo = new TableDataInfoVO();
        BeanUtils.copyProperties(dataTable,vo);
        vo.setExc(totalAmountMap);
        return vo;
    }

    /**
     * 查询入库单列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:nopage')")
    @GetMapping("/listNoPage")
    public AjaxResult listNoPage(StorageVO psiStorage)
    {
        Long tenantId = this.getTenantId();
        psiStorage.setTenantId(tenantId);
        psiStorage.setStatus(null);
        List<PsiStorage> list = psiStorageService.selectPsiStorageList(psiStorage);
        Map<String,Object> totalAmountMap = psiStorageService.getTotalAmount(psiStorage);
        AjaxResult ajaxResult = AjaxResult.success(list);
        ajaxResult.put("totalAmount",totalAmountMap.get("totalAmount"));
        return ajaxResult;
    }

    /**
     * 获得所有入库单详情
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:query')")
    @PostMapping(value = "/getInfoByIds")
    public AjaxResult getInfoByIds(@RequestBody List<Long> ids)
    {
        AjaxResult ajaxResult = AjaxResult.success(psiStorageService.getInfoByIds(ids));
        ajaxResult.put("ids",ids);
        return ajaxResult;
    }

    /**
     * 获取入库单详细信息
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(psiStorageService.selectPsiStorageById(id));
    }


    /**
     * 导出入库单列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:export')")
    // @Log(title = "入库单", businessType = BusinessType.EXPORT)
    // @PostMapping("/export")
    // public void export(HttpServletResponse response, PsiStorage psiStorage)
    // {
    //     List<PsiStorage> list = psiStorageService.selectPsiStorageList(psiStorage);
    //     ExcelUtil<PsiStorage> util = new ExcelUtil<PsiStorage>(PsiStorage.class);
    //     util.exportExcel(response, list, "入库单数据");
    // }

    /**
     * 修改
     */
    @PreAuthorize("@ss.hasPermi('psi:storage:edit')")
    @Log(title = "入库单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PsiStorage psiStorage)
    {
        return toAjax(psiStorageService.updatePsiStorage(psiStorage));
    }

    /**
     * 出库 (审批)
     */
    @PreAuthorize("@ss.hasPermi('psi:storage:outbound')")
    @Log(title = "入库单", businessType = BusinessType.UPDATE)
    @PutMapping("/outbound")
    public AjaxResult outbound(@RequestBody PsiStorage psiStorage)
    {
        return toAjax(psiStorageService.outbound(psiStorage));
    }

    /**
     * 删除入库单
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:remove')")
    // @Log(title = "入库单", businessType = BusinessType.DELETE)
	// @DeleteMapping("/{ids}")
    // public AjaxResult remove(@PathVariable Long[] ids)
    // {
    //     return toAjax(psiStorageService.deletePsiStorageByIds(ids));
    // }


    /**
     * 商品入库
     */
//    @PreAuthorize("@ss.hasPermi('psi:storage:add')")
    @Log(title = "入库单", businessType = BusinessType.INSERT)
    @PostMapping("/warehousing")
    public AjaxResult add(@RequestBody SupplyVO supplyVO)
    {
        return toAjax(psiStorageService.insertPsiStorage(supplyVO));
    }

    /**
     *  根据供应商id 月份 格式yyyy-MM-dd HH:mm:ss 查询入库汇总
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:storageList')")
    @PostMapping("/storageList")
    public AjaxResult storageList(@RequestBody Map<String,Object> param)
    {
        Long tenantId = this.getTenantId();
        param.put("tenantId",tenantId);
        Map rMap = psiStorageService.storageList(param);
        return success(rMap);
    }

    /**
     * 新增汇总信息（入库）
     * @param param
     * @return
     */
    // @PreAuthorize("@ss.hasPermi('psi:storage:addSummary')")
    @Log(title = "入库单", businessType = BusinessType.INSERT)
    @PostMapping("/addSummary")
    public AjaxResult addSummary(@RequestBody Map<String,Object> param)
    {
        Long tenantId = this.getTenantId();
        param.put("tenantId",tenantId);
        return toAjax(psiStorageService.addSummary(param));
    }


    /**
     * 导出入库单详情列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:item:export')")
    @Log(title = "入库单", businessType = BusinessType.EXPORT)
    @PostMapping("/exportItem")
    public void exportItem(HttpServletResponse response, PsiSrorageItem psiSrorageItem)
    {
        List<StorageItemExportVO> vos = psiStorageService.exportStorageItem(psiSrorageItem);
        ExcelUtil<StorageItemExportVO> util = new ExcelUtil<StorageItemExportVO>(StorageItemExportVO.class);
        //"入库单详情数据"
        util.exportExcel(response, vos, "入库单详情数据","入库单");
    }

    /**
     * 批量导出入库单详情列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:item:export')")
    @Log(title = "批量导出入库单", businessType = BusinessType.EXPORT)
    @PostMapping("/exportItems")
    public void exportItems(HttpServletResponse response, PsiSrorageItem psiSrorageItem)
    {
        Map<String, Object> params = psiSrorageItem.getParams();
        String ids = params.get("ids") == null ? "" : params.get("ids").toString();
        if(!StringUtils.isBlank(ids)){
            List<Long> collect = Arrays.stream(ids.split(",")).map(Long::parseLong).collect(Collectors.toList());
            params.put("idsList",collect);
        }
        List<StorageItemExportVO> vos = psiStorageService.exportStorageItems(psiSrorageItem);
        ExcelUtil<StorageItemExportVO> util = new ExcelUtil<StorageItemExportVO>(StorageItemExportVO.class);
        util.exportExcel(response, vos, "入库单详情数据","入库单");
    }


    /**
     * 汇总报表，目录下入库明细
     */
    // @PreAuthorize("@ss.hasPermi('psi:purchase:list')")
    @GetMapping("/listAll")
    public TableDataInfo listAll(StorageVO psiStorage)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStorage.setTenantId(tenantId);
        List<PsiSrorageItem> list = psiStorageService.listAll(psiStorage);
        return getDataTable(list);
    }

    /**
     * 汇总报表，目录下入库明细 导出
     */
    // @PreAuthorize("@ss.hasPermi('psi:purchase:list')")
    @PostMapping("/exportQueryListAll")
    public void exportQueryListAll(HttpServletResponse response,StorageVO psiStorage)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStorage.setTenantId(tenantId);
        List<PsiSrorageItemVO> list = psiStorageService.exportQueryListAll(psiStorage);
        ExcelUtil<PsiSrorageItemVO> util = new ExcelUtil<PsiSrorageItemVO>(PsiSrorageItemVO.class);
        util.exportExcel(response, list, "入库单","入库明细");
    }

    /**
     * 撤回 (将已出库的订单改为未出库)
     */
    @PreAuthorize("@ss.hasPermi('psi:storage:revoke')")
    @GetMapping(value = "/revokeStorage/{id}")
    public AjaxResult revokeStorage(@PathVariable("id") Long id)
    {
        return toAjax(psiStorageService.revokeStorage(id));
    }

    /**
     * 库存数量分页列表
     * @param storageNumberVO
     * @return
     */
    @PreAuthorize("@ss.hasPermi('psi:storage:listInventoryQuantity')")
    @GetMapping("/listInventoryQuantity")
    public TableDataInfoVO listInventoryQuantity(StorageNumberVO storageNumberVO)
    {
        Long tenantId = this.getTenantId();
        storageNumberVO.setTenantId(tenantId);

        startPage();
        List<StorageNumberVO> list = psiStorageService.listInventoryQuantity(storageNumberVO);
        TableDataInfo dataTable = getDataTable(list);
        TableDataInfoVO vo = new TableDataInfoVO();
        BeanUtils.copyProperties(dataTable,vo);
        return vo;
    }

    /**
     * 库存数量 列表导出
     */
    @PreAuthorize("@ss.hasPermi('psi:storage:exportListInventoryQuantity')")
    @Log(title = "库存数量", businessType = BusinessType.EXPORT)
    @PostMapping("/exportListInventoryQuantity")
    public void exportListInventoryQuantity(HttpServletResponse response,StorageNumberVO storageNumberVO)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        storageNumberVO.setTenantId(tenantId);
        List<StorageNumberVO> list = psiStorageService.listInventoryQuantity(storageNumberVO);

        ExcelUtil<StorageNumberVO> util = new ExcelUtil<StorageNumberVO>(StorageNumberVO.class);
        util.exportExcel(response,list, "库存数量","库存数量详细信息");
    }
}
