package com.ruoyi.psi.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.psi.domain.PsiStore;
import com.ruoyi.psi.service.IPsiStoreService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 仓库Controller
 * 
 * @author ruoyi
 * @date 2024-01-03
 */
@RestController
@RequestMapping("/psi/store")
public class PsiStoreController extends BaseController
{
    @Autowired
    private IPsiStoreService psiStoreService;

    /**
     * 查询仓库列表
     */
    @GetMapping("/listAll")
    public TableDataInfo listAll(PsiStore psiStore)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStore.setTenantId(tenantId);
        List<PsiStore> list = psiStoreService.selectPsiStoreList(psiStore);
        return getDataTable(list);
    }

    /**
     * 查询仓库列表
     */
    @PreAuthorize("@ss.hasPermi('psi:store:list')")
    @GetMapping("/list")
    public TableDataInfo list(PsiStore psiStore)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStore.setTenantId(tenantId);
        startPage();
        List<PsiStore> list = psiStoreService.selectPsiStoreList(psiStore);
        return getDataTable(list);
    }

    /**
     * 导出仓库列表
     */
    @PreAuthorize("@ss.hasPermi('psi:store:export')")
    @Log(title = "仓库", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiStore psiStore)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStore.setTenantId(tenantId);
        List<PsiStore> list = psiStoreService.selectPsiStoreList(psiStore);
        ExcelUtil<PsiStore> util = new ExcelUtil<PsiStore>(PsiStore.class);
        util.exportExcel(response, list, "仓库数据");
    }

    /**
     * 获取仓库详细信息
     */
    @PreAuthorize("@ss.hasPermi('psi:store:query')")
    @GetMapping(value = "/{storeId}")
    public AjaxResult getInfo(@PathVariable("storeId") Long storeId)
    {
        return success(psiStoreService.selectPsiStoreByStoreId(storeId));
    }

    /**
     * 新增仓库
     */
    @PreAuthorize("@ss.hasPermi('psi:store:add')")
    @Log(title = "仓库", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PsiStore psiStore)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStore.setTenantId(tenantId);
        return toAjax(psiStoreService.insertPsiStore(psiStore));
    }

    /**
     * 修改仓库
     */
    @PreAuthorize("@ss.hasPermi('psi:store:edit')")
    @Log(title = "仓库", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PsiStore psiStore)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiStore.setTenantId(tenantId);
        return toAjax(psiStoreService.updatePsiStore(psiStore));
    }

    /**
     * 删除仓库
     */
    @PreAuthorize("@ss.hasPermi('psi:store:remove')")
    @Log(title = "仓库", businessType = BusinessType.DELETE)
	@DeleteMapping("/{storeIds}")
    public AjaxResult remove(@PathVariable Long[] storeIds)
    {
        return toAjax(psiStoreService.deletePsiStoreByStoreIds(storeIds));
    }
}
