package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.psi.domain.PsiSummary;
import com.ruoyi.psi.domain.vo.SummaryVO;
import com.ruoyi.psi.domain.vo.TableDataInfoVO;
import com.ruoyi.psi.service.IPsiSummaryService;
import com.ruoyi.psi.util.EUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * Controller
 *
 * @author ruoyi
 * @date 2023-12-25
 */
@RestController
@RequestMapping("/psi/summary")
public class PsiSummaryController extends BaseController
{
    @Autowired
    private IPsiSummaryService psiSummaryService;

    /**
     * 查询列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:list')")
    @GetMapping("/list")
    public TableDataInfoVO list(PsiSummary psiSummary)
    {
        Long tenantId = this.getTenantId();
        psiSummary.setTenantId(tenantId);
        startPage();
        List<PsiSummary> list = psiSummaryService.selectPsiSummaryList(psiSummary);
        // 获得入库总金额
        Map<String,Object> totalAmountMap = psiSummaryService.getTotalAmount(psiSummary);
        TableDataInfo dataTable = getDataTable(list);
        TableDataInfoVO vo = new TableDataInfoVO();
        BeanUtils.copyProperties(dataTable,vo);
        vo.setExc(totalAmountMap);
        return vo;
    }

    /**
     * 获取详细信息
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:query')")
    @GetMapping(value = "/{id}/{type}")
    public AjaxResult getInfo(@PathVariable("id") Long id, @PathVariable Integer type)
    {
        return success(psiSummaryService.selectPsiSummaryById(id,type));
    }

    /**
     * 更改入库汇总状态
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:query')")
    @PutMapping(value = "/{id}")
    public AjaxResult updateType(@PathVariable("id") Long id)
    {
        return success(psiSummaryService.updateType(id));
    }

    /**
     * 导出列表
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:export')")
    @Log(title = "【汇总】", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiSummary psiSummary) {
        Integer type = psiSummary.getType();
        String msg = null;
        if (type == 1) {
            msg = "鄂尔多斯供电分公司职工餐厅入库汇总表";
        } else {
            msg = "鄂尔多斯供电分公司职工餐厅出库汇总表";
        }
        // todo 验证
        SummaryVO vo = psiSummaryService.selectPsiSummaryById(psiSummary.getId(),type);
        EUtil u = new EUtil();
        u.exportData(msg, vo, response);
    }


    /**
     * 新增
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:add')")
    // @Log(title = "【请填写功能名称】", businessType = BusinessType.INSERT)
    // @PostMapping
    // public AjaxResult add(@RequestBody PsiSummary psiSummary)
    // {
    //     return toAjax(psiSummaryService.insertPsiSummary(psiSummary));
    // }

    /**
     * 修改
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:edit')")
    // @Log(title = "【请填写功能名称】", businessType = BusinessType.UPDATE)
    // @PutMapping
    // public AjaxResult edit(@RequestBody PsiSummary psiSummary)
    // {
    //     return toAjax(psiSummaryService.updatePsiSummary(psiSummary));
    // }

    /**
     * 删除【请填写功能名称】
     */
    // @PreAuthorize("@ss.hasPermi('psi:summary:remove')")
    // @Log(title = "【请填写功能名称】", businessType = BusinessType.DELETE)
    // @DeleteMapping("/{ids}")
    // public AjaxResult remove(@PathVariable Long[] ids)
    // {
    //     return toAjax(psiSummaryService.deletePsiSummaryByIds(ids));
    // }

    /**
     * 出库汇总撤回 (将已出库汇总的改为已入库汇总，将已入库汇总的记录删除)
     */
    @PreAuthorize("@ss.hasPermi('psi:summary:revokeOut')")
    @GetMapping(value = "/revokeSummaryOut/{id}")
    public AjaxResult revokeSummaryOut(@PathVariable("id") Long id)
    {
        return toAjax(psiSummaryService.revokeSummary(id, 2));
    }

    /**
     * 入库汇总撤回 (将已出库汇总的改为已入库汇总，将已入库汇总的记录删除)
     */
    @PreAuthorize("@ss.hasPermi('psi:summary:revokeIn')")
    @GetMapping(value = "/revokeSummaryIn/{id}")
    public AjaxResult revokeSummaryIn(@PathVariable("id") Long id)
    {
        return toAjax(psiSummaryService.revokeSummary(id, 1));
    }
}
