package com.ruoyi.psi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.psi.domain.PsiSupplier;
import com.ruoyi.psi.domain.vo.PsiSupplierVO;
import com.ruoyi.psi.service.IPsiSupplierService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 供应商Controller
 *
 * @author ruoyi
 * @date 2023-12-12
 */
@RestController
@RequestMapping("/psi/supplier")
public class PsiSupplierController extends BaseController
{
    @Autowired
    private IPsiSupplierService psiSupplierService;

    /**
     * 查询供应商列表
     */
//    @PreAuthorize("@ss.hasPermi('psi:supplier:list')")
    @GetMapping("/list")
    public TableDataInfo list(PsiSupplier psiSupplier)
    {
        startPage();
        List<PsiSupplierVO> list = psiSupplierService.selectPsiSupplierList(psiSupplier);
        return getDataTable(list);
    }

    /**
     * 租户登录查询关联的供应商列表
     */
    @PreAuthorize("@ss.hasPermi('psi:supplier:tlist')")
        @GetMapping("/tlist")
    public TableDataInfo tlist(PsiSupplier psiSupplier)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        startPage();
        List<PsiSupplier> list = psiSupplierService.selectPsiSupplierListByTenant(psiSupplier, tenantId);
        return getDataTable(list);
    }

    /**
     * 导出供应商列表
     */
    @PreAuthorize("@ss.hasPermi('psi:supplier:export')")
    @Log(title = "供应商", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiSupplier psiSupplier)
    {
        List<PsiSupplierVO> list = psiSupplierService.selectPsiSupplierList(psiSupplier);
        ExcelUtil<PsiSupplierVO> util = new ExcelUtil<PsiSupplierVO>(PsiSupplierVO.class);
        util.exportExcel(response, list, "供应商数据");
    }

    /**
     * 获取供应商详细信息
     */
    @PreAuthorize("@ss.hasPermi('psi:supplier:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(psiSupplierService.selectPsiSupplierById(id));
    }

    /**
     * 新增供应商
     */
    @PreAuthorize("@ss.hasPermi('psi:supplier:add')")
    @Log(title = "供应商", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PsiSupplierVO vo)
    {
        return toAjax(psiSupplierService.insertPsiSupplier(vo));
    }

    /**
     * 修改供应商
     */
    @PreAuthorize("@ss.hasPermi('psi:supplier:edit')")
    @Log(title = "供应商", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PsiSupplierVO vo)
    {
        return toAjax(psiSupplierService.updatePsiSupplier(vo));
    }

    @PostMapping("/updateSupplierStatus")
    public AjaxResult updateSupplierStatus(@RequestBody PsiSupplierVO vo)
    {
        return toAjax(psiSupplierService.updateSupplierStatus(vo));
    }


    /**
     * 删除供应商
     */
    @PreAuthorize("@ss.hasPermi('psi:supplier:remove')")
    @Log(title = "供应商", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(psiSupplierService.deletePsiSupplierByIds(ids));
    }

    /**
     * 供应商导入模板
     * @param response
     */
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<PsiSupplier> util = new ExcelUtil<PsiSupplier>(PsiSupplier.class);
        util.importTemplateExcel(response, "供应商数据");
    }

    /**
     * 供应商导入
     * @param file
     * @param
     * @return
     * @throws Exception
     */
    @Log(title = "供应商管理", businessType = BusinessType.IMPORT)
    @PreAuthorize("@ss.hasPermi('system:user:import')")
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file) throws Exception
    {
        ExcelUtil<PsiSupplier> util = new ExcelUtil<PsiSupplier>(PsiSupplier.class);
        List<PsiSupplier> psiSupplierList = util.importExcel(file.getInputStream());
        String message = psiSupplierService.importPsiSupplier(psiSupplierList);
        return success(message);
    }

    /**
     * 根据登录租户获得所有的供应商
     */
    // @PreAuthorize("@ss.hasPermi('psi:supplier:query')")
    @GetMapping("/getSupplierTenant")
    public AjaxResult getSupplierTenant()
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiSupplierService.getSupplierTenant(tenantId));
    }
}
