package com.ruoyi.psi.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.vo.CalculateGoodsReq;
import com.ruoyi.psi.domain.vo.SupplyVO;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.psi.domain.PsiSupply;
import com.ruoyi.psi.service.IPsiSupplyService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 供货单Controller
 * 
 * @author ruoyi
 * @date 2023-12-14
 */
@RestController
@RequestMapping("/psi/supply")
public class PsiSupplyController extends BaseController
{
    @Autowired
    private IPsiSupplyService psiSupplyService;


    /**
     * 新增供货单
     */
    // @PreAuthorize("@ss.hasPermi('psi:supply:add')")
    // @Log(title = "供货单", businessType = BusinessType.INSERT)
    // @PostMapping
    // public AjaxResult add(@RequestBody PsiSupply psiSupply)
    // {
    //     return toAjax(psiSupplyService.insertPsiSupply(psiSupply));
    // }

    /**
     * 修改供货单
     */
    // @PreAuthorize("@ss.hasPermi('psi:supply:edit')")
    // @Log(title = "供货单", businessType = BusinessType.UPDATE)
    // @PutMapping
    // public AjaxResult edit(@RequestBody PsiSupply psiSupply)
    // {
    //     return toAjax(psiSupplyService.updatePsiSupply(psiSupply));
    // }

    /**
     * 删除供货单
     */
    // @PreAuthorize("@ss.hasPermi('psi:supply:remove')")
    // @Log(title = "供货单", businessType = BusinessType.DELETE)
	// @DeleteMapping("/{ids}")
    // public AjaxResult remove(@PathVariable Long[] ids)
    // {
    //     return toAjax(psiSupplyService.deletePsiSupplyByIds(ids));
    // }

    // ****************************************

    /**
     * 导出供货单列表
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:export')")
    @Log(title = "供货单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PsiSupply psiSupply)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiSupply.setTenantId(tenantId);
        psiSupply.setDeliveryMethod(1);
        List<PsiSupply> list = psiSupplyService.selectPsiSupplyList(psiSupply);
        ExcelUtil<PsiSupply> util = new ExcelUtil<PsiSupply>(PsiSupply.class);
        util.exportExcel(response, list, "供货单数据");
    }

    /**
     * 查询供货单列表
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:list')")
    @GetMapping("/list")
    public TableDataInfo list(PsiSupply psiSupply)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        psiSupply.setTenantId(tenantId);
        psiSupply.setDeliveryMethod(1);
        startPage();
        List<PsiSupply> list = psiSupplyService.selectPsiSupplyList(psiSupply);
        return getDataTable(list);
    }

    /**
     * 获取供货单详细信息
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiSupplyService.selectPsiSupplyById(id,tenantId));
    }

    /**
     * 获取供货单详细信息(已入库数量 != 实际数量)
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfoTwo(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiSupplyService.selectPsiSupplyByIdTwo(id,tenantId));
    }

    /**
     * 根据供货单id和指定金额，计算出哪些商品列表总金额最接近指定金额
     */
//    @PreAuthorize("@ss.hasPermi('psi:supply:query')")
    @PostMapping(value = "/calculateGoods")
    public AjaxResult calculateGoods(@RequestBody CalculateGoodsReq calVo)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiSupplyService.calculateGoods(calVo,tenantId));
    }

    /**
     * 根据采购单id 以及供货商id 生成供货单数据
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:query')")
    @GetMapping(value = "/getSupply/{purchaseId}/{supplierId}")
    public AjaxResult getSupplyByPurchaseId(@PathVariable("purchaseId") Long purchaseId,
                                            @PathVariable("supplierId") Long supplierId)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        return success(psiSupplyService.getSupplyByPurchaseId(purchaseId,tenantId,supplierId));
    }

    /**
     * 根据采购单生成一条供货单
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:add')")
    @Log(title = "供货单", businessType = BusinessType.INSERT)
    @PostMapping("/addSupply")
    public AjaxResult addSupply(@RequestBody SupplyVO supplyVO)
    {
        // 获取当前登录用户所属租户ID
        Long tenantId = this.getTenantId();
        supplyVO.setTenantId(tenantId);
        return toAjax(psiSupplyService.addSupply(supplyVO));
    }

    /**
     * 撤回 (将订货单撤回为采购单)
     */
    @PreAuthorize("@ss.hasPermi('psi:supply:revoke')")
    @GetMapping(value = "/revokeSupply/{id}")
    public AjaxResult revokeSupply(@PathVariable("id") Long id)
    {
        return toAjax(psiSupplyService.revokeSupply(id));
    }
}
