package com.ruoyi.psi.mapper;

import com.ruoyi.psi.domain.ApprovalRecording;
import com.ruoyi.psi.domain.vo.ApprovalRecordingVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 审批记录Mapper接口
 * 
 * @author ruoyi
 * @date 2024-01-19
 */
public interface ApprovalRecordingMapper 
{
    /**
     * 查询审批记录
     *
     * @return 审批记录
     */
    public List<ApprovalRecording> selectApprovalRecordingById(ApprovalRecording approvalRecording);

    /**
     * 查询审批记录列表
     * 
     * @param approvalRecording 审批记录
     * @return 审批记录集合
     */
    public List<ApprovalRecordingVO> selectApprovalRecordingList(ApprovalRecording approvalRecording);

    /**
     * 新增审批记录
     * 
     * @param approvalRecording 审批记录
     * @return 结果
     */
    public int insertApprovalRecording(ApprovalRecording approvalRecording);

    /**
     * 修改审批记录
     * 
     * @param approvalRecording 审批记录
     * @return 结果
     */
    public int updateApprovalRecording(ApprovalRecording approvalRecording);

    /**
     * 删除审批记录
     * 
     * @param documentId 审批记录主键
     * @return 结果
     */
    public int deleteApprovalRecordingByDocumentIdAndTenantId(@Param("documentId") Long documentId, @Param("tenantId") Long tenantId,
                                                              @Param("documentType") Integer documentType);

    /**
     * 批量删除审批记录
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteApprovalRecordingByIds(Long[] ids);

    /**
     * 批量新增审批记录*
     * @param approvalRecording
     * @return
     */
    int insertBatchApprovalRecording(@Param("items") List<ApprovalRecording> approvalRecording);

    /**
     * 根据单据id、审批顺序号、租户id查询审批记录
     * @param documentId
     * @param approvalOrder
     * @return
     */
    ApprovalRecording selectNextApprovalRecording(@Param("documentId") Long documentId,
                                                  @Param("approvalOrder") Integer approvalOrder,

                                                  @Param("documentType") Integer documentType);

    /**
     * 批量更新
     * @param approvalRecordingList
     * @return
     */
    int updateBatchApprovalRecording(@Param("items") List<ApprovalRecording> approvalRecordingList);

    /**
     * 查询未审批记录(审批按钮)
     * @param approvalRecording
     * @return
     */
    ApprovalRecording selectApprovalRecordingNoApproval(ApprovalRecording approvalRecording);

    /**
     * 查询该笔订单是否全部通过
     * @param documentId
     * @param documentType
     * @param tenantId
     * @return
     */
    int selectDocumentIsPass(@Param("documentId") Long documentId, @Param("documentType")Integer documentType, @Param("tenantId")Long tenantId);


    List<ApprovalRecording> selectApprovalRecordingListByDocument(ApprovalRecording approvalRecording);


    List<Long> selectApprovalTenantId(@Param("documentId") Long documentId, @Param("documentType") Integer documentType);

    /**
     * 已办列表*
     * @param approvalRecording
     * @return
     */
    List<ApprovalRecordingVO> selectApprovalDoneRecordingList(ApprovalRecording approvalRecording);

    /**
     * 根据单据类型和单据id查询审批记录
     * @param documentType
     * @param documentId
     * @return
     */
    Integer selectApprovalRecordingByTypeAndId(@Param("documentType") Integer documentType, @Param("documentId") Long documentId);
}
