package com.ruoyi.psi.mapper;

import java.util.List;
import com.ruoyi.psi.domain.PsiGoods;
import com.ruoyi.psi.domain.PsiGoodsAnnex;
import com.ruoyi.psi.domain.PsiSupplier;
import com.ruoyi.psi.domain.vo.PsiGoodsAnnexVO;
import org.apache.ibatis.annotations.Param;

/**
 * 商品Mapper接口
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
public interface PsiGoodsMapper 
{
    /**
     * 查询商品
     * 
     * @param id 商品主键
     * @return 商品
     */
    public PsiGoods selectPsiGoodsById(@Param("id") Long id, @Param("tenantId") Long tenantId);

    /**
     * 查询商品列表
     * 
     * @param psiGoods 商品
     * @return 商品集合
     */
    public List<PsiGoods> selectPsiGoodsList(PsiGoods psiGoods, List<PsiSupplier> supplierList);

    /**
     * 新增商品
     * 
     * @param psiGoods 商品
     * @return 结果
     */
    public int insertPsiGoods(PsiGoods psiGoods);

    /**
     * 修改商品
     * 
     * @param psiGoods 商品
     * @return 结果
     */
    public int updatePsiGoods(PsiGoods psiGoods);

    /**
     * 删除商品
     * 
     * @param id 商品主键
     * @return 结果
     */
    public int deletePsiGoodsById(@Param("id") Long id, @Param("tenantId") Long tenantId);

    /**
     * 批量删除商品
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePsiGoodsByIds(@Param("array") Long[] ids, @Param("tenantId") Long tenantId);

    /**
     * 商品名称+规格+供应商+租户 唯一
     * @param psiGoods
     * @return
     */
    PsiGoods selectOneGoods(PsiGoods psiGoods);

    PsiGoods selectOneGoodsByNameAndSpec(@Param("goodsName") String goodsName, @Param("spec") String spec, @Param("supplierId") Long supplierId, @Param("tenantId") Long tenantId);

    /**
     * 根据商品名称+规格+租户 查询，判断该租户是否有该商品，可能有多条，即有多个供应商可以供应该商品
     * @param goodsName
     * @param spec
     * @param tenantId
     * @return
     */
    Integer selectGoodsCountByNameAndSpec(@Param("goodsName") String goodsName, @Param("spec") String spec, @Param("tenantId") Long tenantId);


    /**
     * 商品名称+规格+租户 获得商品
     * @param psiGoods
     * @return
     */
    List<PsiGoods> selectGoodsByPurchase(PsiGoods psiGoods);

    /**
     * 已审核的商品
     * @param psiGoods
     * @param supplierList
     * @return
     */
    List<PsiGoods> selectIsApprovalPsiGoodsList(PsiGoods psiGoods, List<PsiSupplier> supplierList);

    /**
     * 插入审核表数据
     * @param psiGoods
     */
    int insertGoodsApproval(PsiGoods psiGoods);

    /**
     * 查询审批的商品
     * @param documentId
     * @param tenantId
     * @return
     */
    PsiGoods selectApprovalGoodsById(@Param("id") Long documentId, @Param("tenantId") Long tenantId);

    /**
     * 商品列表*
     * @param psiGoods
     * @param supplierList
     * @return
     */
    List<PsiGoods> goodsList(PsiGoods psiGoods, List<PsiSupplier> supplierList);

    /**
     * 删除商品审批表数据
     * @param id
     */
    public int deleteApprovalPsiGoodsById(@Param("id") Long id, @Param("tenantId") Long tenantId);


    public PsiGoodsAnnexVO selectApprovalGoodsByIdNoTenantId(Long documentId);


    PsiGoodsAnnexVO selectGoodsWithGoods(Long documentId);

    /**
     * 插入附件数据
     * @param annex
     */
    void insertGoodsAnnex(PsiGoodsAnnex annex);

    /**
     * 删除附件
     * @param id
     */
    void deleteAnnexById(Long id);
}
