package com.ruoyi.psi.mapper;

import java.util.List;

import com.ruoyi.psi.domain.PsiSupplier;
import com.ruoyi.psi.domain.vo.PsiSupplierVO;
import com.ruoyi.psi.domain.vo.SupplierTenantVO;
import org.apache.ibatis.annotations.Param;

/**
 * 供应商Mapper接口
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
public interface PsiSupplierMapper 
{
    /**
     * 查询供应商
     * 
     * @param id 供应商主键
     * @return 供应商
     */
    public PsiSupplier selectPsiSupplierById(Long id);

    /**
     * 查询供应商列表
     * 
     * @param psiSupplier 供应商
     * @return 供应商集合
     */
    public List<PsiSupplierVO> selectPsiSupplierList(PsiSupplier psiSupplier);

    public List<PsiSupplier> selectPsiSupplierListByTenant(@Param("supplier") PsiSupplier psiSupplier, @Param("tenantId") Long tenantId);

    /**
     * 新增供应商
     * 
     * @param
     * @return 结果
     */
    public int insertPsiSupplier(PsiSupplierVO vo);

    /**
     * 修改供应商
     * 
     * @param
     * @return 结果
     */
    public int updatePsiSupplier(PsiSupplierVO vo);

    /**
     * 删除供应商
     * 
     * @param id 供应商主键
     * @return 结果
     */
    public int deletePsiSupplierById(Long id);

    /**
     * 批量删除供应商
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePsiSupplierByIds(Long[] ids);

    /**
     * 根据名称查询供应商
     * *
     * @param supplierName
     * @return
     */
    PsiSupplier selectByName(String supplierName);

    /**
     * @description 新增关联关系
     *
     * @param supplierId
     * @param tenantId
     * @return void
     * @author fm
     * @date 2023/12/13 17:48
     */
    void insertPsiSupplierTenantRef(SupplierTenantVO tvo);

    /**
     * @description
     *
     * @param supplierId
     * @return void
     * @author fm
     * @date 2023/12/13 18:00
     */
    void deletePsiSupplierTenantRef(Long supplierId);

    /**
     * @description 根据当前登录租户获得所有的供应商
     *
     * @param tenantId
     * @return void
     * @author fm
     * @date 2023/12/13 19:16
     */
    List<PsiSupplier> getSupplierTenant(Long tenantId);

    /**
     * 查询供应商id*
     * @param supplierName
     * @return
     */
    Long getSupplierByName(String supplierName);

    List<PsiSupplier> selectPsiSupplierListByIds(Long[] ids);

    List<SupplierTenantVO> getTenantByPhone(String phoneNumber);
}
