package com.ruoyi.psi.mapper;

import java.util.List;
import java.util.Map;

import com.ruoyi.psi.domain.PsiSupply;
import com.ruoyi.psi.domain.PsiSupplyItem;
import com.ruoyi.psi.domain.vo.SupplyVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.security.core.parameters.P;

/**
 * 供货单Mapper接口
 * 
 * @author ruoyi
 * @date 2023-12-14
 */
public interface PsiSupplyMapper 
{
    /**
     * 查询供货单
     * 
     * @param id 供货单主键
     * @return 供货单
     */
    public PsiSupply selectPsiSupplyById(@Param("id") Long id, @Param("tenantId") Long tenantId);

    /**
     * 查询供货单列表
     * 
     * @param psiSupply 供货单
     * @return 供货单集合
     */
    public List<PsiSupply> selectPsiSupplyList(PsiSupply psiSupply);

    /**
     * 新增供货单
     * 
     * @param psiSupply 供货单
     * @return 结果
     */
    public int insertPsiSupply(PsiSupply psiSupply);

    /**
     * 修改供货单
     * 
     * @param psiSupply 供货单
     * @return 结果
     */
    public int updatePsiSupply(PsiSupply psiSupply);

    /**
     * 删除供货单
     * 
     * @param id 供货单主键
     * @return 结果
     */
    public int deletePsiSupplyById(Long id);

    /**
     * 批量删除供货单
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePsiSupplyByIds(Long[] ids);

    /**
     * 批量插入供货单详情
     *
     * @param supplyItems
     * @return
     */
    int insertSupplyItem(@Param("items") List<PsiSupplyItem> supplyItems, @Param("supplyId") Long supplyId);

    /**
     * 根据供货单id 查询供货单详细信息
     * @param id
     * @return
     */
    List<PsiSupplyItem> selectSupplyItem(Long id);

    void updateSupplyStorageAmount(PsiSupply psiSupply);

    List<PsiSupplyItem> selectNoStoragySupplyItem(Long id);

    /**
     * 修改收货单状态为未确认
     * @param id
     * @return
     */
    int updateSupplyStatusById(Long id);

    /**
     * 修改收货单状态为订货状态
     * @param id
     * @return
     */
    int revokeSupplyToOrdering(Long id);

    PsiSupply selectSupplyByPurchaseId(Long purchaseId);
}
