package com.ruoyi.psi.service;

import java.util.List;
import java.util.Map;

import com.ruoyi.psi.domain.PsiPurchase;
import com.ruoyi.psi.domain.PsiPurchaseItem;
import com.ruoyi.psi.domain.vo.PsiPurchaseVo;

/**
 * 采购订单Service接口
 * 
 * @author ruoyi
 * @date 2023-12-13
 */
public interface IPsiPurchaseService 
{
    /**
     * 查询采购订单
     * 
     * @param id 采购订单主键
     * @return 采购订单
     */
    public PsiPurchaseVo selectPsiPurchaseById(Long id, Long tenantId);

    /**
     * 查询采购订单列表
     * 
     * @param psiPurchase 采购订单
     * @return 采购订单集合
     */
    public List<PsiPurchase> selectPsiPurchaseList(PsiPurchase psiPurchase);

    /**
     * 查询采购订单列表
     *
     * @param psiPurchase 采购订单
     * @return 采购订单集合
     */
    public List<Map> selectMapResultList(PsiPurchase psiPurchase);

    /**
     * 新增采购订单
     * 
     * @param psiPurchaseVo 采购订单
     * @return 结果
     */
    public int insertPsiPurchase(PsiPurchaseVo psiPurchaseVo);

    /**
     * 修改采购订单
     * 
     * @param psiPurchaseVo 采购订单
     * @return 结果
     */
    public int updatePsiPurchase(PsiPurchaseVo psiPurchaseVo);

    /**
     * 修改订单状态
     * @param psiPurchase
     * @return
     */
    public int updatePsiPurchaseStatus(PsiPurchase psiPurchase);

    /**
     * 批量删除采购订单
     * 
     * @param ids 需要删除的采购订单主键集合
     * @return 结果
     */
    public int deletePsiPurchaseByIds(Long[] ids, Long tenantId);

    /**
     * 删除采购订单信息
     * 
     * @param id 采购订单主键
     * @return 结果
     */
    public int deletePsiPurchaseById(Long id, Long tenantId);

    /***
     * 汇总报表目录下采购明细
     * @param psiPurchase
     * @return
     */
    List<PsiPurchaseItem> listAll(PsiPurchase psiPurchase);

    /**
     * 汇总报表，目录下采购查询 导出
     */
    List<PsiPurchaseItem> exportListAll(PsiPurchase psiPurchase);
}
