package com.ruoyi.psi.service;

import java.util.List;

import com.ruoyi.psi.domain.PsiSupplier;
import com.ruoyi.psi.domain.vo.PsiSupplierVO;
import com.ruoyi.psi.domain.vo.SupplierTenantVO;

/**
 * 供应商Service接口
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
public interface IPsiSupplierService 
{
    /**
     * 查询供应商
     * 
     * @param id 供应商主键
     * @return 供应商
     */
    public PsiSupplier selectPsiSupplierById(Long id);

    /**
     * 查询供应商列表
     * 
     * @param psiSupplier 供应商
     * @return 供应商集合
     */
    public List<PsiSupplierVO> selectPsiSupplierList(PsiSupplier psiSupplier);

    /**
     * 租户登录查询关联的供应商列表
     *
     * @param psiSupplier 供应商
     * @return 供应商集合
     */
    public List<PsiSupplier> selectPsiSupplierListByTenant(PsiSupplier psiSupplier, Long tenantId);

    /**
     * 新增供应商
     * 
     * @param
     * @return 结果
     */
    public int insertPsiSupplier(PsiSupplierVO vo);

    /**
     * 修改供应商
     * 
     * @param
     * @return 结果
     */
    public int updatePsiSupplier(PsiSupplierVO vo);

    public int updateSupplierStatus(PsiSupplierVO vo);

    /**
     * 批量删除供应商
     * 
     * @param ids 需要删除的供应商主键集合
     * @return 结果
     */
    public int deletePsiSupplierByIds(Long[] ids);

    /**
     * 删除供应商信息
     * 
     * @param id 供应商主键
     * @return 结果
     */
    public int deletePsiSupplierById(Long id);

    /**
     * 批量导入供应商
     *
     * @param psiSupplierList
     * @param
     * @return
     */
    public String importPsiSupplier(List<PsiSupplier> psiSupplierList);

    /**
     * 根据登录租户 获得供应商
     * @param tenantId
     * @return
     */
    List<PsiSupplier> getSupplierTenant(Long tenantId);

    /**
     * 根据名称查询供应商id
     * @param supplierName
     * @return
     */
    Long getSupplierByName(String supplierName);

    List<SupplierTenantVO> getTenantByPhone(String phoneNumber);
}
