package com.ruoyi.psi.service.impl;

import java.util.List;
import java.util.Map;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.psi.mapper.ApprovalProcessMapper;
import com.ruoyi.psi.domain.ApprovalProcess;
import com.ruoyi.psi.service.IApprovalProcessService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * 审批流程Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-01-08
 */
@Service
public class ApprovalProcessServiceImpl implements IApprovalProcessService 
{
    @Autowired
    private ApprovalProcessMapper approvalProcessMapper;

    @Autowired
    private SysTenantMapper sysTenantMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    /**
     * 查询审批流程
     * @return 审批流程
     */
    @Override
    public List<ApprovalProcess> selectApprovalProcessById(ApprovalProcess approvalProcess)
    {
        return approvalProcessMapper.selectApprovalProcessById(approvalProcess);
    }

    /**
     * 查询审批流程列表
     * 
     * @param approvalProcess 审批流程
     * @return 审批流程
     */
    @Override
    public List<ApprovalProcess> selectApprovalProcessList(ApprovalProcess approvalProcess)
    {
        return approvalProcessMapper.selectApprovalProcessList(approvalProcess);
    }

    /**
     * 新增审批流程
     * 
     * @param approvalProcess 审批流程
     * @return 结果
     */
    @Override
    public int insertApprovalProcess(ApprovalProcess approvalProcess)
    {
        // 获得租户名称
        // SysTenant sysTenant = sysTenantMapper.selectSysTenantById(approvalProcess.getTenantId());
        // approvalProcess.setTenantName(sysTenant.getTenantName());
        // approvalProcess.setTenantId(sysTenant.getId());
        // 审批人名称
        // userType 01代表租户，02代表供应商
        // SysUser sysUser = sysUserMapper.selectUserById(approvalProcess.getApprovalUser(), sysTenant.getId());
        // approvalProcess.setApprovalUser(sysUser.getUserId());
        // approvalProcess.setApprovalUserName(sysUser.getNickName());
        return approvalProcessMapper.insertApprovalProcess(approvalProcess);
    }

    /**
     * 修改审批流程
     * @return 结果
     */
    @Override
    @Transactional
    public int updateApprovalProcess(List<ApprovalProcess> approvalProcessList, Long tenantId)
    {
        if(CollectionUtils.isEmpty(approvalProcessList)){
            return 1;
        }
        for (ApprovalProcess approvalProcess : approvalProcessList) {
            approvalProcess.setIsEnd(0);
        }
        ApprovalProcess approvalProcess = approvalProcessList.get(approvalProcessList.size() - 1);
        Integer documentType = approvalProcess.getDocumentType();
        approvalProcess.setIsEnd(1);
        // 删除
        approvalProcessMapper.deleteByDocumentTypeAndTenantId(documentType,tenantId);
        return approvalProcessMapper.insertBatchApprovalProcess(tenantId,approvalProcessList);
    }

    /**
     * 批量删除审批流程
     * 
     * @param ids 需要删除的审批流程主键
     * @return 结果
     */
    @Override
    public int deleteApprovalProcessByIds(Long[] ids)
    {
        return approvalProcessMapper.deleteApprovalProcessByIds(ids);
    }

    /**
     * 删除审批流程信息
     * 
     * @param id 审批流程主键
     * @return 结果
     */
    @Override
    public int deleteApprovalProcessById(Long id)
    {
        return approvalProcessMapper.deleteApprovalProcessById(id);
    }

}
