package com.ruoyi.psi.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.ruoyi.common.constant.ApprovalConstants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.psi.domain.*;
import com.ruoyi.psi.domain.vo.*;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.service.IPsiStorageService;
import com.ruoyi.psi.service.IPsiSummaryService;
import com.ruoyi.psi.util.CommonUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.psi.service.IApprovalRecordingService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 审批记录Service业务层处理
 *
 * @author ruoyi
 * @date 2024-01-19
 */
@Service
public class ApprovalRecordingServiceImpl implements IApprovalRecordingService
{
    @Autowired
    private ApprovalRecordingMapper approvalRecordingMapper;

    @Autowired
    private PsiPurchaseMapper purchaseMapper;

    @Autowired
    private PsiStorageMapper storageMapper;

    @Autowired
    private IPsiStorageService storageService;

    @Autowired
    private PsiSummaryMapper summaryMapper;

    @Autowired
    private IPsiSummaryService summaryService;

    @Autowired
    private PsiGoodsMapper goodsMapper;

    @Autowired
    private PsiPurchaseItemMapper purchaseItemMapper;

    @Autowired
    private CommonUtil commonUtil;

    @Autowired
    private PsiSrorageItemMapper srorageItemMapper;

    @Autowired
    private PsiSummaryItemMapper summaryItemMapper;

    private static final Logger log = LoggerFactory.getLogger(ApprovalRecordingServiceImpl.class);

    /**
     * 查询审批记录
     *
     * @return 审批记录
     */
    @Override
    public List<ApprovalRecording> selectApprovalRecordingById(ApprovalRecording approvalRecording)
    {
        return approvalRecordingMapper.selectApprovalRecordingById(approvalRecording);
    }

    /**
     * 查询审批记录列表
     *
     * @param approvalRecording 审批记录
     * @return 审批记录
     */
    @Override
    public List<ApprovalRecordingVO> selectApprovalRecordingList(ApprovalRecording approvalRecording)
    {
        return approvalRecordingMapper.selectApprovalRecordingList(approvalRecording);
    }

    /**
     * 新增审批记录
     *
     * @param approvalRecordingList 审批记录
     * @return 结果
     */
    @Override
    @Transactional
    public int insertApprovalRecording(List<ApprovalRecording> approvalRecordingList)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Long userId = user.getUserId();
        String userName = user.getNickName();
        Long tenantId = user.getTenantId();
        // 查看是否有重复的审核人
        // List<Long> collect = approvalRecordingList.stream().collect(Collectors.groupingBy(ApprovalRecording::getApprovalUser))
        //         .entrySet().stream().filter(entry -> entry.getValue().size() > 1).map(Map.Entry::getKey)
        //         .collect(Collectors.toList());
        // if(!CollectionUtils.isEmpty(collect)){
        //     throw new ServiceException("不能出现重复的审核人!");
        // }
        if(!CollectionUtils.isEmpty(approvalRecordingList)){
            // 获得餐厅的租户id
            Long diningTenantId = approvalRecordingList.get(1).getTenantId();
            if(!Objects.equals(tenantId, diningTenantId)){
                tenantId = diningTenantId;
            }
            /*
                2024/06/13新增 防止重复提交
             */
            Integer documentType = approvalRecordingList.get(0).getDocumentType();
            Long documentId = approvalRecordingList.get(0).getDocumentId();
            Integer count = approvalRecordingMapper.selectApprovalRecordingByTypeAndId(documentType, documentId);
            log.info("提交审批时 [documentType] :" + "[" + documentType + "] ==== [documentId] : [" + documentId + "]");
            if (count > 0) {
                throw new ServiceException("该单据已提交审批!");
            }
            //------------------------------------------------
            for (ApprovalRecording recording : approvalRecordingList) {
                // Long tenantId1 = recording.getTenantId();
                // if(!Objects.equals(tenantId1, tenantId)){
                //     tenantId = tenantId1;
                // }
                Integer approvalOrder = recording.getApprovalOrder();
                if(null != approvalOrder && 1 == approvalOrder){
                    // 第一条默认已审核通过
                    recording.setSubmitId(userId);
                    recording.setSubmitName(userName);
                    recording.setSubmitTime(DateUtils.getNowDate());
                    recording.setApprovarTime(DateUtils.getNowDate());
                    recording.setApprovarType(1);
                    recording.setApprovarOpinion(0);
                }else if(null != approvalOrder && 2 == approvalOrder){
                    // 第二条的提交人设置为第一个审核人
                    recording.setSubmitId(userId);
                    recording.setSubmitName(userName);
                    recording.setSubmitTime(DateUtils.getNowDate());
                }
                recording.setIsEnd(0);
                // recording.setzTenantId(tenantId);
            }
            // 最后节点
            ApprovalRecording approvalRecording = approvalRecordingList.get(approvalRecordingList.size() - 1);
            approvalRecording.setIsEnd(1);
            // 改变订单状态
            this.updateDocumentStatus(approvalRecording.getDocumentId(), tenantId, approvalRecording.getDocumentType(),1);
        }
        Long zTenantId = null;
        for (ApprovalRecording approvalRecording : approvalRecordingList) {
            zTenantId = approvalRecording.getzTenantId();
            if(Objects.isNull(zTenantId)){
                // 用户新增
                approvalRecording.setzTenantId(approvalRecording.getTenantId());
            }
            approvalRecording.setTenantId(tenantId);
            approvalRecordingMapper.insertApprovalRecording(approvalRecording);
        }
        return 1;
    }



    /**
     * 修改审批记录
     *
     * @param approvalRecordings 审批记录
     * @return 结果
     */
    @Override
    @Transactional
    public int updateApprovalRecording(List<ApprovalRecording> approvalRecordings, Long tenantId)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        for (ApprovalRecording approvalRecording : approvalRecordings) {
            Integer approvarOpinion = approvalRecording.getApprovarOpinion();
            ApprovalRecording nextApprovalRecording = null;
            if(approvarOpinion == 0){
                // approvarOpinion == 0 代表通过
                // 查询下一节点
                nextApprovalRecording = approvalRecordingMapper.selectNextApprovalRecording(approvalRecording.getDocumentId(),
                        approvalRecording.getApprovalOrder() + 1,
                        approvalRecording.getDocumentType());
                if(Objects.isNull(nextApprovalRecording)){
                    // 查询是否全部通过
                    int noPassCount = approvalRecordingMapper.selectDocumentIsPass(approvalRecording.getDocumentId(),
                            approvalRecording.getDocumentType(),approvalRecording.getTenantId());
                    if(noPassCount <= 0){
                        this.updateDocumentStatus(approvalRecording.getDocumentId(),approvalRecording.getTenantId(),approvalRecording.getDocumentType(),2);
                    }
                }else {
                    nextApprovalRecording.setSubmitId(user.getUserId());
                    nextApprovalRecording.setSubmitName(user.getNickName());
                    nextApprovalRecording.setSubmitTime(DateUtils.getNowDate());
                }
            }
            // approvalRecording.setSubmitTime(DateUtils.getNowDate());
            approvalRecording.setApprovarTime(DateUtils.getNowDate());
            approvalRecording.setApprovarType(1);

            List<ApprovalRecording> approvalRecordingList = new ArrayList<>();
            approvalRecordingList.add(approvalRecording);
            if(!Objects.isNull(nextApprovalRecording)){
                approvalRecordingList.add(nextApprovalRecording);
            }
            if(!CollectionUtils.isEmpty(approvalRecordingList)){
                approvalRecordingMapper.updateBatchApprovalRecording(approvalRecordingList);
            }
        }
        return 1;
    }

    /**
     * 批量删除审批记录
     *
     * @param ids 需要删除的审批记录主键
     * @return 结果
     */
    @Override
    public int deleteApprovalRecordingByIds(Long[] ids)
    {
        return approvalRecordingMapper.deleteApprovalRecordingByIds(ids);
    }


    /**
     * 查询未审批记录(审批按钮)
     * @param approvalRecording
     * @return
     */
    @Override
    public ApprovalRecording selectApprovalRecordingNoApproval(ApprovalRecording approvalRecording) {
        return approvalRecordingMapper.selectApprovalRecordingNoApproval(approvalRecording);
    }

    /**
     * 获取单据详情
     * @param documentId
     * @param documentType
     * @return
     */
    @Override
    public AjaxResult psiGetRecordingInfo(Long documentId, Integer documentType) {
        // 单据类型:1供应商2商品3采购需求6入库7出库8入库汇总9出库汇总
        switch (documentType){
            case 1:
                break;
            case 2:
                // 先查询是否是修改的审核商品
                PsiGoodsAnnexVO goods = goodsMapper.selectApprovalGoodsByIdNoTenantId(documentId);
                if(Objects.isNull(goods)){
                    goods = goodsMapper.selectGoodsWithGoods(documentId);
                    goods.setType(0);
                }else{
                    // 修改前的数据
                    PsiGoodsAnnexVO originalGoods = goodsMapper.selectGoodsWithGoods(documentId);
                    Map<String,Object> map = new HashMap<>();
                    map.put("originalGoods",originalGoods);
                    goods.setExc(map);
                    goods.setType(1);
                }
                return AjaxResult.success(goods);
            case 3:
                PsiPurchase purchase = purchaseMapper.selectOnePsipurchaseById(documentId);
                PsiPurchaseVo psiPurchaseVo = new PsiPurchaseVo();
                BeanUtils.copyProperties(purchase, psiPurchaseVo);
                List<PsiPurchaseItem> items = purchaseItemMapper.selectPsiPurchaseItemByPurchaseId(documentId);
                psiPurchaseVo.setItems(items);
                return AjaxResult.success(psiPurchaseVo);
            case 6:
                PsiStorage storage = storageMapper.selectPsiStorageById(documentId);
                StorageVO storageVO = new StorageVO();
                org.springframework.beans.BeanUtils.copyProperties(storage,storageVO);
                List<PsiSrorageItem> psiSrorageItems = srorageItemMapper.selectPsiStorageItemByStorageId(storageVO.getId());

                // 查询入库流程详细
                ApprovalRecording approvalRecording = new ApprovalRecording();
                approvalRecording.setDocumentId(documentId);
                approvalRecording.setDocumentType(ApprovalConstants.RUKU);
                approvalRecording.setzTenantId(storage.getTenantId());
                List<ApprovalRecording> inApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);

                // 查询出库流程详细
                approvalRecording.setDocumentType(ApprovalConstants.CHUKU);
                List<ApprovalRecording> outApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);

                storageVO.setItems(psiSrorageItems);
                storageVO.setInApprovalRecordingList(inApprovalRecordingList);
                storageVO.setOutApprovalRecordingList(outApprovalRecordingList);
                return AjaxResult.success(storageVO);
            case 7:
                StorageVO outStorageVo = storageMapper.getOutboundInfoById(documentId);
                List<PsiSrorageItem> outStorageItem = srorageItemMapper.selectOutboundStorageItemByStorageId(documentId);
                outStorageVo.setItems(outStorageItem);
                // 查询出库流程详细
                ApprovalRecording outApprovalRecording = new ApprovalRecording();
                outApprovalRecording.setDocumentId(documentId);
                outApprovalRecording.setDocumentType(ApprovalConstants.CHUKU);
                outApprovalRecording.setzTenantId(outStorageVo.getTenantId());
                List<ApprovalRecording> outApprovalRecordingList1 = commonUtil.getRecordingList(outApprovalRecording);

                // 查询入库流程详细
                outApprovalRecording.setDocumentType(ApprovalConstants.RUKU);
                List<ApprovalRecording> inApprovalRecordingList1 = commonUtil.getRecordingList(outApprovalRecording);
                outStorageVo.setOutApprovalRecordingList(outApprovalRecordingList1);
                outStorageVo.setInApprovalRecordingList(inApprovalRecordingList1);
                return AjaxResult.success(outStorageVo);
            case 8:
                SummaryVO summaryVO = new SummaryVO();
                PsiSummary summary = summaryMapper.selectPsiSummaryById(documentId);
                org.springframework.beans.BeanUtils.copyProperties(summary,summaryVO);
                List<PsiSummaryItem> list = summaryItemMapper.selectSummaryItemBySummaryId(documentId);
                summaryVO.setSummaryItems(list);

                // 查询流程详细
                ApprovalRecording approvalRecordingTwo = new ApprovalRecording();
                approvalRecordingTwo.setDocumentId(documentId);
                approvalRecordingTwo.setDocumentType(ApprovalConstants.RUKUHUIZONG);
                approvalRecordingTwo.setzTenantId(summary.getTenantId());

                List<ApprovalRecording> ruApprovalRecordingList = commonUtil.getRecordingList(approvalRecordingTwo);
                summaryVO.setInApprovalRecordingList(ruApprovalRecordingList);

                approvalRecordingTwo.setDocumentType(ApprovalConstants.CHUKUHUIZONG);
                List<ApprovalRecording> chuApprovalRecordingList = commonUtil.getRecordingList(approvalRecordingTwo);
                summaryVO.setOutApprovalRecordingList(chuApprovalRecordingList);
                return AjaxResult.success(summaryVO);
            case 9:
                SummaryVO outSummaryVo = new SummaryVO();
                PsiSummary outSummary = summaryMapper.selectPsiSummaryById(documentId);
                org.springframework.beans.BeanUtils.copyProperties(outSummary,outSummaryVo);
                List<PsiSummaryItem> outSummaryList = summaryItemMapper.selectSummaryItemBySummaryId(documentId);
                outSummaryVo.setSummaryItems(outSummaryList);

                // 查询流程详细
                ApprovalRecording outSummaryApprovalRecording = new ApprovalRecording();
                outSummaryApprovalRecording.setDocumentId(documentId);
                outSummaryApprovalRecording.setDocumentType(ApprovalConstants.CHUKUHUIZONG);
                outSummaryApprovalRecording.setzTenantId(outSummary.getTenantId());

                List<ApprovalRecording> chuSummaryApprovalRecordingList1 = commonUtil.getRecordingList(outSummaryApprovalRecording);
                outSummaryVo.setOutApprovalRecordingList(chuSummaryApprovalRecordingList1);

                outSummaryApprovalRecording.setDocumentType(ApprovalConstants.RUKUHUIZONG);
                List<ApprovalRecording> ruSummaryApprovalRecordingList1 = commonUtil.getRecordingList(outSummaryApprovalRecording);
                outSummaryVo.setInApprovalRecordingList(ruSummaryApprovalRecordingList1);
                return AjaxResult.success(outSummaryVo);
            default:
                System.out.println("错误");
                break;
        }
        return AjaxResult.success();
    }

    /**
     * 已办列表
     * @param approvalRecording
     * @return
     */
    @Override
    public List<ApprovalRecordingVO> selectApprovalDoneRecordingList(ApprovalRecording approvalRecording) {
        return approvalRecordingMapper.selectApprovalDoneRecordingList(approvalRecording);
    }

    /**
     * 删除审批记录信息
     *
     * @return 结果
     */
    // @Override
    // public int deleteApprovalRecordingById(Long id)
    // {
    //     return approvalRecordingMapper.deleteApprovalRecordingById(id);
    // }

    // 更新订单状态    flag 1 审批  2 完成
    private void updateDocumentStatus(Long documentId, Long tenantId, Integer documentType, Integer flag) {
        // documentType 单据类型:1供应商2商品3采购需求6入库7出库8入库汇总9出库汇总
        List<Long> tenantIds = approvalRecordingMapper.selectApprovalTenantId(documentId,documentType);
        if(!CollectionUtils.isEmpty(tenantIds)){
            tenantId = tenantIds.get(0);
        }
        switch (documentType){
            case 1:
                break;
            case 2:
                PsiGoods goods = new PsiGoods();
                goods.setId(documentId);
                goods.setTenantId(tenantId);
                if(flag == 1){
                    goods.setApprovalType(2);
                    goodsMapper.updatePsiGoods(goods);
                }else {
                    // 删除审批记录
                    ApprovalRecording approvalRecording = new ApprovalRecording();
                    approvalRecording.setTenantId(tenantId);
                    approvalRecording.setDocumentId(documentId);
                    approvalRecording.setDocumentType(ApprovalConstants.SHANGPIN);
                    commonUtil.edit(approvalRecording);
                    PsiGoods goods1 = goodsMapper.selectApprovalGoodsById(documentId,tenantId);
                    if(Objects.isNull(goods1)){
                       // 新增
                        goods1 = new PsiGoods();
                        goods1.setId(documentId);
                    }
                    goods1.setApprovalType(1);
                    goodsMapper.updatePsiGoods(goods1);
                }
                break;
            case 3:
                PsiPurchase purchase = new PsiPurchase();
                purchase.setId(documentId);
                purchase.setTenantId(tenantId);
                if(flag == 1){
                    // 审批中
                    purchase.setStatus(4);
                }else {
                    // 完成审批
                    purchase.setStatus(2);
                }
                purchaseMapper.updatePsiPurchase(purchase);
                break;
            case 4:
                break;
            case 5:
                break;
            case 6:
                PsiStorage storage = new PsiStorage();
                storage.setId(documentId);
                storage.setTenantId(tenantId);
                if(flag == 1){
                    // 审批中
                    storage.setAudit(2);
                    storageMapper.updatePsiStorage(storage);
                }else {
                    // 完成审批
                    // storage.setAudit(1);
                    storage.setAudit(0);
                    storageService.outbound(storage);
                }
                break;
            case 7:
                PsiStorage storage1 = new PsiStorage();
                storage1.setId(documentId);
                storage1.setTenantId(tenantId);
                if(flag == 1){
                    // 审批中
                    // storage1.setStatus(2);
                    storage1.setAudit(2);
                    storageMapper.updatePsiStorage(storage1);
                }else {
                    // 完成审批
                    // storage1.setStatus(3);
                    storage1.setAudit(1);
                    storageMapper.updatePsiStorage(storage1);
                }
                break;
            case 8:
                if(flag == 1 ){
                    PsiSummary summary = new PsiSummary();
                    summary.setId(documentId);
                    summary.setTenantId(tenantId);
                    // 审核中
                    summary.setType(3);
                    summaryMapper.updatePsiSummary(summary);
                }else {
                    summaryService.updateType(documentId);
                }
                break;
            case 9:
                if(flag == 1 ){
                    PsiSummary summary = new PsiSummary();
                    summary.setId(documentId);
                    summary.setTenantId(tenantId);
                    // 出库审核中
                    summary.setType(4);
                    summaryMapper.updatePsiSummary(summary);
                }else {
                    PsiSummary summary = new PsiSummary();
                    summary.setId(documentId);
                    summary.setTenantId(tenantId);
                    // 出库审核中
                    summary.setType(5);
                    summaryMapper.updatePsiSummary(summary);
                }
                break;
            case 10:
                PsiStorage salesStorage = new PsiStorage();
                salesStorage.setId(documentId);
                if(flag == 1 ){
                    salesStorage.setSalesAuditStatus(2);
                    storageMapper.updatePsiStorage(salesStorage);
                }else {
                    salesStorage.setSalesAuditStatus(1);
                    storageMapper.updatePsiStorage(salesStorage);
                }
                break;
            default:
                System.err.println("不确定的单据类型");
                break;
        }
    }
}
