package com.ruoyi.psi.service.impl;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

import com.ruoyi.common.constant.ApprovalConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.psi.domain.ApprovalRecording;
import com.ruoyi.psi.domain.PsiGoodsAnnex;
import com.ruoyi.psi.domain.PsiSupplier;
import com.ruoyi.psi.domain.dto.ImportRequestDTO;
import com.ruoyi.psi.domain.dto.PsiGoodsDTO;
import com.ruoyi.psi.mapper.ApprovalRecordingMapper;
import com.ruoyi.psi.mapper.PsiSupplierMapper;
import com.ruoyi.psi.service.IPsiSupplierService;
import com.ruoyi.psi.util.CommonUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.psi.mapper.PsiGoodsMapper;
import com.ruoyi.psi.domain.PsiGoods;
import com.ruoyi.psi.service.IPsiGoodsService;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Validator;

/**
 * 商品Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
@Service
public class PsiGoodsServiceImpl implements IPsiGoodsService 
{
    @Autowired
    private PsiGoodsMapper psiGoodsMapper;

    @Autowired
    protected Validator validator;

    @Autowired
    protected IPsiSupplierService supplierService;

    @Autowired
    private PsiSupplierMapper supplierMapper;

    @Autowired
    private CommonUtil commonUtil;

    @Autowired
    private ApprovalRecordingMapper approvalRecordingMapper;


    private static final Logger log = LoggerFactory.getLogger(PsiGoodsServiceImpl.class);

    /**
     * 查询商品
     * 
     * @param id 商品主键
     * @return 商品
     */
    @Override
    public PsiGoods selectPsiGoodsById(Long id, Long tenantId)
    {
        return psiGoodsMapper.selectPsiGoodsById(id,tenantId);
    }

    /**
     * 查询商品列表
     * 
     * @param psiGoods 商品
     * @return 商品
     */
    @Override
    public List<PsiGoods> selectPsiGoodsList(PsiGoods psiGoods)
    {
        // 获得当前租户下所有的供应商
        List<PsiSupplier> supplierList = supplierMapper.getSupplierTenant(psiGoods.getTenantId());
        PageUtils.startPage();
        return psiGoodsMapper.selectPsiGoodsList(psiGoods,supplierList);
    }

    /**
     * 新增商品
     * 
     * @param psiGoods 商品
     * @return 结果
     */
    @Override
    public int insertPsiGoods(PsiGoodsDTO psiGoods)
    {
        // 重复校验
        String spec = psiGoods.getSpec() == null ? "" : psiGoods.getSpec().toLowerCase();
        psiGoods.setSpec(spec);
        PsiGoods goods = psiGoodsMapper.selectOneGoods(psiGoods);
        if(!Objects.isNull(goods)){
            log.error("======================新增商品报错，该名称商品已存在{}======================",goods.getGoodsName());
            throw new ServiceException("该商品已经存在！");
        }
        Integer priceType = psiGoods.getPriceType();
        // 时价默认为0
        if(null != priceType && priceType.equals(0)){
            psiGoods.setPrice(BigDecimal.ZERO);
        }

        PsiGoodsAnnex annex = new PsiGoodsAnnex();
        annex.setRemark(psiGoods.getExRemark());
        annex.setFilePath(psiGoods.getFileName());
        psiGoodsMapper.insertGoodsAnnex(annex);

        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        psiGoods.setCreateTime(DateUtils.getNowDate());
        psiGoods.setCreateBy(userId.toString());
        psiGoods.setUpdateBy(userId.toString());
        psiGoods.setUpdateTime(DateUtils.getNowDate());
        psiGoods.setApprovalType(0);
        psiGoods.setAnnexId(annex.getId());

        return psiGoodsMapper.insertPsiGoods(psiGoods);
    }
    /**
     * 修改商品
     * 
     * @param psiGoods 商品
     * @return 结果
     */
    @Override
    @Transactional
    public int updatePsiGoods(PsiGoodsDTO psiGoods)
    {
        // 重复校验
        String spec =  psiGoods.getSpec() == null ? "" : psiGoods.getSpec().toLowerCase();
        psiGoods.setSpec(spec);
        PsiGoods goods = psiGoodsMapper.selectOneGoods(psiGoods);
        if(!Objects.isNull(goods) && !goods.getId().equals(psiGoods.getId())){
            log.error("======================修改商品报错，该名称商品已存在{}======================",goods.getGoodsName());
            throw new ServiceException("该商品已经存在！");
        }
        PsiGoodsAnnex annex = new PsiGoodsAnnex();
        annex.setFilePath(psiGoods.getFileName());
        annex.setRemark(psiGoods.getExRemark());
        psiGoodsMapper.insertGoodsAnnex(annex);

        psiGoods.setUpdateTime(DateUtils.getNowDate());
        // 未审核
        PsiGoods approvalGoods = new PsiGoods();
        approvalGoods.setId(psiGoods.getId());
        approvalGoods.setTenantId(psiGoods.getTenantId());
        approvalGoods.setApprovalType(0);
        approvalGoods.setAnnexId(annex.getId());

        psiGoodsMapper.updatePsiGoods(approvalGoods);
        // 插入中间表
        // 查询是否已经更改过
        PsiGoods goods1 = psiGoodsMapper.selectApprovalGoodsById(psiGoods.getId(), psiGoods.getTenantId());
        if (!Objects.isNull(goods1)) {
            psiGoodsMapper.deleteApprovalPsiGoodsById(psiGoods.getId(), psiGoods.getTenantId());
            // psiGoodsMapper.deleteAnnexById(annex.getId());
        }
        psiGoodsMapper.insertGoodsApproval(psiGoods);
        return 1;
    }

    /**
     * 批量删除商品
     * 
     * @param ids 需要删除的商品主键
     * @return 结果
     */
    @Override
    public int deletePsiGoodsByIds(Long[] ids,Long tenantId)
    {
        return psiGoodsMapper.deletePsiGoodsByIds(ids,tenantId);
    }

    /**
     * 删除商品信息
     * 
     * @param id 商品主键
     * @return 结果
     */
    @Override
    public int deletePsiGoodsById(Long id, Long tenantId)
    {
        return psiGoodsMapper.deletePsiGoodsById(id,tenantId);
    }

    /**
     * 导入商品
     * @param psiGoodsList
     *
     * @return
     */
    @Override
    public String importPsiGoods(List<PsiGoods> psiGoodsList, Long tenantId, ImportRequestDTO requestDTO) {
        Long supplierId = requestDTO.getSupplierId();
        String remark = requestDTO.getRemark();
        String filePath = requestDTO.getFileName();

        if (StringUtils.isNull(psiGoodsList) || psiGoodsList.size() == 0)
        {
            throw new ServiceException("导入商品数据不能为空！");
        }
        //成功数
        int successNum = 0;
        //失败数
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();

        PsiGoodsAnnex annex = new PsiGoodsAnnex();
        annex.setFilePath(filePath);
        annex.setRemark(remark);
        psiGoodsMapper.insertGoodsAnnex(annex);

        for (PsiGoods psiGoods : psiGoodsList)
        {
            try
            {
                psiGoods.setGoodsName(psiGoods.getGoodsName().trim());
                psiGoods.setSupplierId(supplierId);
                psiGoods.setTenantId(tenantId);
                psiGoods.setSpec(psiGoods.getSpec() == null ? "" : psiGoods.getSpec().toLowerCase().trim());
                // 去重校验
                PsiGoods goods = psiGoodsMapper.selectOneGoods(psiGoods);
                if(Objects.isNull(goods)){
                    // 不存在则新增
                    BeanValidators.validateWithException(validator, psiGoods);
                    // 查询供应商信息
                    // String supplierName = psiGoods.getSupplierName();
                    // Long supplierId = supplierService.getSupplierByName(supplierName);
                    // if(!Objects.isNull(supplierId)){
                    //     psiGoods.setSupplierId(supplierId);
                    // }
                    psiGoods.setCreateBy(userId.toString());
                    psiGoods.setUpdateBy(userId.toString());
                    psiGoods.setUpdateTime(DateUtils.getNowDate());
                    psiGoods.setCreateTime(DateUtils.getNowDate());
                    psiGoods.setTenantId(tenantId);
                    // psiGoods.setGoogsCode(psiGoods.getId().toString());
                    // ===2023/12/23
                    Integer priceType = psiGoods.getPriceType();
                    // 时价默认为0
                    if(null != priceType && priceType.equals(0)){
                        psiGoods.setPrice(BigDecimal.ZERO);
                    }

                    // 默认启用
                    psiGoods.setStatus(1);
                    psiGoods.setApprovalType(0);
                    psiGoods.setAnnexId(annex.getId());
                    psiGoodsMapper.insertPsiGoods(psiGoods);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、名称 " + psiGoods.getGoodsName() + " 新增成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、名称 " + psiGoods.getGoodsName() + " 新增失败(已存在该商品)");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、名称 " +  psiGoods.getGoodsName() + " 导入失败";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 查询商品列表
     *
     * @param psiGoods
     * @return
     */
    @Override
    public List<PsiGoods> listAll(PsiGoods psiGoods) {
        // 获得当前租户下所有的供应商
        List<PsiSupplier> supplierList = supplierMapper.getSupplierTenant(psiGoods.getTenantId());
        return psiGoodsMapper.selectIsApprovalPsiGoodsList(psiGoods,supplierList);
    }

    @Override
    public List<PsiGoods> selectExportPsiGoodsList(PsiGoods psiGoods) {
        // 获得当前租户下所有的供应商
        List<PsiSupplier> supplierList = supplierMapper.getSupplierTenant(psiGoods.getTenantId());
        return psiGoodsMapper.selectPsiGoodsList(psiGoods,supplierList);
    }



    @Override
    public List<PsiGoods> goodsList(PsiGoods psiGoods) {
        // 获得当前租户下所有的供应商
        List<PsiSupplier> supplierList = supplierMapper.getSupplierTenant(psiGoods.getTenantId());
        PageUtils.startPage();
        return psiGoodsMapper.goodsList(psiGoods,supplierList);
    }

    /**
     * 撤回商品审核
     * @param psiGoods
     * @return
     */
    @Override
    @Transactional
    public int revokePsiGoods(PsiGoods psiGoods) {
        PsiGoods goods = psiGoodsMapper.selectApprovalGoodsById(psiGoods.getId(), psiGoods.getTenantId());
        if(Objects.isNull(goods)){
            // 新增撤回
            // 删除表中记录
            psiGoodsMapper.deletePsiGoodsById(psiGoods.getId(),psiGoods.getTenantId());
        }else {
            // 修改撤回
            psiGoods.setApprovalType(1);
            psiGoodsMapper.updatePsiGoods(psiGoods);
            psiGoodsMapper.deleteApprovalPsiGoodsById(goods.getId(),goods.getTenantId());
        }
        ApprovalRecording approvalRecording = new ApprovalRecording();
        approvalRecording.setDocumentType(ApprovalConstants.SHANGPIN);
        approvalRecording.setDocumentId(psiGoods.getId());
        approvalRecording.setTenantId(psiGoods.getTenantId());
        commonUtil.edit(approvalRecording);
        return 1;
    }
}
