package com.ruoyi.psi.service.impl;

import com.ruoyi.common.constant.ApprovalConstants;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.ApprovalRecording;
import com.ruoyi.psi.domain.PsiSrorageItem;
import com.ruoyi.psi.domain.PsiStorage;
import com.ruoyi.psi.domain.vo.StorageItemExportVO;
import com.ruoyi.psi.domain.vo.StorageVO;
import com.ruoyi.psi.mapper.PsiSrorageItemMapper;
import com.ruoyi.psi.mapper.PsiStorageMapper;
import com.ruoyi.psi.service.PsiOutboundService;
import com.ruoyi.psi.util.CommonUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * @ClassName PsiOutboundServiceImpl
 * @Description
 * @Author fm
 * @Date 2023/12/29 15:32
 * @Version V1.0
 */
@Service
public class PsiOutboundServiceImpl implements PsiOutboundService {

    @Autowired
    private PsiStorageMapper storageMapper;

    @Autowired
    private PsiSrorageItemMapper itemMapper;

    @Autowired
    private CommonUtil commonUtil;

    @Override
    public List<StorageVO> selectPsiOutboundList(StorageVO psiStorage) {
        return storageMapper.selectPsiOutboundList(psiStorage);
    }

    @Override
    public StorageVO getOutboundInfoById(Long id) {
        StorageVO vo = storageMapper.getOutboundInfoById(id);
        List<PsiSrorageItem> items = itemMapper.selectOutboundStorageItemByStorageId(id);
        vo.setItems(items);
        // 查询入库流程详细
        ApprovalRecording approvalRecording = new ApprovalRecording();
        approvalRecording.setDocumentId(id);
        approvalRecording.setDocumentType(ApprovalConstants.CHUKU);
        approvalRecording.setzTenantId(vo.getTenantId());
        List<ApprovalRecording> outApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);

        // 查询出库流程详细
        approvalRecording.setDocumentType(ApprovalConstants.RUKU);
        List<ApprovalRecording> inApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);
        vo.setOutApprovalRecordingList(outApprovalRecordingList);
        vo.setInApprovalRecordingList(inApprovalRecordingList);
        return vo;
    }

    @Override
    public List<StorageItemExportVO> exportItem(PsiStorage psiStorage) {
        List<StorageItemExportVO> infoByIds = itemMapper.exportItem(psiStorage);
        return infoByIds;
    }

    @Override
    @Transactional
    public int updateStatus(Long id) {
        storageMapper.updateStatus(id);
        itemMapper.updateStatus(id);
        // 删除审批记录
        Long tenantId = SecurityUtils.getLoginUser().getUser().getTenantId();
        ApprovalRecording approvalRecording = new ApprovalRecording();
        approvalRecording.setTenantId(tenantId);
        approvalRecording.setDocumentId(id);
        approvalRecording.setDocumentType(ApprovalConstants.RUKU);
        commonUtil.edit(approvalRecording);
        approvalRecording.setDocumentType(ApprovalConstants.CHUKU);
        commonUtil.edit(approvalRecording);
        return 1;
    }

    @Override
    public Map<String, Object> getTotalAmount(StorageVO psiStorage) {
        return storageMapper.selectPsiOutboundTotalAmountMap(psiStorage);
    }
}