package com.ruoyi.psi.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.psi.domain.ApprovalRecording;
import com.ruoyi.psi.domain.PsiPurchaseItem;
import com.ruoyi.psi.domain.PsiStore;
import com.ruoyi.psi.domain.vo.PsiPurchaseVo;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.util.CommonUtil;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.psi.domain.PsiPurchase;
import com.ruoyi.psi.service.IPsiPurchaseService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * 采购订单Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-13
 */
@Service
public class PsiPurchaseServiceImpl implements IPsiPurchaseService 
{
    @Autowired
    private PsiPurchaseMapper psiPurchaseMapper;

    @Autowired
    private PsiPurchaseItemMapper itemMapper;

    @Autowired
    private PsiGoodsMapper psiGoodsMapper;

    @Autowired
    private SysTenantMapper sysTenantMapper;

    @Autowired
    private PsiStoreMapper psiStoreMapper;

    @Autowired
    private CommonUtil commonUtil;

    @Autowired
    private PsiSupplyMapper supplyMapper;
    /**
     * 查询采购订单
     * 
     * @param id 采购订单主键
     * @return 采购订单
     */
    @Override
    public PsiPurchaseVo selectPsiPurchaseById(Long id, Long tenantId)
    {
        PsiPurchase purchase = psiPurchaseMapper.selectPsiPurchaseById(id, tenantId);
        PsiPurchaseVo vo = new PsiPurchaseVo();
        BeanUtils.copyProperties(purchase, vo);

        List<PsiPurchaseItem> items = itemMapper.selectPsiPurchaseItemByPurchaseId(id);
        vo.setItems(items);
        return vo;
    }

    /**
     * 查询采购订单列表
     * 
     * @param psiPurchase 采购订单
     * @return 采购订单
     */
    @Override
    public List<PsiPurchase> selectPsiPurchaseList(PsiPurchase psiPurchase)
    {
        return psiPurchaseMapper.selectPsiPurchaseList(psiPurchase);
    }

    @Override
    public List<Map> selectMapResultList(PsiPurchase psiPurchase) {
        return psiPurchaseMapper.selectMapResultList(psiPurchase);
    }

    /**
     * 新增采购订单
     * 
     * @param psiPurchaseVo 采购订单
     * @return 结果
     */
    @Override
    @Transactional
    public int insertPsiPurchase(PsiPurchaseVo psiPurchaseVo)
    {
        List<PsiPurchaseItem> items = psiPurchaseVo.getItems();
        // 需要校验每条明细，商品名称+规格 是否在该租户的商品列表中，如果没有，会影响后续的询价、竞价流程
        // StringBuilder stb = new StringBuilder();
        // int errorNum = 0;
        // String spec = null;
        // for(PsiPurchaseItem item : items){
        //     spec = item.getSpec() == null ? "" : item.getSpec().toLowerCase().trim();
        //     item.setSpec(spec);
        //     item.setGoodsName(item.getGoodsName().trim());
        //     int goodsNum = psiGoodsMapper.selectGoodsCountByNameAndSpec(item.getGoodsName(), item.getSpec(), psiPurchaseVo.getTenantId());
        //     if(goodsNum == 0){
        //         errorNum++;
        //         stb.append(item.getGoodsName() + " " + item.getSpec() + "\n");
        //     }
        // }
        // if(errorNum > 0){
        //     stb.append("以上共" + errorNum + "条商品不在商品库中，请先到商品库中维护。");
        //     throw new ServiceException(stb.toString());
        // }

        SysTenant tenant = sysTenantMapper.selectSysTenantById(psiPurchaseVo.getTenantId());
        psiPurchaseVo.setTenantName(tenant.getTenantName());

        PsiStore psiStore = psiStoreMapper.selectPsiStoreByStoreId(psiPurchaseVo.getStoreId(), psiPurchaseVo.getTenantId());
        psiPurchaseVo.setStoreName(psiStore.getStoreName());

        psiPurchaseVo.setCreateTime(DateUtils.getNowDate());
        psiPurchaseVo.setQuotationMode(tenant.getQuotationMode());
        int i = psiPurchaseMapper.insertPsiPurchase(psiPurchaseVo);

        Long id = psiPurchaseVo.getId();
        // 单条插入
//        for(PsiPurchaseItem item : items){
//            item.setPurchaseId(id);
//            itemMapper.insertPsiPurchaseItem(item);
//        }
        // 批量插入
        itemMapper.batchInsertItem(id, items);
        return i;
    }

    /**
     * 修改采购订单
     * 
     * @param psiPurchaseVo 采购订单
     * @return 结果
     */
    @Override
    @Transactional
    public int updatePsiPurchase(PsiPurchaseVo psiPurchaseVo)
    {
        Long id = psiPurchaseVo.getId();
        PsiPurchase p = psiPurchaseMapper.selectPsiPurchaseById(id, psiPurchaseVo.getTenantId());
        // 没有查询到，或者状态不是未提交，不允许修改
        if(p == null || p.getStatus() != 0){
            throw new ServiceException("采购单未找到，或已提交！");
        }
        List<PsiPurchaseItem> items = psiPurchaseVo.getItems();
        // 需要校验每条明细，商品名称+规格 是否在该租户的商品列表中，如果没有，会影响后续的询价、竞价流程
//        StringBuilder stb = new StringBuilder();
//        int errorNum = 0;
//        String spec = null;
//        for(PsiPurchaseItem item : items){
//            spec = item.getSpec() == null ? "" : item.getSpec().toLowerCase().trim();
//            item.setSpec(spec);
//            item.setGoodsName(item.getGoodsName().trim());
//            int goodsNum = psiGoodsMapper.selectGoodsCountByNameAndSpec(item.getGoodsName(), item.getSpec(), psiPurchaseVo.getTenantId());
//            if(goodsNum == 0){
//                errorNum++;
//                stb.append(item.getGoodsName() + " " + item.getSpec() + "\n");
//            }
//        }
//        if(errorNum > 0){
//            stb.append("以上共" + errorNum + "条商品不在商品库中，请先到商品库中维护。");
//            throw new ServiceException(stb.toString());
//        }

        PsiStore psiStore = psiStoreMapper.selectPsiStoreByStoreId(psiPurchaseVo.getStoreId(), psiPurchaseVo.getTenantId());
        psiPurchaseVo.setStoreName(psiStore.getStoreName());

        psiPurchaseVo.setUpdateTime(DateUtils.getNowDate());
        int i = psiPurchaseMapper.updatePsiPurchase(psiPurchaseVo);

        itemMapper.deletePsiPurchaseItemByPurchaseId(id);

        // 单条插入
//        for(PsiPurchaseItem item : items){
//            item.setPurchaseId(id);
//            itemMapper.insertPsiPurchaseItem(item);
//        }
        // 批量插入
        itemMapper.batchInsertItem(id, items);

        return i;
    }

    /**
     * 修改状态
     * @param psiPurchase
     * @return
     */
    public int updatePsiPurchaseStatus(PsiPurchase psiPurchase){
        if(psiPurchase.getStatus() == 1){
            // 20240111 -------------------采购申请 保存时，不校验商品是否在商品库中， 提交 按钮再校验-------------------------
            // 需要校验每条明细，商品名称+规格 是否在该租户的商品列表中，如果没有，会影响后续的询价、竞价流程
            StringBuilder stb = new StringBuilder();
            List<PsiPurchaseItem> items = itemMapper.selectPsiPurchaseItemByPurchaseId(psiPurchase.getId());
            int errorNum = 0;
            String spec = null;
            for(PsiPurchaseItem item : items){
                spec = item.getSpec() == null ? "" : item.getSpec().toLowerCase().trim();
                item.setSpec(spec);
                item.setGoodsName(item.getGoodsName().trim());
                int goodsNum = psiGoodsMapper.selectGoodsCountByNameAndSpec(item.getGoodsName(), item.getSpec(), psiPurchase.getTenantId());
                if(goodsNum == 0){
                    errorNum++;
                    stb.append(item.getGoodsName() + " " + item.getSpec() + "\n");
                }
            }
            if(errorNum > 0){
                stb.append("以上共" + errorNum + "条商品不在商品库中，请先到商品库中维护。");
                throw new ServiceException(stb.toString());
            }
        }else if(psiPurchase.getStatus() == 0){
//            PsiPurchase psiPurchase1 = psiPurchaseMapper.selectPsiPurchaseById(psiPurchase.getId(), psiPurchase.getTenantId());
//            Integer quotationMode = psiPurchase1.getQuotationMode();
//            if(2 == quotationMode){
//                // 竞价模式 todo 需要确认是否删除竞价单。。。 还是不让用户撤回
//            }

            // 将订单设置为草稿(撤回操作),先查看审批记录表里面是否有数据,有的话则删除记录重新走流程.
            ApprovalRecording approvalRecording = new ApprovalRecording();
            approvalRecording.setTenantId(psiPurchase.getTenantId());
            approvalRecording.setDocumentId(psiPurchase.getId());
            approvalRecording.setDocumentType(3);
            commonUtil.edit(approvalRecording);
        }
        return psiPurchaseMapper.updatePsiPurchase(psiPurchase);
    }

    /**
     * 批量删除采购订单
     * 
     * @param ids 需要删除的采购订单主键
     * @return 结果
     */
    @Override
    public int deletePsiPurchaseByIds(Long[] ids, Long tenantId)
    {
        int i = psiPurchaseMapper.deletePsiPurchaseByIds(ids, tenantId);
        for(Long id : ids){
            itemMapper.deletePsiPurchaseItemByPurchaseId(id);
        }
        return i;
    }

    /**
     * 删除采购订单信息
     * 
     * @param id 采购订单主键
     * @return 结果
     */
    @Override
    public int deletePsiPurchaseById(Long id, Long tenantId)
    {
        int i = psiPurchaseMapper.deletePsiPurchaseById(id, tenantId);
        itemMapper.deletePsiPurchaseItemByPurchaseId(id);
        return i;
    }

    /***
     * 汇总报表目录下采购明细
     * @param psiPurchase
     * @return
     */
    @Override
    public List<PsiPurchaseItem> listAll(PsiPurchase psiPurchase) {
        // 获得所有符合条件的采购单id
        PageUtils.startPage();
        List<PsiPurchaseItem> list = itemMapper.selectPurchaseItem(psiPurchase);
        return list;
    }

    /**
     * 汇总报表，目录下采购查询 导出
     */
    @Override
    public List<PsiPurchaseItem> exportListAll(PsiPurchase psiPurchase) {
        List<PsiPurchaseItem> list = itemMapper.selectPurchaseItem(psiPurchase);
        return list;
    }
}
