package com.ruoyi.psi.service.impl;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.*;
import com.ruoyi.psi.domain.vo.ReceiptExportVO;
import com.ruoyi.psi.domain.vo.SupplyVO;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.service.PsiReceiptService;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @ClassName PsiReceiptServiceImpl
 * @Description
 * @Author fm
 * @Date 2023/12/20 11:03
 * @Version V1.0
 */
@Service
public class PsiReceiptServiceImpl implements PsiReceiptService {

    @Autowired
    private PsiReceiptMapper receiptMapper;

    @Autowired
    private PsiGoodsMapper goodsMapper;

    @Autowired
    private SysTenantMapper tenantMapper;

    @Autowired
    private PsiStorageMapper psiStorageMapper;

    @Autowired
    private PsiSupplyItemMapper supplyItemMapper;

    @Autowired
    private PsiSupplyMapper supplyMapper;

    @Autowired
    private PsiStoreMapper psiStoreMapper;

    private static final Logger log = LoggerFactory.getLogger(PsiReceiptServiceImpl.class);
    // 收货
    @Override
    @Transactional
    public int addReceipt(SupplyVO supplyVO) {
        try {
            List<PsiSupplyItem> supplyItems = supplyVO.getSupplyItems();
            // 实际金额
            BigDecimal actualAmount = new BigDecimal("0.00").setScale(2, RoundingMode.HALF_UP);
            // 实际数量
            BigDecimal actualQuantity = null;
            BigDecimal priceInput = null;
            BigDecimal amount = null;
            for (PsiSupplyItem supplyItem : supplyItems) {
                supplyItem.setSpec(supplyItem.getSpec() == null ? "" : supplyItem.getSpec());
                actualQuantity = supplyItem.getActualQuantity();
                priceInput = supplyItem.getPriceInput();
                amount = actualQuantity.multiply(priceInput).setScale(2, RoundingMode.HALF_UP);
                supplyItem.setAmount(amount);
                actualAmount = actualAmount.add(amount);
            }
            // 先更新主表信息
            // 设置为已收货
            supplyVO.setStatus(1);
            supplyVO.setCreateTime(DateUtils.getNowDate());
            supplyVO.setActualAmount(actualAmount);
            // 采购录入
            supplyVO.setDeliveryMethod(1);
            receiptMapper.updatePsiSupply(supplyVO);
            Long supplyId = supplyVO.getId();
            // 删除关联表数据
            receiptMapper.deleteItem(supplyId);
            receiptMapper.insertSupplyItem(supplyItems,supplyVO.getId());
            return 1;
        } catch (Exception e) {
            log.error("===收货失败===",e);
            throw new ServiceException("收货失败");
        }
    }

    // 已收货订单
    @Override
    public List<PsiSupply> selectList(PsiSupply psiSupply) {

        return receiptMapper.selectList(psiSupply);
    }

    // 已收货订单详细
    @Override
    public SupplyVO selectPsiSupplyById(Long id, Long tenantId) {
        PsiSupply supply = receiptMapper.selectPsiSupplyById(id, tenantId);
        SupplyVO vo = new SupplyVO();
        BeanUtils.copyProperties(supply,vo);
        // 查询详情
        List<PsiSupplyItem> items = receiptMapper.selectSupplyItem(id);
        vo.setSupplyItems(items);
        return vo;
    }

    // 手动收货
    @Override
    public int manuallyReceipt(SupplyVO supplyVO) {
        if(Objects.isNull(supplyVO)){
            throw new ServiceException("===供货单不能为空===");
        }
        SysTenant sysTenant = tenantMapper.selectSysTenantById(supplyVO.getTenantId());
        SysUser user = SecurityUtils.getLoginUser().getUser();
        supplyVO.setCreateUser(user.getUserId());
        supplyVO.setTenantId(user.getTenantId());
        supplyVO.setCreateUsername(user.getUserName());
        supplyVO.setTenantName(sysTenant.getTenantName());
        supplyVO.setCreateTime(DateUtils.getNowDate());

        PsiStore psiStore = psiStoreMapper.selectPsiStoreByStoreId(supplyVO.getStoreId(), user.getTenantId());
        supplyVO.setStoreName(psiStore.getStoreName());
        // 已收货
        supplyVO.setStatus(1);
        // 手动收货的订单   默认采购单id为1
        supplyVO.setPurchaseId(1L);
        List<PsiSupplyItem> supplyItems = supplyVO.getSupplyItems();
        if(CollectionUtils.isEmpty(supplyItems)){
            throw new ServiceException("===录入收货单商品不存在===");
        }
        StringBuilder sb = new StringBuilder();
        boolean isOk = true;
        // 插入主表信息

        Long supplierId = supplyVO.getSupplierId();
        PsiGoods goods = null;
        BigDecimal totalAmount = new BigDecimal("0");
        for (PsiSupplyItem supplyItem : supplyItems) {
            goods = new PsiGoods();
            goods.setSupplierId(supplierId);
            goods.setTenantId(user.getTenantId());
            //BeanUtils.copyProperties(supplyItem,goods);
            String spec = supplyItem.getSpec() == null ? "" : supplyItem.getSpec().toLowerCase().trim();
            goods.setSpec(spec);
            supplyItem.setSpec(spec);
            goods.setGoodsName(supplyItem.getGoodsName().trim());
            goods.setUnit(supplyItem.getUnit());
            supplyItem.setGoodsName(supplyItem.getGoodsName().trim());
            PsiGoods oneGoods = goodsMapper.selectOneGoods(goods);
            if(oneGoods == null){
                isOk = false;
                sb.append(goods.getGoodsName() + " " + goods.getSpec() + "; ");
                continue;
            }
            supplyItem.setGoodsId(oneGoods.getId());
            supplyItem.setPrice(oneGoods.getPrice());
            // 数量
            BigDecimal actualQuantity = supplyItem.getActualQuantity().setScale(2, RoundingMode.HALF_UP);
            supplyItem.setQuantity(actualQuantity);
            // 执行单价
            BigDecimal priceInput = supplyItem.getPriceInput().setScale(2, RoundingMode.HALF_UP);
            // 金额
            BigDecimal amount = actualQuantity.multiply(priceInput).setScale(2, RoundingMode.HALF_UP);
            supplyItem.setAmount(amount);
            supplyItem.setSpec(spec);
            totalAmount = totalAmount.add(amount);
        }
        if(!isOk){
            log.error("手动收货失败!");
            throw new ServiceException("该供应商不能提供如下商品：" + sb.toString() + "。请检查商品名称、规格是否和商品库中一致，特别注意是否有空格、.等多余字符。");
        }
        supplyVO.setTotalAmount(totalAmount);
        supplyVO.setActualAmount(totalAmount);
        // 手动收货
        supplyVO.setDeliveryMethod(2);
        receiptMapper.insertPsiSupply(supplyVO);
        Long supplyId = supplyVO.getId();
        // 批量插入
        receiptMapper.insertSupplyItem(supplyItems,supplyId);
        return 1;
//        }catch (Exception e){
//            log.error("手动收货失败!",e);
//            throw new ServiceException("该供应商不能提供完整的商品,请确认!");
//        }

    }

    // 已收货订单不分页
    @Override
    public List<PsiSupply> listAll(PsiSupply psiSupply) {
        return receiptMapper.selectAllList(psiSupply);
    }

    // 更改已收货订单状态
    @Override
    public int updateReceiptStatus(PsiSupply supply) {
        return receiptMapper.updatePsiSupply(supply);
    }

    // 修改暂存的数据
    @Override
    public int updateReceipt(SupplyVO supplyVO) {
        List<PsiSupplyItem> supplyItems = supplyVO.getSupplyItems();
        // 实际金额
        BigDecimal actualAmount = new BigDecimal("0.00").setScale(2, RoundingMode.HALF_UP);
        // 实际数量
        BigDecimal actualQuantity = null;
        BigDecimal priceInput = null;
        BigDecimal amount = null;
        PsiGoods goods = null;
        boolean isOk = true;
        StringBuilder stringBuilder = new StringBuilder();
        for (PsiSupplyItem supplyItem : supplyItems) {
            // 检查商品是否是这个供应商可以提供的
            goods = new PsiGoods();
            goods.setSupplierId(supplyVO.getSupplierId());
            goods.setTenantId(supplyVO.getTenantId());
            //BeanUtils.copyProperties(supplyItem,goods);
            String spec = supplyItem.getSpec() == null ? "" : supplyItem.getSpec().toLowerCase().trim();
            goods.setSpec(spec);
            goods.setGoodsName(supplyItem.getGoodsName().trim());
            goods.setUnit(supplyItem.getUnit());
            PsiGoods oneGoods = goodsMapper.selectOneGoods(goods);
            if(null == oneGoods){
                isOk = false;
                stringBuilder.append(goods.getGoodsName() + " " + goods.getSpec() + "; ");
                continue;
            }
            supplyItem.setSpec(spec);
            supplyItem.setGoodsId(oneGoods.getId());

            actualQuantity = supplyItem.getActualQuantity();
            supplyItem.setQuantity(actualQuantity);
            priceInput = supplyItem.getPriceInput();
            amount = actualQuantity.multiply(priceInput).setScale(2, RoundingMode.HALF_UP);
            supplyItem.setAmount(amount);
            actualAmount = actualAmount.add(amount);
        }
        if(!isOk){
            log.error("修改失败!");
            throw new ServiceException("该供应商不能提供如下商品：" + stringBuilder.toString() + "。请检查商品名称、规格是否和商品库中一致，特别注意是否有空格、.等多余字符。");
        }
        // 先更新主表信息
        // 设置为已收货
        supplyVO.setActualAmount(actualAmount);
        receiptMapper.updatePsiSupply(supplyVO);
        Long supplyId = supplyVO.getId();
        // 删除关联表数据
        receiptMapper.deleteItem(supplyId);
        receiptMapper.insertSupplyItem(supplyItems,supplyVO.getId());
        return 1;
    }

    @Override
    @Transactional
    public int revokeReceipt(Long id){
        // 检查是否有入库单
        int rukuCount = psiStorageMapper.selectBySupplyId(id);
        if(rukuCount > 0){
            throw new ServiceException("已入库，请先撤回入库单!");
        }

        SysUser user = SecurityUtils.getLoginUser().getUser();
        PsiSupply psiSupply = supplyMapper.selectPsiSupplyById(id, user.getTenantId());
        Integer deliveryMethod = psiSupply.getDeliveryMethod();
        // 如果是手动录入的，直接删除明细表和主表
        if(deliveryMethod == 2){
            supplyMapper.deletePsiSupplyById(id);
            supplyItemMapper.deletePsiSupplyItemBySupplyId(id);
        } else {
            // 如果是采购单过来的，修改状态为未收货，同时情况明细表中的生产日期
            supplyMapper.revokeSupplyToOrdering(id);
            supplyItemMapper.revokeSupplyItemToOrdering(id);
        }
        return 1;
    }

    @Override
    public Map<String, Object> getTotalAmount(PsiSupply psiSupply) {
        Map<String, Object> totalMap = receiptMapper.getTotalAmount(psiSupply);
        return totalMap;
    }

    @Override
    public List<ReceiptExportVO> exportItem(PsiSupply psiSupply) {
        List<PsiSupplyItem> supplyItems = receiptMapper.selectSupplyItem(psiSupply.getId());
        List<ReceiptExportVO> result = new ArrayList<>();
        if (!CollectionUtils.isEmpty(supplyItems)) {
            for (PsiSupplyItem supplyItem : supplyItems) {
                ReceiptExportVO vo = new ReceiptExportVO();
                BeanUtils.copyProperties(supplyItem,vo);
                result.add(vo);
            }
        }
        return result;
    }
}