package com.ruoyi.psi.service.impl;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.constant.ApprovalConstants;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.*;
import com.ruoyi.psi.domain.vo.*;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.service.IPsiStorageService;
import com.ruoyi.psi.util.CommonUtil;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 入库单Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-22
 */
@Service
public class PsiStorageServiceImpl implements IPsiStorageService 
{
    @Autowired
    private PsiStorageMapper psiStorageMapper;

    @Autowired
    private SysTenantMapper tenantMapper;

    @Autowired
    private PsiSrorageItemMapper srorageItemMapper;

    @Autowired
    private PsiSupplyItemMapper supplyItemMapper;

    @Autowired
    private PsiSupplyMapper supplyMapper;

    @Autowired
    private PsiSummaryMapper summaryMapper;

    @Autowired
    private PsiSummaryItemMapper summaryItemMapper;

    @Autowired
    private PsiSupplierMapper supplierMapper;

    @Autowired
    private PsiStoreMapper psiStoreMapper;

    @Autowired
    private CommonUtil commonUtil;

    /**
     * 查询入库单
     * 
     * @param id 入库单主键
     * @return 入库单
     */
    @Override
    public StorageVO selectPsiStorageById(Long id)
    {
        PsiStorage storage = psiStorageMapper.selectPsiStorageById(id);
        StorageVO vo = new StorageVO();
        BeanUtils.copyProperties(storage,vo);
        List<PsiSrorageItem> items = srorageItemMapper.selectPsiStorageItemByStorageId(vo.getId());

        // 查询入库流程详细
        ApprovalRecording approvalRecording = new ApprovalRecording();
        approvalRecording.setDocumentId(id);
        approvalRecording.setDocumentType(ApprovalConstants.RUKU);
        approvalRecording.setzTenantId(storage.getTenantId());
        List<ApprovalRecording> inApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);

        // 查询出库流程详细
        approvalRecording.setDocumentType(ApprovalConstants.CHUKU);
        List<ApprovalRecording> outApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);

        // 销货单流程详细
        approvalRecording.setDocumentType(ApprovalConstants.XIAOHUODAN);
        List<ApprovalRecording> salesApprovalRecordingList = commonUtil.getRecordingList(approvalRecording);

        vo.setItems(items);
        vo.setInApprovalRecordingList(inApprovalRecordingList);
        vo.setOutApprovalRecordingList(outApprovalRecordingList);
        vo.setSalesApprovalRecordingList(salesApprovalRecordingList);
        return vo;
    }

    /**
     * 查询入库单列表
     * 
     * @param psiStorage 入库单
     * @return 入库单
     */
    @Override
    public List<PsiStorage> selectPsiStorageList(StorageVO psiStorage)
    {
        List tenantIds = psiStorage.getStoreIds();
        return psiStorageMapper.selectPsiStorageList(psiStorage,tenantIds);
    }

    /**
     * 新增入库单
     * 
     * @param supplyVO 入库单
     * @return 结果
     */
    @Transactional
    @Override
    public int insertPsiStorage(SupplyVO supplyVO) {
        List<PsiSupplyItem> supplyItems = supplyVO.getSupplyItems();
        // 校验参数是否异常
        PsiSupply psiSupply = supplyMapper.selectPsiSupplyById(supplyVO.getId(), supplyVO.getTenantId());
        // 实际收货金额
        BigDecimal actualAmount = psiSupply.getActualAmount() == null ? BigDecimal.ZERO : psiSupply.getActualAmount();
        // 已经入库金额
        BigDecimal storageAmount = psiSupply.getStorageAmount() == null ? BigDecimal.ZERO : psiSupply.getStorageAmount();
        // 本次入库金额
        BigDecimal amount = BigDecimal.ZERO; //setScale(2, RoundingMode.HALF_UP)
        BigDecimal priceInput = null;
        BigDecimal thisQuantity = null;
        for (PsiSupplyItem supplyItem : supplyItems) {
            priceInput = supplyItem.getPriceInput().setScale(2, RoundingMode.HALF_UP);
            // 此次入库数量
            thisQuantity = supplyItem.getThisQuantity() == null ? BigDecimal.ZERO : supplyItem.getThisQuantity().setScale(2, RoundingMode.HALF_UP);
            amount = amount.add(priceInput.multiply(thisQuantity).setScale(2, RoundingMode.HALF_UP));
        }
        // amount = amount.setScale(2, RoundingMode.HALF_UP);
        if((amount.add(storageAmount)).compareTo(actualAmount) > 0){
            throw new ServiceException("入库失败!入库金额大于实际金额!");
        }
        List<PsiSrorageItem> srorageItems = new ArrayList<>();
        PsiSrorageItem srorageItem = null;
        for (PsiSupplyItem supplyItem : supplyItems) {
            srorageItem = new PsiSrorageItem();
            BeanUtils.copyProperties(supplyItem,srorageItem,"id");
            srorageItem.setSupplyItemId(supplyItem.getId());
            // 默认未出库
            srorageItem.setStatus(0);
            srorageItem.setTenantId(supplyVO.getTenantId());
            // 批次
            Integer integer = srorageItemMapper.selectBatchByGoodsId(srorageItem.getGoodsId(),supplyVO.getTenantId());
            srorageItem.setBatch(integer + 1L);
            // srorageItem.setThisQuantity(supplyItem.getThisQuantity());
            srorageItem.setAmount(
                    srorageItem.getThisQuantity() == null ? BigDecimal.ZERO : srorageItem.getThisQuantity().setScale(2, RoundingMode.HALF_UP)
                    .multiply(srorageItem.getPriceInput() == null ? BigDecimal.ZERO : srorageItem.getPriceInput().setScale(2, RoundingMode.HALF_UP)
                            .setScale(2, RoundingMode.HALF_UP)));
            srorageItems.add(srorageItem);
            // 更新订货单入库数量
            thisQuantity = supplyItem.getThisQuantity() == null ? new BigDecimal("0") : supplyItem.getThisQuantity();
            supplyItem.setStorageQuantity(thisQuantity);
            supplyItemMapper.updateSupplyItemStorageQuantity(supplyItem);
        }
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 更新订货单 入库总金额
        psiSupply.setStorageAmount(amount);
        supplyMapper.updateSupplyStorageAmount(psiSupply);
        PsiStorage storage = new PsiStorage();
        // 新增一条入库单
        storage.setSupplyId(supplyVO.getId());
        storage.setSupplierId(supplyVO.getSupplierId());
        storage.setSupplierName(supplyVO.getSupplierName());
        storage.setCreateUser(user.getUserId());
        storage.setCreateUserName(user.getUserName());
        storage.setTenantId(user.getTenantId());
        SysTenant sysTenant = tenantMapper.selectSysTenantById(user.getTenantId());
        storage.setTenantName(sysTenant.getTenantName());
        storage.setAmount(amount);
        storage.setCreateTime(supplyVO.getCreateTime());
        storage.setUpdateTime(DateUtils.getNowDate());
        storage.setRemark(supplyVO.getRemark());//20231229新增备注摘要
        storage.setBusinessTime(supplyVO.getBusinessTime());
        // 默认未出库
        storage.setStatus(0);
        // 默认未审核
        storage.setAudit(0);
        storage.setStoreId(supplyVO.getStoreId());
        storage.setStoreName(supplyVO.getStoreName());
        storage.setSalesAuditStatus(0);
        // 插入主表数据
        psiStorageMapper.insertPsiStorage(storage);
        Long storageId = storage.getId();
        srorageItemMapper.insertSrorageItem(srorageItems, storageId);
        return 1;
    }

    private void verification(List<PsiSupplyItem> supplyItems) {
        for (PsiSupplyItem supplyItem : supplyItems) {
            // 数据库里面的详情信息
            PsiSupplyItem mySupply = supplyItemMapper.selectPsiSupplyItemById(supplyItem.getId());
            // 已入库数量
            BigDecimal mySupplyStorageQuantity = mySupply.getStorageQuantity() == null ? new BigDecimal("0") : mySupply.getStorageQuantity();
            // 入库数量
            BigDecimal storageQuantity = supplyItem.getStorageQuantity();
            // 实际收货数量
            BigDecimal actualQuantity = mySupply.getActualQuantity();
            if(mySupplyStorageQuantity.add(storageQuantity).compareTo(actualQuantity) > 0){
                throw new ServiceException("===入库数量大于实际收货数量请检查===");
            }
        }
    }

    /**
     *  修改
     * 
     * @param psiStorage 入库单
     * @return 结果
     */
    @Override
    public int updatePsiStorage(PsiStorage psiStorage)
    {
        psiStorage.setUpdateTime(DateUtils.getNowDate());
        return psiStorageMapper.updatePsiStorage(psiStorage);
    }

    /**
     * 批量删除入库单
     * 
     * @param ids 需要删除的入库单主键
     * @return 结果
     */
    @Override
    public int deletePsiStorageByIds(Long[] ids)
    {
        return psiStorageMapper.deletePsiStorageByIds(ids);
    }

    /**
     * 删除入库单信息
     * 
     * @param id 入库单主键
     * @return 结果
     */
    @Override
    public int deletePsiStorageById(Long id)
    {
        return psiStorageMapper.deletePsiStorageById(id);
    }

    /**
     * @description
     * 入库汇总
     * 按照供应商、结算部门（即餐厅）、月度、同名称同规格同单价要进行累加汇总，形成月度商品入库汇总表
     *
     * @param param
     * @return java.util.List<java.util.Map>
     * @author fm
     * @date 2023/12/25 9:22
     */
    @Override
    public Map storageList(Map<String,Object> param) {
        if(null == param.get("supplierId")){
            throw new ServiceException("请选择指定供应商!");
        }
        Long supplierId = Long.valueOf(param.get("supplierId").toString());
        // 月份
        String date = param.get("date").toString();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime startDate = LocalDateTime.parse(date,formatter);
        // 该月份最后一天的时间
        LocalDateTime lastDay  = startDate.with(TemporalAdjusters.lastDayOfMonth()).withHour(23).withMinute(59).withSecond(59);
        param.put("supplierId",supplierId);
        param.put("startDate",date);
        param.put("endDate", lastDay.format(formatter));
        List<Map> list = psiStorageMapper.storageList(param);
        BigDecimal totalAmount = BigDecimal.ZERO;
        if(!CollectionUtils.isEmpty(list)){
            BigDecimal amount = null;
            for (Map map : list) {
                amount = new BigDecimal(map.get("goodsAmount").toString());
                totalAmount = totalAmount.add(amount);
            }

        }
        Map rMap = new HashMap();
        rMap.put("list",list);
        rMap.put("totalAmount",totalAmount);
        DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyyMM");
        rMap.put("summaryDate",startDate.format(dtf2));
        return rMap;
    }

    /**
     * 新增汇总信息*
     * @param param
     * @return
     */
    @Override
    @Transactional
    public int addSummary(Map<String, Object> param) {
        Long supplierId = Long.valueOf(param.get("supplierId").toString()) ;
        Long tenantId = (Long) param.get("tenantId");
        String remark = param.get("remark") == null ? "" : (String) param.get("remark");
        String timePeriod = param.get("timePeriod") == null ? "" : (String) param.get("timePeriod");
        String[] split = timePeriod.split(",");
        SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
        Date start = null;
        Date end = null;
        try {
            start = ft.parse(split[0]);
            end = ft.parse(split[1]);
        }catch (Exception e){
            throw new ServiceException("格式转换错误");
        }
        // type 1:入库 2:出库
        Integer type = Integer.valueOf(param.get("type").toString());
        // String summaryDate = (String)param.get("summaryDate");
        // String summaryDateEnd = (String)param.get("summaryDateEnd");
        SysTenant sysTenant = tenantMapper.selectSysTenantById(tenantId);
        PsiSupplier psiSupplier = supplierMapper.selectPsiSupplierById(supplierId);

        SysUser user = SecurityUtils.getLoginUser().getUser();
        PsiSummary summary = new PsiSummary();
        summary.setSupplierId(supplierId);
        summary.setSupplierName(psiSupplier.getSupplierName());
        summary.setTenantId(tenantId);
        summary.setTenantName(sysTenant.getTenantName());
        summary.setCreateUser(user.getUserId());
        summary.setCreateUserName(user.getUserName());
        summary.setCreateTime(DateUtils.getNowDate());
        summary.setRemark(remark);
        summary.setType(type);
        summary.setTimePeriodStart(start);
        summary.setTimePeriodEnd(end);

        Long storeId = Long.valueOf(param.get("storeId").toString());
        summary.setStoreId(storeId);
        PsiStore psiStore = psiStoreMapper.selectPsiStoreByStoreId(storeId, tenantId);
        summary.setStoreName(psiStore.getStoreName());

        List<Map<String,Object>> srorageItems =  (List<Map<String,Object>>)param.get("summaryItems");
        List<Long> ids = (List<Long>)param.get("ids");

        BigDecimal totalAmount = new BigDecimal("0.00");
        PsiSummaryItem item = null;
        List<PsiSummaryItem> summaryItems = new ArrayList<>();
        for (Map<String,Object> srorageItem : srorageItems) {
            item = new PsiSummaryItem();
            item.setGoodsId(Long.valueOf(srorageItem.get("goodsId").toString()));
            item.setGoodsName(srorageItem.get("goodsName").toString());
            item.setPriceInput(new BigDecimal(srorageItem.get("priceInput").toString()));
            item.setSpec(srorageItem.get("spec").toString());
            item.setUnit(srorageItem.get("unit").toString());
            item.setThisQuantity(new BigDecimal(srorageItem.get("thisQuantity").toString()));
            BigDecimal goodsAmount = srorageItem.get("amount") == null ? BigDecimal.ZERO : new BigDecimal(srorageItem.get("amount").toString());
            totalAmount = totalAmount.add(goodsAmount);
            item.setGoodsAmount(goodsAmount);
            // item.setSummaryId(summaryId);
            summaryItems.add(item);
        }
        summary.setTotalAmount(totalAmount);
        summaryMapper.insertPsiSummary(summary);
        Long summaryId = summary.getId();
        summaryItemMapper.insertPsiSummaryItemByList(summaryItems,summaryId);
        // 插入中间表
        summaryMapper.insertSummaryStorageRef(summaryId,ids);
        return 1;
    }

    /**
     * 出库*
     * @param psiStorage
     * @return
     */
    @Override
    @Transactional
    public int outbound(PsiStorage psiStorage) {
        PsiStorage storage = psiStorageMapper.selectPsiStorageById(psiStorage.getId());
        psiStorage.setUpdateTime(DateUtils.getNowDate());
        psiStorage.setAuditTime(DateUtils.getNowDate());
        psiStorage.setStatus(1);
        // psiStorage.setAudit(1);
        SysUser user = SecurityUtils.getLoginUser().getUser();
        psiStorage.setAuditUser(user.getUserId());
        psiStorage.setAuditUserName(user.getUserName());
        psiStorage.setBusinessOutTime(storage.getBusinessTime());
        psiStorageMapper.updatePsiStorage(psiStorage);
        List<PsiSrorageItem> items = srorageItemMapper.selectsrorageItemByStorageId(psiStorage.getId());
        if(CollectionUtils.isEmpty(items)){
            return 1;
        }
        for (PsiSrorageItem item : items) {
            // 已出库
            item.setStatus(1);
            // 批次
           // Integer flag = srorageItemMapper.selectBatchByGoodsId(item.getGoodsId());
           //  item.setBatch(flag + 1L);
        }
        // 批量更新
        srorageItemMapper.updateByItems(items);
        return 1;
    }

    /**
     * 导出入库单详情*
     * @param psiSrorageItem
     * @return
     */
    @Override
    public List<StorageItemExportVO> exportStorageItem(PsiSrorageItem psiSrorageItem) {
        List<StorageItemExportVO> items = srorageItemMapper.exportStorageItem(psiSrorageItem.getId());
        return items;
    }

    /**
     * 获得入库单详情 根据id集合
     * @param ids
     * @return
     */
    @Override
    public List<PsiSrorageItem> getInfoByIds(List<Long> ids){
        // 获得主表信息
        if(CollectionUtils.isEmpty(ids)){
            return new ArrayList<>();
        }
        // 获得详细数据
        List<PsiSrorageItem> infoByIds = srorageItemMapper.getInfoByIds(ids);
        return infoByIds;
    }

    /**
     *  汇总报表，目录下入库明细
     */
    @Override
    public List<PsiSrorageItem> listAll(StorageVO psiStorage) {
        PageUtils.startPage();
        List<PsiSrorageItem> infoList = srorageItemMapper.getInfo(psiStorage);
        return infoList;
    }

    /**
     * 汇总报表，目录下入库明细 导出
     * @param psiStorage
     * @return
     */
    @Override
    public List<PsiSrorageItemVO> exportQueryListAll(StorageVO psiStorage) {
        List<PsiSrorageItemVO> infoByIds = srorageItemMapper.exportQueryListAll(psiStorage);
        return infoByIds;
    }

    @Override
    public List<StorageVO> selectPsiStorageVoList(StorageVO psiStorage) {
        return psiStorageMapper.selectPsiStorageVoList(psiStorage);
    }

    @Override
    @Transactional
    public int revokeStorage(Long id) {
        // 判断是否已经出库，已出库，提示需要先撤回出库单
        PsiStorage psiStorage = psiStorageMapper.selectPsiStorageById(id);
        if(psiStorage.getStatus() == 1){
            throw new ServiceException("已出库，请先撤回出库单!");
        }
        // 找到收货单id
        Long supplyId = psiStorage.getSupplyId();
        List<Long> storageIds = psiStoreMapper.selectStorageIdBySupplyId(psiStorage.getSupplyId(),psiStorage.getTenantId());
        // 删除所有的入库单明细
        srorageItemMapper.deleteItemBySupplyId(supplyId);
        // 根据收货单id，删除所有的入库单（一个收货单可能分多次入库）
        psiStorageMapper.deletePsiStorageBySupplyId(supplyId);
        // 修改收货单状态及数据
        supplyMapper.updateSupplyStatusById(supplyId);
        supplyItemMapper.resetSupplyItemStorageQuantity(supplyId);

        // 删除入库单审批记录(一个收货单可能有多个入库单)
        ApprovalRecording approvalRecording = null;
        if(!CollectionUtils.isEmpty(storageIds)){
            for (Long storageId : storageIds) {
                approvalRecording = new ApprovalRecording();
                approvalRecording.setTenantId(psiStorage.getTenantId());
                approvalRecording.setDocumentId(storageId);
                // 删除入库单审批记录
                approvalRecording.setDocumentType(ApprovalConstants.RUKU);
                commonUtil.edit(approvalRecording);
                // 删除出库单审批记录
                approvalRecording.setDocumentType(ApprovalConstants.CHUKU);
                commonUtil.edit(approvalRecording);
                // 删除销货单审批记录
                approvalRecording.setDocumentType(ApprovalConstants.XIAOHUODAN);
                commonUtil.edit(approvalRecording);
            }
        }
        return 1;
    }

    /**
     * 获得列表总金额
     * @param psiStorage
     * @return
     */
    @Override
    public Map<String, Object> getTotalAmount(StorageVO psiStorage) {
        Map<String,Object> totalAmountMap = psiStorageMapper.getTotalAmount(psiStorage);
        return totalAmountMap;
    }

    /**
     * 批量导出入库单
     * @param psiSrorageItem
     * @return
     */
    @Override
    public List<StorageItemExportVO> exportStorageItems(PsiSrorageItem psiSrorageItem) {
        List<StorageItemExportVO> items = srorageItemMapper.exportStorageItems(psiSrorageItem);
        return items;
    }

    /**
     * 库存数量列表
     * @param storageNumberVO
     * @return
     */
    @Override
    public List<StorageNumberVO> listInventoryQuantity(StorageNumberVO storageNumberVO) {
        List<StorageNumberVO> list = psiStorageMapper.listInventoryQuantity(storageNumberVO);
        return list;
    }
}
