package com.ruoyi.psi.service.impl;

import java.util.List;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.psi.mapper.PsiStoreMapper;
import com.ruoyi.psi.domain.PsiStore;
import com.ruoyi.psi.service.IPsiStoreService;

/**
 * 仓库Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-01-03
 */
@Service
public class PsiStoreServiceImpl implements IPsiStoreService 
{
    @Autowired
    private PsiStoreMapper psiStoreMapper;

    /**
     * 查询仓库
     * 
     * @param storeId 仓库主键
     * @return 仓库
     */
    @Override
    public PsiStore selectPsiStoreByStoreId(Long storeId)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Long tentantId = user.getTenantId();
        return psiStoreMapper.selectPsiStoreByStoreId(storeId, tentantId);
    }

    /**
     * 查询仓库列表
     * 
     * @param psiStore 仓库
     * @return 仓库
     */
    @Override
    public List<PsiStore> selectPsiStoreList(PsiStore psiStore)
    {
        return psiStoreMapper.selectPsiStoreList(psiStore);
    }

    /**
     * 新增仓库
     * 
     * @param psiStore 仓库
     * @return 结果
     */
    @Override
    public int insertPsiStore(PsiStore psiStore)
    {
        return psiStoreMapper.insertPsiStore(psiStore);
    }

    /**
     * 修改仓库
     * 
     * @param psiStore 仓库
     * @return 结果
     */
    @Override
    public int updatePsiStore(PsiStore psiStore)
    {
        return psiStoreMapper.updatePsiStore(psiStore);
    }

    /**
     * 批量删除仓库
     * 
     * @param storeIds 需要删除的仓库主键
     * @return 结果
     */
    @Override
    public int deletePsiStoreByStoreIds(Long[] storeIds)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Long tentantId = user.getTenantId();
        return psiStoreMapper.deletePsiStoreByStoreIds(storeIds, tentantId);
    }

    /**
     * 删除仓库信息
     * 
     * @param storeId 仓库主键
     * @return 结果
     */
    @Override
    public int deletePsiStoreByStoreId(Long storeId)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Long tentantId = user.getTenantId();
        return psiStoreMapper.deletePsiStoreByStoreId(storeId, tentantId);
    }
}
