package com.ruoyi.psi.service.impl;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.psi.domain.PsiSupplier;
import com.ruoyi.psi.domain.vo.PsiSupplierVO;
import com.ruoyi.psi.domain.vo.SupplierTenantVO;
import com.ruoyi.psi.mapper.PsiSupplierMapper;
import com.ruoyi.psi.service.IPsiSupplierService;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.mapper.*;
import com.ruoyi.system.service.ISysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Validator;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 供应商Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
@Service
public class PsiSupplierServiceImpl implements IPsiSupplierService 
{
    @Autowired
    private PsiSupplierMapper psiSupplierMapper;

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ServerConfig serverConfig;

    private static final Logger log = LoggerFactory.getLogger(PsiSupplierServiceImpl.class);

    @Autowired
    protected Validator validator;

    @Autowired
    private ISysUserService userService;

    /**
     * 查询供应商
     * 
     * @param id 供应商主键
     * @return 供应商
     */
    @Override
    public PsiSupplier selectPsiSupplierById(Long id)
    {
        return psiSupplierMapper.selectPsiSupplierById(id);
    }

    /**
     * 查询供应商列表
     * 
     * @param psiSupplier 供应商
     * @return 供应商
     */
    @Override
    public List<PsiSupplierVO> selectPsiSupplierList(PsiSupplier psiSupplier)
    {
        // 拼接url
        // String url = serverConfig.getUrl() ;
        // Long tenantId = SecurityUtils.getLoginUser().getUser().getTenantId();
        List<PsiSupplierVO> psiSupplierList = psiSupplierMapper.selectPsiSupplierList(psiSupplier);

        return psiSupplierList;
    }

    /**
     * 租户登录查询关联的供应商列表
     *
     * @param psiSupplier 供应商
     * @return 供应商集合
     */
    public List<PsiSupplier> selectPsiSupplierListByTenant(PsiSupplier psiSupplier, Long tenantId){
        List<PsiSupplier> psiSupplierList = psiSupplierMapper.selectPsiSupplierListByTenant(psiSupplier, tenantId);
        return psiSupplierList;
    }

    /**
     * 新增供应商
     * 
     * @param
     * @return 结果
     */
    @Override
    @Transactional
    public int insertPsiSupplier(PsiSupplierVO vo)
    {
        // 重复校验
        PsiSupplier p = psiSupplierMapper.selectByName(vo.getSupplierName());
        if(!Objects.isNull(p)){
            log.error("===============该供应商已经存在{}===============",vo.getSupplierName());
            throw new ServiceException("该供应商已经存在!!!");
        }
        SysUser info = userMapper.checkUserNameUnique(vo.getContactMobile());
        if (info != null){
            throw new ServiceException("新增失败，该手机号码已使用，请修改或联系管理员！");
        }
        // 当前登录人
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        vo.setCreateTime(DateUtils.getNowDate());
        vo.setCreateBy(userId.toString());
        int result = psiSupplierMapper.insertPsiSupplier(vo);
        Long supplierId = vo.getId();
        List<SupplierTenantVO> tenantIds = vo.getTenantIds();
        if(null != tenantIds && tenantIds.size() > 0){
            for (SupplierTenantVO voI : tenantIds) {
                SupplierTenantVO tvo = new SupplierTenantVO();
                tvo.setSupplierId(supplierId);
                tvo.setTenantId(voI.getTenantId());
                psiSupplierMapper.insertPsiSupplierTenantRef(tvo);
            }
        }
        // TODO 生成供应商账号密码
        // 为该租户生成sys_role、sys_dept、sys_post相关数据，保证该用户可以以租户管理员身份登录后台系统
        // 4. sys_user
        SysUser user = new SysUser();
//        user.setDeptId(dept.getDeptId());
        user.setUserName(vo.getContactMobile());
        user.setNickName(vo.getContactName());
        user.setUserType("02");
        user.setEmail(vo.getEmail());
        user.setPhonenumber(vo.getContactMobile());
        user.setPassword("123456");
        user.setStatus("0");
//        user.setPostIds(new Long[]{post.getPostId()});
        // 新创建的租户管理员，默认给 6 供应商管理员
        user.setRoleIds(new Long[]{6l});
        user.setRemark("");
        user.setIsAdmin("0");//租户创建用户默认为超级管理员
        user.setTenantId(vo.getId());
        user.setCreateBy(SecurityUtils.getLoginUser().getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        userMapper.insertUser(user);

        // 6. sys_user_role 1条数据
        SysUserRole userRole = new SysUserRole();
        userRole.setUserId(user.getUserId());
        // 新创建的租户管理员，默认给 6 供应商管理员
        userRole.setRoleId(6l);
        userRoleMapper.insertUserRole(userRole);
        return result;
    }

    /**
     * 修改供应商
     * 
     * @param
     * @return 结果
     */
    @Override
    public int updatePsiSupplier(PsiSupplierVO vo)
    {
        // 重复校验
        PsiSupplier p = psiSupplierMapper.selectByName(vo.getSupplierName());
        if(!Objects.isNull(p) && !p.getId().equals(vo.getId())){
            log.error("===============该供应商已经存在{}===============",vo.getSupplierName());
            throw new ServiceException("该供应商已经存在!!!");
        }
        vo.setUpdateTime(DateUtils.getNowDate());
        // 当前登录人
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        vo.setUpdateBy(userId.toString());
        int result = psiSupplierMapper.updatePsiSupplier(vo);
        List<SupplierTenantVO> tenantIds = vo.getTenantIds();
        if(null != tenantIds){
            // 删除关联管理
            psiSupplierMapper.deletePsiSupplierTenantRef(vo.getId());
            SupplierTenantVO tvo = null;
            if(tenantIds.size() > 0){
                for (SupplierTenantVO voI : tenantIds) {
                    tvo = new SupplierTenantVO();
                    tvo.setSupplierId(vo.getId());
                    tvo.setTenantId(voI.getTenantId());
                    psiSupplierMapper.insertPsiSupplierTenantRef(tvo);
                }
            }
        }
        return result;
    }

    @Override
    public int updateSupplierStatus(PsiSupplierVO vo)
    {
        vo.setUpdateTime(DateUtils.getNowDate());
        // 当前登录人
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        vo.setUpdateBy(userId.toString());
        int result = psiSupplierMapper.updatePsiSupplier(vo);
        if(result>0){//修改成功后修改用户表，主要修改用户表状态
            SysUser sysUser = new SysUser();
            sysUser.setTenantId(vo.getId());
            sysUser.setNickName(vo.getContactName());
            sysUser.setUserName(vo.getContactMobile());
            if(vo.getStatus()==0){
                sysUser.setStatus("1");
            }else{
                sysUser.setStatus("0");
            }
            userMapper.resetTenantUserPwd(sysUser);
        }
        return result;
    }

    /**
     * 批量删除供应商
     * 
     * @param ids 需要删除的供应商主键
     * @return 结果
     */
    @Override
    public int deletePsiSupplierByIds(Long[] ids)
    {
        return psiSupplierMapper.deletePsiSupplierByIds(ids);
    }

    /**
     * 删除供应商信息
     * 
     * @param id 供应商主键
     * @return 结果
     */
    @Override
    public int deletePsiSupplierById(Long id)
    {
        return psiSupplierMapper.deletePsiSupplierById(id);
    }

    /**
     * 批量导入供应商信息
     *
     * @param psiSupplierList
     * @param
     * @return
     */
    @Override
    @Transactional
    public String importPsiSupplier(List<PsiSupplier> psiSupplierList)  {
        if (StringUtils.isNull(psiSupplierList) || psiSupplierList.size() == 0)
        {
            throw new ServiceException("导入供应商数据不能为空！");
        }
        // 获取当前登录用户
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        //成功数
        int successNum = 0;
        //失败数
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        for (PsiSupplier psiSupplier : psiSupplierList)
        {
            try
            {
                // 查询该供应商信息
                PsiSupplier p = psiSupplierMapper.selectByName(psiSupplier.getSupplierName());
                if(StringUtils.isNull(p)){
                    // 不存在新增
                    BeanValidators.validateWithException(validator, psiSupplier);
                    // psiSupplier.setId(tenantId);
                    psiSupplier.setCreateTime(new Date());
                    psiSupplier.setUpdateTime(new Date());
                    psiSupplier.setStatus(1);
                    psiSupplier.setCreateBy(userId.toString());
                    // psiSupplier.setCreateBy(tenantId);
                    // psiSupplier.setUpdateBy(tenantId);
                    PsiSupplierVO vo = new PsiSupplierVO();
                    BeanUtils.copyProperties(psiSupplier,vo);
                    psiSupplierMapper.insertPsiSupplier(vo);
                    // TODO 生成供应商账号密码
                    // SysUser user = new SysUser();
                    // // 登录名为手机号
                    // user.setUserName(vo.getContactMobile());
                    // // 默认密码123456
                    // user.setPassword(SecurityUtils.encryptPassword("123456"));
                    // userService.insertUser(user);
                    SysUser user = new SysUser();
                    //user.setDeptId(dept.getDeptId());
                    user.setUserName(vo.getContactMobile());
                    user.setNickName(vo.getContactName());
                    user.setUserType("02");
                    user.setEmail(vo.getEmail());
                    user.setPhonenumber(vo.getContactMobile());
                    user.setPassword("123456");
                    user.setStatus("0");
                    //user.setPostIds(new Long[]{post.getPostId()});
                    // 新创建的租户管理员，默认给 6 供应商管理员
                    user.setRoleIds(new Long[]{6l});
                    user.setRemark("");
                    user.setIsAdmin("0");//租户创建用户默认为超级管理员
                    //user.setTenantId(sysTenant.getId());
                    user.setCreateBy(SecurityUtils.getLoginUser().getUsername());
                    user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
                    userMapper.insertUser(user);

                    // 6. sys_user_role 1条数据
                    SysUserRole userRole = new SysUserRole();
                    userRole.setUserId(user.getUserId());
                    // 新创建的租户管理员，默认给 6 供应商管理员
                    userRole.setRoleId(6l);
                    userRoleMapper.insertUserRole(userRole);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、名称 " + vo.getSupplierName() + " 插入成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、名称 " + psiSupplier.getSupplierName() + " 名称重复,导入失败");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、名称 " + psiSupplier.getSupplierName() + " 导入失败";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 根据登录租户 获得供应商
     * @param tenantId
     * @return
     */
    @Override
    public List<PsiSupplier> getSupplierTenant(Long tenantId) {
        if(Objects.isNull(tenantId)){
            log.error("===========获得供应商时未获取到当前租户ID===========");
            throw new ServiceException("未获取到当前租户");
        }
        return psiSupplierMapper.getSupplierTenant(tenantId);
    }

    /**
     * 根据名称查询供应商id
     * @param supplierName
     * @return
     */
    @Override
    public Long getSupplierByName(String supplierName) {
        return psiSupplierMapper.getSupplierByName(supplierName);
    }


    @Override
    public List<SupplierTenantVO> getTenantByPhone(String phoneNumber) {
        return psiSupplierMapper.getTenantByPhone(phoneNumber);
    }
}
