package com.ruoyi.psi.service.impl;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.*;
import com.ruoyi.psi.domain.vo.CalculateGoodsReq;
import com.ruoyi.psi.domain.vo.PsiPurchaseVo;
import com.ruoyi.psi.domain.vo.SupplyVO;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.service.IPsiPurchaseService;
import com.ruoyi.psi.service.IPsiSupplyService;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 供货单Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-14
 */
@Service
public class PsiSupplyServiceImpl implements IPsiSupplyService 
{
    @Autowired
    private PsiSupplyMapper psiSupplyMapper;

    @Autowired
    private PsiSupplyItemMapper psiSupplyItemMapper;

    @Autowired
    private PsiPurchaseItemMapper itemMapper;

    @Autowired
    private PsiGoodsMapper goodsMapper;

    @Autowired
    private IPsiPurchaseService purchaseService;

    @Autowired
    private SysTenantMapper tenantMapper;

    @Autowired
    private PsiSupplierMapper supplierMapper;

    private static final Logger log = LoggerFactory.getLogger(PsiSupplyServiceImpl.class);
    /**
     * 查询供货单
     * 
     * @param id 供货单主键
     * @return 供货单
     */
    @Override
    public SupplyVO selectPsiSupplyById(Long id,Long tenantId)
    {
        PsiSupply supply = psiSupplyMapper.selectPsiSupplyById(id, tenantId);
        SupplyVO vo = new SupplyVO();
        BeanUtils.copyProperties(supply,vo);
        List<PsiSupplyItem> items = psiSupplyMapper.selectSupplyItem(id);
        // List<PsiSupplyItem> items = psiSupplyMapper.selectNoStoragySupplyItem(id);
        vo.setSupplyItems(items);
        return vo;
    }

    public SupplyVO selectPsiSupplyByIdTwo(Long id,Long tenantId)
    {
        PsiSupply supply = psiSupplyMapper.selectPsiSupplyById(id, tenantId);
        if(supply == null){
            throw new ServiceException("未查询到该收货单!");
        }
        SupplyVO vo = new SupplyVO();
        BeanUtils.copyProperties(supply,vo);
        // List<PsiSupplyItem> items = psiSupplyMapper.selectSupplyItem(id);
        List<PsiSupplyItem> items = psiSupplyMapper.selectNoStoragySupplyItem(id);
        vo.setSupplyItems(items);
        return vo;
    }

    public List<PsiSupplyItem> calculateGoods(CalculateGoodsReq calVo, Long tenantId){

        List<PsiSupplyItem> items = psiSupplyMapper.selectNoStoragySupplyItem(calVo.getId());
        // 指定金额
        BigDecimal targetAmount = calVo.getTargetAmount();

        // 总金额
        BigDecimal totalAmount = BigDecimal.ZERO;
        for (PsiSupplyItem item : items) {
            // 执行价*实际收货数量-已收货数量
            BigDecimal thisQuantity = item.getActualQuantity().subtract(item.getStorageQuantity());
            item.setThisQuantity(thisQuantity);
            
            BigDecimal itemAmount = item.getPriceInput().multiply(thisQuantity);
            totalAmount = totalAmount.add(itemAmount);
        }
        System.out.println("总金额：" + totalAmount);
        if(targetAmount.compareTo(totalAmount) == 0){
            throw new ServiceException("指定金额大于未收货总金额");
        }

        List<PsiSupplyItem> selectedItems = null;
        BigDecimal sAmount = BigDecimal.ZERO;
        // 先正向扫一遍，看下是否有符合指定金额的数据
        selectedItems = getSelectedItems(false, items, targetAmount);
        for (PsiSupplyItem item : selectedItems) {
            BigDecimal lineAmount = item.getThisQuantity().multiply(item.getPriceInput());
            sAmount = sAmount.add(lineAmount);
        }
        // 没有找到，再按比例减少数量
        if(sAmount.compareTo(targetAmount) != 0){
            // 比例
            int quantityMultiplier = totalAmount.divide(targetAmount, 0, RoundingMode.DOWN).intValue();
            if(quantityMultiplier == 1){
                selectedItems = getSelectedItems(true, items, targetAmount);
            } else {
                selectedItems = calculateSelectedItems(items, targetAmount, quantityMultiplier);
            }
        }

        sAmount = BigDecimal.ZERO;
        for (PsiSupplyItem item : selectedItems) {
            System.out.println(item);
            BigDecimal lineAmount = item.getThisQuantity().multiply(item.getPriceInput());
            sAmount = sAmount.add(lineAmount);
        }
        System.out.println("选中商品总金额：" + sAmount);
        
        return selectedItems;
    }

    private List<PsiSupplyItem> getSelectedItems(boolean isNeedSort, List<PsiSupplyItem> itemList, BigDecimal targetAmount){
        if(isNeedSort) {
            // 按照价格降序排序
            Collections.sort(itemList, new Comparator<PsiSupplyItem>() {
                @Override
                public int compare(PsiSupplyItem o1, PsiSupplyItem o2) {
                    return o2.getPriceInput().compareTo(o1.getPriceInput());
                }
            });
        }

        List<PsiSupplyItem> selectedGoods = new ArrayList<>();
        BigDecimal remainingAmount = targetAmount;

        for (PsiSupplyItem item : itemList) {
            BigDecimal goodsAmount = item.getThisQuantity().multiply(item.getPriceInput()); // 计算商品金额

            if (goodsAmount.compareTo(remainingAmount) <= 0) {
                selectedGoods.add(item);
                remainingAmount = remainingAmount.subtract(goodsAmount);
            }
        }

        return selectedGoods;
    }
    private List<PsiSupplyItem> calculateSelectedItems(List<PsiSupplyItem> itemList, BigDecimal targetAmount, int quantityMultiplier) {
        BigDecimal adjustedTotalAmount = targetAmount.multiply(new BigDecimal(quantityMultiplier));

        List<PsiSupplyItem> selectedItems = new ArrayList<>();
        for (PsiSupplyItem item : itemList) {
            BigDecimal itemAmount = item.getPriceInput().add(adjustedTotalAmount);
            BigDecimal originalQuantity = item.getThisQuantity();

            BigDecimal adjustedQuantity = originalQuantity.divide(new BigDecimal(quantityMultiplier), 0, RoundingMode.DOWN);
            item.setThisQuantity(adjustedQuantity);

            while (itemAmount.compareTo(targetAmount) <= 0 && item.getThisQuantity().compareTo(originalQuantity) <= 0) {
                item.setThisQuantity(item.getThisQuantity().add(BigDecimal.ONE));
                itemAmount = item.getPriceInput().multiply(item.getThisQuantity()).add(adjustedTotalAmount);
            }

            if (item.getThisQuantity().compareTo(originalQuantity) > 0) {
                item.setThisQuantity(originalQuantity);
            }

            if(item.getThisQuantity().compareTo(BigDecimal.ZERO) != 0){
                selectedItems.add(item);
            }
        }

        return selectedItems;
    }

    /**
     * 查询供货单列表
     * 
     * @param psiSupply 供货单
     * @return 供货单
     */
    @Override
    public List<PsiSupply> selectPsiSupplyList(PsiSupply psiSupply)
    {
        return psiSupplyMapper.selectPsiSupplyList(psiSupply);
    }

    /**
     * 新增供货单
     * 
     * @param psiSupply 供货单
     * @return 结果
     */
    @Override
    public int insertPsiSupply(PsiSupply psiSupply)
    {
        psiSupply.setCreateTime(DateUtils.getNowDate());
        return psiSupplyMapper.insertPsiSupply(psiSupply);
    }

    /**
     * 修改供货单
     * 
     * @param psiSupply 供货单
     * @return 结果
     */
    @Override
    public int updatePsiSupply(PsiSupply psiSupply)
    {
        return psiSupplyMapper.updatePsiSupply(psiSupply);
    }

    /**
     * 批量删除供货单
     * 
     * @param ids 需要删除的供货单主键
     * @return 结果
     */
    @Override
    public int deletePsiSupplyByIds(Long[] ids)
    {
        return psiSupplyMapper.deletePsiSupplyByIds(ids);
    }

    /**
     * 删除供货单信息
     * 
     * @param id 供货单主键
     * @return 结果
     */
    @Override
    public int deletePsiSupplyById(Long id)
    {
        return psiSupplyMapper.deletePsiSupplyById(id);
    }

    /**
     *
     * 根据采购单id获得供货单
     * @param purchaseId
     * @param tenantId
     * @return
     */
    @Override
    public SupplyVO getSupplyByPurchaseId(Long purchaseId, Long tenantId,Long supplierId) {
        // 调用采购单接口获得采购单信息
        PsiPurchaseVo psiPurchaseVo = purchaseService.selectPsiPurchaseById(purchaseId, tenantId);
        List<PsiPurchaseItem> purchaseItems = psiPurchaseVo.getItems();
        if(Objects.isNull(purchaseItems)){
            log.error("==============根据采购单id未查询到该条信息=============={}",purchaseId);
            throw new ServiceException("未查询到该采购单!");
        }
        List<PsiSupplyItem> supplyItems = new ArrayList<>();
        StringBuilder sb = new StringBuilder();
        // 根据商品名称以及租户id查询对应的商品信息封装
        if(!CollectionUtils.isEmpty(purchaseItems)){
            PsiGoods goods = null;
            PsiSupplyItem supplyItem = null;
            for (PsiPurchaseItem purchaseItem : purchaseItems) {
                if(Objects.isNull(purchaseItem)){
                    continue;
                }
                goods = new PsiGoods();
                BeanUtils.copyProperties(purchaseItem,goods);
                goods.setTenantId(tenantId);
                goods.setSupplierId(supplierId);
                String spec = purchaseItem.getSpec() == null ? "" : purchaseItem.getSpec().toLowerCase();
                goods.setSpec(spec);
                BigDecimal quantity = purchaseItem.getQuantity();
                String remark = purchaseItem.getRemark();
                PsiGoods psiGoods = goodsMapper.selectOneGoods(goods);
                if(Objects.isNull(psiGoods)){
                    sb.append(goods.getGoodsName() + " " + goods.getSpec() + "; ");
                    continue;
                }
                supplyItem = new PsiSupplyItem();
                BeanUtils.copyProperties(psiGoods,supplyItem,"id");
                supplyItem.setGoodsId(psiGoods.getId());
                supplyItem.setQuantity(quantity);
                supplyItem.setPriceInput(psiGoods.getPrice());
                supplyItem.setRemark(remark);
                supplyItems.add(supplyItem);
            }
        }
        if(purchaseItems.size() != supplyItems.size()){
            log.error("================供应商不能提供完整商品信息================");
            throw new ServiceException("该供应商不能提供如下商品：" + sb.toString() + "。请检查商品名称、规格是否和商品库中一致，特别注意是否有空格、.等多余字符。");
        }
        SupplyVO supplyVO = new SupplyVO();
        supplyVO.setStoreId(psiPurchaseVo.getStoreId());
        supplyVO.setStoreName(psiPurchaseVo.getStoreName());
        supplyVO.setSupplierId(supplierId);
        supplyVO.setPurchaseId(purchaseId);
        supplyVO.setTenantId(tenantId);
        supplyVO.setSupplyItems(supplyItems);
        supplyVO.setRemark(psiPurchaseVo.getRemark());
        supplyVO.setQuotationMode(psiPurchaseVo.getQuotationMode());
        //**********2023/12/26新增**********
        SysUser user = SecurityUtils.getLoginUser().getUser();
        supplyVO.setTenantName(psiPurchaseVo.getTenantName());
        supplyVO.setCreateUser(user.getUserId());
        supplyVO.setCreateUsername(user.getNickName());
        supplyVO.setCreateTime(DateUtils.getNowDate());
        PsiSupplier psiSupplier = supplierMapper.selectPsiSupplierById(supplierId);
        supplyVO.setSupplierName(psiSupplier.getSupplierName());
        supplyVO.setStoreId(psiPurchaseVo.getStoreId());
        supplyVO.setStoreName(psiPurchaseVo.getStoreName());
        return supplyVO;
    }

    /**
     * 新增供货单
     *
     * @param supplyVO
     * @return
     */
    @Override
    @Transactional
    public int addSupply(SupplyVO supplyVO) {
        if(Objects.isNull(supplyVO)){
            log.error("==================采购单为空==================");
            throw new ServiceException("采购单不能为空");
        }
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        String userName = SecurityUtils.getLoginUser().getUser().getNickName();
        Long tenantId = SecurityUtils.getLoginUser().getUser().getTenantId();
        // 查询租户名称
        SysTenant tenant = tenantMapper.selectSysTenantById(tenantId);
        String tenantName = tenant.getTenantName();
        Date nowDate = DateUtils.getNowDate();
        supplyVO.setCreateUser(userId);
        supplyVO.setCreateUsername(userName);
        supplyVO.setCreateTime(nowDate);
        supplyVO.setTenantName(tenantName);
        PsiSupply psiSupply = new PsiSupply();
        BeanUtils.copyProperties(supplyVO,psiSupply);
        List<PsiSupplyItem> supplyItems = supplyVO.getSupplyItems();
        BigDecimal totalAmount = new BigDecimal("0.00");
        if(!CollectionUtils.isEmpty(supplyItems)){
            for (PsiSupplyItem supplyItem : supplyItems) {
                // 数量
                BigDecimal quantity = supplyItem.getQuantity();
                // 将数量四舍五入到两位小数
                quantity = quantity.setScale(2, RoundingMode.HALF_UP);
                // 实际收货数量
                supplyItem.setActualQuantity(quantity);
                // 执行单价
                BigDecimal priceInput = supplyItem.getPriceInput();
                // 将单价四舍五入到两位小数
                priceInput = priceInput.setScale(2, RoundingMode.HALF_UP);
                // 金额
                BigDecimal amount = quantity.multiply(priceInput).setScale(2,RoundingMode.HALF_UP);
                supplyItem.setAmount(amount);
                totalAmount = totalAmount.add(amount);
            }
        }
        psiSupply.setTotalAmount(totalAmount);
        // 默认未收货
        psiSupply.setStatus(0);
        // 采购录入
        psiSupply.setDeliveryMethod(1);
        // 修改采购单状态为3：已订货
        PsiPurchaseVo psiPurchase = new PsiPurchaseVo();
        psiPurchase.setId(psiSupply.getPurchaseId());
        psiPurchase.setStatus(3);
        psiPurchase.setTenantId(tenantId);
        try {
            psiSupplyMapper.insertPsiSupply(psiSupply);
            // 批量插入item
            Long supplyId = psiSupply.getId();
            psiSupplyMapper.insertSupplyItem(supplyItems,supplyId);

            purchaseService.updatePsiPurchaseStatus(psiPurchase);
            return 1;
        } catch (Exception e) {
            log.error("====生成订货单失败=====",e);
            throw new ServiceException("生成订货单失败");
        }
    }

    @Override
    @Transactional
    public int revokeSupply(Long id){
        Long tenantId = SecurityUtils.getLoginUser().getUser().getTenantId();
        //检查是否已有订货单
        PsiSupply psiSupply = psiSupplyMapper.selectPsiSupplyById(id, tenantId);
        if(psiSupply.getStatus() > 0){
            throw new ServiceException("已收货，请先撤回收货单");
        }
        //删除订货单主表和明细表
        psiSupplyMapper.deletePsiSupplyById(id);
        psiSupplyItemMapper.deletePsiSupplyItemBySupplyId(id);
        //修改采购单状态
        // 修改采购单状态为3：已订货
        PsiPurchaseVo psiPurchase = new PsiPurchaseVo();
        psiPurchase.setId(psiSupply.getPurchaseId());
        psiPurchase.setStatus(0);
        psiPurchase.setTenantId(tenantId);
        purchaseService.updatePsiPurchaseStatus(psiPurchase);
        return 1;
    }
}
