package com.ruoyi.sup.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.psi.domain.PsiSupply;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.sup.domain.SupSupply;
import com.ruoyi.sup.service.ISupSupplyService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 供应商角色接口
 * 供应商获取自己的供货单，报价单
 */
@RestController
@RequestMapping("/sup/supply")
public class SupSupplyController extends BaseController
{
    @Autowired
    private ISupSupplyService supplyService;

    /**
     * 查询供货单列表
     */
    @PreAuthorize("@ss.hasPermi('sup:supply:list')")
    @GetMapping("/list")
    public TableDataInfo list(SupSupply psiSupply)
    {
        // TenantId  为供应商id
        psiSupply.setSupplierId(this.getTenantId());
        startPage();
        List<SupSupply> list = supplyService.selectSupSupplyList(psiSupply);
        return getDataTable(list);
    }

    /**
     * 获取供货单详细信息
     */
    // @PreAuthorize("@ss.hasPermi('sup:supply:query')")
    // @GetMapping(value = "/{id}")
    // public AjaxResult getInfo(@PathVariable("id") Long id)
    // {
    //     return success(supplyService.selectSupSupplyById(id));
    // }


    /**
     * 导出供货单列表
     */
    @PreAuthorize("@ss.hasPermi('sup:supply:export')")
    @Log(title = "供货单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SupSupply supSupply)
    {
        // 获取当前登录用户所属ID
        supSupply.setSupplierId(this.getTenantId());
        List<SupSupply> list = supplyService.selectSupSupplyList(supSupply);
        ExcelUtil<SupSupply> util = new ExcelUtil<SupSupply>(SupSupply.class);
        util.exportExcel(response, list, "供货单数据");
    }

    /**
     * 获取供货单详细信息
     */
    @PreAuthorize("@ss.hasPermi('sup:supply:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        // 获取当前登录用户所属ID
        Long supplierId = this.getTenantId();
        return success(supplyService.selectSupSupplyById(id,supplierId));
    }


}
