package com.ruoyi.sup.service.impl;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.PsiGoods;
import com.ruoyi.psi.domain.PsiPurchase;
import com.ruoyi.psi.domain.PsiPurchaseItem;
import com.ruoyi.psi.domain.PsiSupplyItem;
import com.ruoyi.psi.domain.vo.SupplyVO;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.service.IPsiSupplyService;
import com.ruoyi.sup.domain.PsiBidding;
import com.ruoyi.sup.domain.PsiBiddingItem;
import com.ruoyi.sup.domain.vo.BiddingVO;
import com.ruoyi.sup.mapper.PsiBiddingItemMapper;
import com.ruoyi.sup.mapper.PsiBiddingMapper;
import com.ruoyi.sup.service.IPsiBiddingService;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 竞价单Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-17
 */
@Service
public class PsiBiddingServiceImpl implements IPsiBiddingService 
{
    @Autowired
    private PsiBiddingMapper psiBiddingMapper;

    @Autowired
    private PsiPurchaseItemMapper purchaseItemMapper;

    @Autowired
    private PsiGoodsMapper goodsMapper;

    @Autowired
    private PsiSupplierMapper supplierMapper;

    @Autowired
    private PsiBiddingItemMapper biddingItemMapper;

    @Autowired
    private IPsiSupplyService supplyService;

    @Autowired
    private PsiSupplyMapper supplyMapper;

    @Autowired
    private SysTenantMapper tenantMapper;

    @Autowired
    private PsiPurchaseMapper psiPurchaseMapper;




    private static final Logger log = LoggerFactory.getLogger(PsiBiddingServiceImpl.class);


    /**
     * 查询竞价单
     * 
     * @param id 竞价单主键
     * @return 竞价单
     */
    @Override
    public PsiBidding selectPsiBiddingById(Long id)
    {
        return psiBiddingMapper.selectPsiBiddingById(id);
    }

    /**
     * 查询竞价单列表
     * 
     * @param psiBidding 竞价单
     * @return 竞价单
     */
    @Override
    public List<PsiBidding> selectPsiBiddingList(PsiBidding psiBidding)
    {
        return psiBiddingMapper.selectPsiBiddingList(psiBidding);
    }

    /**
     * 新增竞价单
     * 
     * @param psiBidding 竞价单
     * @return 结果
     */
    @Override
    public int insertPsiBidding(PsiBidding psiBidding)
    {
        psiBidding.setCreateTime(DateUtils.getNowDate());
        return psiBiddingMapper.insertPsiBidding(psiBidding);
    }

    /**
     * 修改竞价单
     * 
     * @param psiBidding 竞价单
     * @return 结果
     */
    @Override
    public int updatePsiBidding(PsiBidding psiBidding)
    {
        return psiBiddingMapper.updatePsiBidding(psiBidding);
    }

    /**
     * 批量删除竞价单
     * 
     * @param ids 需要删除的竞价单主键
     * @return 结果
     */
    @Override
    public int deletePsiBiddingByIds(Long[] ids)
    {
        return psiBiddingMapper.deletePsiBiddingByIds(ids);
    }

    /**
     * 删除竞价单信息
     * 
     * @param id 竞价单主键
     * @return 结果
     */
    @Override
    public int deletePsiBiddingById(Long id)
    {
        return psiBiddingMapper.deletePsiBiddingById(id);
    }

    /**
     * @description 根据采购单id生成竞价单
     *
     * @param purchaseId
     * @param tenantId
     * @return int
     * @author fm
     * @date 2023/12/17 10:12
     */
    @Override
    @Transactional
    public int addBidding(Long purchaseId, Long tenantId) {
        // 获得采购单详情
        List<PsiPurchaseItem> purchaseItems = purchaseItemMapper.selectPsiPurchaseItemByPurchaseId(purchaseId);
        if(CollectionUtils.isEmpty(purchaseItems)){
            throw new ServiceException("采购单不能为空!");
        }

        // 商品数量
        int goodsNum = purchaseItems.size();
        // 创建一个二维数组，存放明细中每个商品对应的商品列表
        // 二维数组 行：采购单明细索引   列：商品列表
        PsiGoods[][] goods_arr = new PsiGoods[goodsNum][];
        PsiGoods goods = null;
        // 遍历采购单，并根据商品名称+规格+租户 找出满足条件的商品列表，可能有多个供应商都提供 可乐+260ml，所以是多个
        for (int i = 0; i < purchaseItems.size(); i++) {
            goods = new PsiGoods();
            goods.setGoodsName(purchaseItems.get(i).getGoodsName());
            goods.setSpec(purchaseItems.get(i).getSpec());
            goods.setTenantId(tenantId);
            // 当前租户 + 规格 + 商品名称 获得数据
            List<PsiGoods> psiGoodsList = goodsMapper.selectGoodsByPurchase(goods);
            if(CollectionUtils.isEmpty(psiGoodsList)){
                throw new ServiceException(purchaseItems.get(i).getGoodsName() + "未找到此商品!");
            }
            goods_arr[i] = psiGoodsList.toArray(new PsiGoods[0]);
        }

        // 获取供应商交集，即可以提供采购单中全部商品的供应商ID
        // 首先创建一个HashSet来存储交集元素,将第一个数组goods_arr[0]的元素添加到集合中
        Set<Long> intersection = new HashSet<>();
        for (int i = 0; i < goods_arr[0].length; i++) {
            intersection.add(goods_arr[0][i].getSupplierId());
        }
        // 然后，遍历剩余的数组，并使用retainAll方法计算交集，这个方法会保留集合中与当前数组元素相同的元素。
        for (int i = 1; i < goods_arr.length; i++) {
            Set<Long> currentSet = new HashSet<>();
            for (int j = 0; j < goods_arr[i].length; j++) {
                currentSet.add(goods_arr[i][j].getSupplierId());
            }
            intersection.retainAll(currentSet);
        }
        log.debug("竞价模式，匹配条件的供应商ID：" + intersection);
        if(CollectionUtils.isEmpty(intersection)){
            throw new ServiceException("未找到符合可以报价的供应商，没有1个供应商可以提供所有商品!");
        }

        // 生成竞价单
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        String userName = SecurityUtils.getLoginUser().getUser().getNickName();
        SysTenant sysTenant = tenantMapper.selectSysTenantById(tenantId);
        String tenantName = sysTenant.getTenantName();
        // 为每个符合条件的供应商生产报价单
        for (Long sId : intersection) {
            PsiBidding bidding = new PsiBidding();
            bidding.setCreateUser(userId);
            bidding.setCreateTime(DateUtils.getNowDate());
            bidding.setSupplierId(sId);
            bidding.setTenantId(tenantId);
            bidding.setPurchaseId(purchaseId);
            bidding.setStatus(0);
            bidding.setCreateUsername(userName);
            bidding.setTenantName(tenantName);
            psiBiddingMapper.insertPsiBidding(bidding);

            Long biddingId = bidding.getId();

            PsiBiddingItem biddingItem = null;
            List<PsiBiddingItem> biddingItems = new ArrayList<>();
            for (int i = 0; i < purchaseItems.size(); i++) {
                PsiPurchaseItem psiPurchaseItem = purchaseItems.get(i);
                biddingItem = new PsiBiddingItem();
                BeanUtils.copyProperties(psiPurchaseItem,biddingItem);
                biddingItem.setBiddingId(biddingId);

                PsiGoods psiGoods = goodsMapper.selectOneGoodsByNameAndSpec(biddingItem.getGoodsName(), biddingItem.getSpec().toLowerCase(), sId, tenantId);
                biddingItem.setGoodsId(psiGoods.getId());
                biddingItem.setPrice(psiGoods.getPrice());
                biddingItems.add(biddingItem);
            }
            // 插入详细表
            psiBiddingMapper.savaBathList(biddingItems,biddingId);
        }
        return 1;
    }
    /**
     * @description 根据采购单id生成竞价单
     *
     * @param purchaseId
     * @param tenantId
     * @return int
     * @author fm
     * @date 2023/12/17 10:12
     */
//    @Override
    @Transactional
    public int addBidding_bak(Long purchaseId, Long tenantId) {
        // 获得采购单详情
        List<PsiPurchaseItem> purchaseItems = purchaseItemMapper.selectPsiPurchaseItemByPurchaseId(purchaseId);
        if(CollectionUtils.isEmpty(purchaseItems)){
            throw new ServiceException("采购单不能为空!");
        }
        // 供应商 - 商品
        Map<Long,List<PsiGoods>> supplierGoods = new HashMap<>();
        PsiGoods goods = null;
        for (PsiPurchaseItem purchaseItem : purchaseItems) {
            if(Objects.isNull(purchaseItem)){
                continue;
            }
            goods = new PsiGoods();
            BeanUtils.copyProperties(purchaseItem,goods);
            goods.setTenantId(tenantId);
            // 当前租户 + 规格 + 商品名称 获得数据
            List<PsiGoods> psiGoodsList = goodsMapper.selectGoodsByPurchase(goods);
            for (PsiGoods psiGoods : psiGoodsList) {
                Long supplierId = psiGoods.getSupplierId();
                List<PsiGoods> goods1 = supplierGoods.get(supplierId);
                if(CollectionUtils.isEmpty(goods1)){
                    goods1 = new ArrayList<>();
                    goods1.add(psiGoods);
                    supplierGoods.put(supplierId,goods1);
                }else {
                    goods1.add(psiGoods);
                }
            }
        }
        // 符合的供应商
        List<Long> supplierList = new ArrayList<>();
        // 过滤出采购单商品的名称
        List<String> purchaseGoods = purchaseItems.stream().map(PsiPurchaseItem::getGoodsName).collect(Collectors.toList());
        // 找到改采购单下所有符合的供应商
        Set<Long> supplierIds = supplierGoods.keySet();
        for (Long supplierId : supplierIds) {
            List<PsiGoods> goodsList = supplierGoods.get(supplierId);
            List<String> collect = goodsList.stream().map(PsiGoods::getGoodsName).collect(Collectors.toList());
            Collection<String> intersection = CollectionUtils.intersection(purchaseGoods, collect);
            if(null != intersection && intersection.size() >= purchaseGoods.size() ){
                // 有交集，符合的供应商  将该供应商添加到集合
                supplierList.add(supplierId);
            }
        }
        if(CollectionUtils.isEmpty(supplierList)){
            throw new ServiceException("未找到符合的供应商!");
        }
        // 根据供应商id 插入数据
        List<PsiBiddingItem> biddingItems = null;
        for (Long sId : supplierList) {
            PsiBidding bidding = new PsiBidding();
            Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
            bidding.setCreateUser(userId);
            bidding.setCreateTime(DateUtils.getNowDate());
            bidding.setSupplierId(sId);
            bidding.setTenantId(tenantId);
            bidding.setPurchaseId(purchaseId);
            psiBiddingMapper.insertPsiBidding(bidding);
            Long biddingId = bidding.getId();
            PsiGoods biddingGoods = null;
            PsiBiddingItem biddingItem = null;
            biddingItems = new ArrayList<>();
            for (PsiPurchaseItem purchaseItem : purchaseItems) {
                if(Objects.isNull(purchaseItem)){
                    continue;
                }
                biddingGoods = new PsiGoods();
                BeanUtils.copyProperties(purchaseItem,biddingGoods);
                biddingGoods.setTenantId(tenantId);
                biddingGoods.setSupplierId(sId);
                BigDecimal quantity = purchaseItem.getQuantity();
                String remark = purchaseItem.getRemark();
                PsiGoods psiGoods = goodsMapper.selectOneGoods(biddingGoods);
                if(!Objects.isNull(psiGoods)){
                    biddingItem = new PsiBiddingItem();
                    BeanUtils.copyProperties(psiGoods,biddingItem,"id");
                    biddingItem.setGoodsId(psiGoods.getId());
                    biddingItem.setQuantity(quantity);
                    biddingItem.setRemark(remark);
                    biddingItems.add(biddingItem);
                }
            }
            // 插入详细表
            psiBiddingMapper.savaBathList(biddingItems,biddingId);
        }
        return 1;
    }

    /**
     * 供应商报价
     *
     * @param vo
     * @return
     */
    @Override
    @Transactional
    public int biddingQuote(BiddingVO vo) {
        if(Objects.isNull(vo)){
            log.error("==================竞价单为空==================");
            throw new ServiceException("竞价单不能为空");
        }
        List<PsiBiddingItem> biddingItems = vo.getItems();
        // 总额
        BigDecimal totalAmount = BigDecimal.ZERO;
        if(!CollectionUtils.isEmpty(biddingItems)){
            BigDecimal quantity = null;
            BigDecimal priceInput = null;
            BigDecimal amount = null;
            for (PsiBiddingItem biddingItem : biddingItems) {
                // 数量
                quantity = biddingItem.getQuantity();
                // 执行单价
                priceInput = biddingItem.getPriceInput();
                // 金额
                if(null != quantity && null != priceInput){
                    amount = quantity.multiply(priceInput);
                    biddingItem.setAmount(amount);
                    totalAmount = totalAmount.add(amount);
                }
            }
        }
        vo.setTotalAmount(totalAmount);
        try {
            // 更新数据
            int flagOne = psiBiddingMapper.updatePsiBidding(vo);
            // 批量更新item
            int flagTwo = biddingItemMapper.updatePsiBiddingItemByList(biddingItems);
            return flagOne == 1 && flagTwo == 1 ? 1 : 0;
        }catch (Exception e){
            log.error("更新竞价单或竞价项时发生错误", e);
            throw new ServiceException("更新竞价单或竞价项时发生错误");
        }
    }


    /**
     * @description 根据id 查看当前竞价单
     *
     * @param id
     * @return com.ruoyi.sup.domain.vo.BiddingVO
     * @author fm
     * @date 2023/12/18 11:30
     */
    @Override
    public BiddingVO selectBiddingDetailById(Long id, Long supplierId) {
        BiddingVO vo = new BiddingVO();
        PsiBidding bidding = psiBiddingMapper.selectBiddingByIdAndTenantId(id, supplierId);
        if(Objects.isNull(bidding)){
            throw new ServiceException("未查询到竞价单!");
        }
        BeanUtils.copyProperties(bidding,vo);
        // 封装item
        List<PsiBiddingItem> psiBiddingItems = biddingItemMapper.selectPsiBiddingItemById(id);
        vo.setItems(psiBiddingItems);
        return vo;
    }


    /**
     * 竞价单提交(供应商提交)    找出最低价格的生成供货单
     * @param biddingId
     * @return
     */
    @Override
    @Transactional
    public int edit(Long biddingId) {
        if(Objects.isNull(biddingId)){
            log.error("==================报价单为空==================");
            throw new ServiceException("报价单不能为空");
        }
        // 根据id查询详情
//        List<PsiBiddingItem> items = biddingItemMapper.selectPsiBiddingItemById(biddingId);
//        long count = items.stream().filter(item -> Objects.isNull(item.getPriceInput())).count();
//        if(count > 1){
//            // 有商品未添价格
//            log.error("==================有商品未添价格==================");
//            throw new ServiceException("请检查所有商品已经填写价格后提交!");
//        }
        PsiBidding psiBidding = psiBiddingMapper.selectPsiBiddingById(biddingId);
        if(null == psiBidding.getTotalAmount() || psiBidding.getTotalAmount().compareTo(BigDecimal.ZERO) < 1){
            // 有商品未添价格
            log.error("==================有商品未添价格==================");
            throw new ServiceException("请检查所有商品已经填写价格后提交!");
        }
        // 将订单状态设置为已提交
        psiBidding.setStatus(1);
        // 更新此条数据状态
        psiBiddingMapper.updatePsiBidding(psiBidding);
        Long purchaseId = psiBidding.getPurchaseId();
        List<PsiBidding> biddingList = psiBiddingMapper.selectBiddingByPurchaseId(purchaseId);
        // 未提交的
        long count1 = biddingList.stream().filter(item -> Objects.isNull(item.getStatus()) || item.getStatus().equals(1)).count();
        if(null != biddingList && biddingList.size() != count1){
            // 存在未提交的
            return 1;
        }
        PsiPurchase purchase = psiPurchaseMapper.selectOnePsipurchaseById(purchaseId);
        // 找到最优价格生成供货单
        List<PsiSupplyItem> supplyItems = null;
        if(!CollectionUtils.isEmpty(biddingList)){
            // 找出最低报价的供应商
            Optional<PsiBidding> min = biddingList.stream().min(((o1, o2) -> o1.getTotalAmount().compareTo(o2.getTotalAmount())));
            if(min.isPresent()){
                PsiBidding bidding = min.get();
                SupplyVO supplyVO = new SupplyVO();
                BeanUtils.copyProperties(bidding,supplyVO,"id","status");
                // 插入供货单
                supplyVO.setCreateTime(DateUtils.getNowDate());
                supplyVO.setStatus(0);
                supplyVO.setDeliveryMethod(1);
                supplyVO.setBusinessTime(DateUtils.getNowDate());
                supplyVO.setStoreId(purchase.getStoreId());
                supplyVO.setStoreName(purchase.getStoreName());
                supplyMapper.insertPsiSupply(supplyVO);

                Long supplyVOId = supplyVO.getId();
                PsiBiddingItem biddingItem = new PsiBiddingItem();
                biddingItem.setBiddingId(bidding.getId());

                // 根据竞价单id查询到竞价单详情
                List<PsiBiddingItem> psiBiddingItems = biddingItemMapper.selectPsiBiddingItemList(biddingItem);
                supplyItems = new ArrayList<>();
                for (PsiBiddingItem psiBiddingItem : psiBiddingItems) {
                    PsiSupplyItem supplyItem = new PsiSupplyItem();
                    BeanUtils.copyProperties(psiBiddingItem,supplyItem,"id");
                    supplyItem.setSupplyId(supplyVOId);
                    supplyItems.add(supplyItem);
                }
                supplyVO.setSupplyItems(supplyItems);
                // 批量插入详情
                supplyMapper.insertSupplyItem(supplyItems,supplyVOId);
            }
        }
        purchase.setStatus(3);
        psiPurchaseMapper.updatePsiPurchase(purchase);
        return 1;
    }
}
