package com.ruoyi.system.service.impl;

import java.util.List;

import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.domain.SysUserPost;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.service.ISysTenantService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * 租户Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-12
 */
@Service
public class SysTenantServiceImpl implements ISysTenantService 
{
    @Autowired
    private SysTenantMapper sysTenantMapper;

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    /**
     * 查询租户
     * 
     * @param id 租户主键
     * @return 租户
     */
    @Override
    public SysTenant selectSysTenantById(Long id)
    {
        return sysTenantMapper.selectSysTenantById(id);
    }

    /**
     * 查询租户列表
     * 
     * @param sysTenant 租户
     * @return 租户
     */
    @Override
    public List<SysTenant> selectSysTenantList(SysTenant sysTenant)
    {
        return sysTenantMapper.selectSysTenantList(sysTenant);
    }

    /**
     * 新增租户
     * 
     * @param sysTenant 租户
     * @return 结果
     */
    @Override
    @Transactional
    public int insertSysTenant(SysTenant sysTenant)
    {
        List<SysTenant> ts = sysTenantMapper.selectSysTenantByName(sysTenant.getTenantName());
        if(!CollectionUtils.isEmpty(ts)){
            throw new ServiceException("新增失败，餐厅已存在，请修改后再添加");
        }
        SysUser info = userMapper.checkUserNameUnique(sysTenant.getTenantAccount());
        if (info != null){
            throw new ServiceException("新增失败，该登录账号已使用，请修改后再添加");
        }
        sysTenant.setCreateTime(DateUtils.getNowDate());
        // 保存租户信息
        sysTenantMapper.insertSysTenant(sysTenant);
        Long tenantId = sysTenant.getId();

        // 为该租户生成sys_role、sys_dept、sys_post相关数据，保证该用户可以以商户管理员身份登录后台系统
        // 1. sys_dept 3条数据
        // 店铺根部门
        SysDept dept = new SysDept();
        dept.setDeptName(sysTenant.getTenantName());
        dept.setTenantId(tenantId);
        dept.setParentId(0l);
        dept.setAncestors("0");
        dept.setOrderNum(1);
        deptMapper.insertDept(dept);
        // 店铺下的管理部门
        SysDept managerDept = new SysDept();
        managerDept.setDeptName("管理部门");
        managerDept.setTenantId(tenantId);
        managerDept.setParentId(dept.getDeptId());
        managerDept.setAncestors("0," + dept.getDeptId());
        managerDept.setOrderNum(2);
        deptMapper.insertDept(managerDept);
        // 2. sys_post 2条数据
        SysPost post = new SysPost();
        post.setPostCode("manager");
        post.setPostName("负责人");
        post.setPostSort(1);
        post.setStatus("0");
        post.setTenantId(tenantId);
        postMapper.insertPost(post);
        // 3. sys_role 2条数据
//        SysRole role = new SysRole();
//        role.setRoleName("管理员");
//        role.setRoleKey("admin");
//        role.setTenantId(tenantId);
//        role.setIsAdminRole("1");
//        role.setRoleSort(1);
//        role.setDataScope("1");
//        role.setStatus("0");
//        roleMapper.insertRole(role);
        // 4. sys_user
        SysUser user = new SysUser();
        user.setDeptId(dept.getDeptId());
        user.setUserName(sysTenant.getTenantAccount());
        user.setNickName(sysTenant.getTenantAccount());
        user.setUserType("01");
        user.setEmail("1@qq.com");
        user.setPhonenumber(sysTenant.getContactMobile());
        user.setPassword("123456");
        user.setStatus("0");
        user.setPostIds(new Long[]{post.getPostId()});
        // 新创建的租户管理员，默认给 4 租户管理员角色
        user.setRoleIds(new Long[]{4l});
        user.setRemark("租户新建自动创建的用户");
        user.setIsAdmin("1");//租户创建用户默认为超级管理员
        user.setTenantId(sysTenant.getId());
        user.setCreateBy(SecurityUtils.getLoginUser().getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        userMapper.insertUser(user);

        // 5. sys_user_post 1条数据
        SysUserPost userPost = new SysUserPost();
        userPost.setUserId(user.getUserId());
        userPost.setPostId(post.getPostId());
        userPostMapper.insertUserPost(userPost);
        // 6. sys_user_role 1条数据
        SysUserRole userRole = new SysUserRole();
        userRole.setUserId(user.getUserId());
        // 新创建的租户管理员，默认给 4 租户管理员角色
        userRole.setRoleId(4l);
        userRoleMapper.insertUserRole(userRole);

        return 1;
    }

    /**
     * 修改租户
     * 
     * @param sysTenant 租户
     * @return 结果
     */
    @Override
    public int updateSysTenant(SysTenant sysTenant)
    {
        sysTenant.setUpdateTime(DateUtils.getNowDate());
        int i = sysTenantMapper.updateSysTenant(sysTenant);
        if(i>0){//修改成功后修改用户表，主要修改用户表状态
            SysUser sysUser = new SysUser();
            sysUser.setTenantId(sysTenant.getId());
            sysUser.setNickName(sysTenant.getTenantName());
            sysUser.setUserName(sysTenant.getTenantAccount());
            sysUser.setStatus(sysTenant.getStatus());
            userMapper.resetTenantUserPwd(sysUser);
        }
        return i;
    }

    /**
     * 批量删除租户
     * 
     * @param ids 需要删除的租户主键
     * @return 结果
     */
    @Override
    public int deleteSysTenantByIds(Long[] ids)
    {
        return sysTenantMapper.deleteSysTenantByIds(ids);
    }

    /**
     * 删除租户信息
     * 
     * @param id 租户主键
     * @return 结果
     */
    @Override
    public int deleteSysTenantById(Long id)
    {
        return sysTenantMapper.deleteSysTenantById(id);
    }

    /**
     * 获得所有租户不分页*
     * @param sysTenant
     * @return
     */
    @Override
    public List<SysTenant> selectSysTenantNoPageList(SysTenant sysTenant) {
        return sysTenantMapper.selectSysTenantNoPageList(sysTenant);
    }
}
